/* -------------------------------------------------------------------------*
 * PKUAS: Peking University Application Server
 * Copyright (C) 2006 Peking University
 *
 * This software is free; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * either version 2.1 or any later version.
 * 
 * This software is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this software; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 * ------------------------------------------------------------------------*/

package org.ow2.xplus;

import java.util.Map;

import javax.persistence.EntityManagerFactory;
import javax.persistence.spi.PersistenceProvider;
import javax.persistence.spi.PersistenceUnitInfo;

import org.ow2.xplus.broker.PersistenceManagerFactory;
import org.ow2.xplus.deployer.AbstractDeployer;
import org.ow2.xplus.deployer.ContainerDeployer;
import org.ow2.xplus.deployer.Deployer;
import org.ow2.xplus.deployer.StandaloneDeployer;
import org.ow2.xplus.metadata.MetaDataRepository;
import org.ow2.xplus.metadata.PersistenceInfo;

/**
 * PKUAS EJB3 persistence provider implementation
 * @author Lin Liang
 */

public class XplusPersistence implements PersistenceProvider {

    private Deployer deployer;

    public XplusPersistence() {
    }

    /**
     * Called by Persistence class when an EntityManagerFactory is to be
     * created.
     * @param emName The name of the persistence unit
     * @param map A Map of properties for use by the persistence provider. These
     *        properties may be used to override the settings in the
     *        persistence.xml.
     * @return EntityManagerFactory for the persistence unit, or null if the
     *         provider is not the right provider
     */
    public EntityManagerFactory createEntityManagerFactory(String persistenceUnitName, Map overridenProperties) {
        deployer = new StandaloneDeployer();
        deployer.start();
        deployer.deploy(persistenceUnitName);
        return new PersistenceManagerFactory(MetaDataRepository.getUnitMetaData(persistenceUnitName));
    }

    /**
     * Called by the container when an EntityManagerFactory is to be created.
     * @param info Metadata for use by the persistence provider
     * @return EntityManagerFactory for the persistence unit specified by the
     *         metadata
     */
    public EntityManagerFactory createContainerEntityManagerFactory(PersistenceUnitInfo info) {
        return createContainerEntityManagerFactory(info, null);
    }

    @SuppressWarnings("unchecked")
    public EntityManagerFactory createContainerEntityManagerFactory(PersistenceUnitInfo info, Map overridenProperties) {
        deployer = new ContainerDeployer();
        deployer.start();

        PersistenceInfo persistenceInfo = new PersistenceInfo(info);
        deployer.deploy(persistenceInfo);
        return new PersistenceManagerFactory(MetaDataRepository.getUnitMetaData(persistenceInfo.getPersistenceUnitName()));
    }

}
