/* -------------------------------------------------------------------------*
 * PKUAS: Peking University Application Server
 * Copyright (C) 2006 Peking University
 *
 * This software is free; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * either version 2.1 or any later version.
 * 
 * This software is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this software; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 * ------------------------------------------------------------------------*/

package org.ow2.xplus.broker;

import java.lang.reflect.Method;
import java.util.Properties;

import javax.persistence.EntityManager;
import javax.persistence.EntityNotFoundException;
import javax.persistence.EntityTransaction;
import javax.persistence.FlushModeType;
import javax.persistence.LockModeType;
import javax.persistence.PersistenceException;
import javax.persistence.Query;
import javax.persistence.TransactionRequiredException;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.apache.ddlutils.io.DataToDatabaseSink;

import pku.as.cache.impl.Lock;
import org.ow2.xplus.cache.CacheAccessor;
import org.ow2.xplus.cache.DeepCopy;
import org.ow2.xplus.callback.Callback;
import org.ow2.xplus.context.Identity;
import org.ow2.xplus.context.PersistenceContext;
import org.ow2.xplus.deployer.DatabaseManager;
import org.ow2.xplus.metadata.MetaDataRepository;
import org.ow2.xplus.metadata.UnitMetaData;
import org.ow2.xplus.metadata.callback.CallbackMetaData;
import org.ow2.xplus.metadata.object.ClassMetaData;
import org.ow2.xplus.metadata.object.FieldMetaData;
import org.ow2.xplus.metadata.query.NamedNativeQueryMapping;
import org.ow2.xplus.metadata.query.NamedQueryMapping;
import org.ow2.xplus.metadata.relational.RdbTable;
import org.ow2.xplus.metadata.strategy.GeneratorStrategy;
import org.ow2.xplus.operator.EntityReader;
import org.ow2.xplus.operator.Persister;
import org.ow2.xplus.operator.Remover;
import org.ow2.xplus.query.EJBQLQuery;
import org.ow2.xplus.query.SQLQuery;
import org.ow2.xplus.transaction.PersistenceTransaction;
import org.ow2.xplus.util.NamingStrategy;

/**
 * @author Lin Liang
 */

public class PersistenceManager implements EntityManager {

    private PersistenceManagerFactory factory;

    /**
     * The database model.
     */
    private DatabaseManager database;

    private FlushModeType flushMode = FlushModeType.AUTO;

    private PersistenceContext context = new PersistenceContext();

    private UnitMetaData unitMetaData;

    private boolean open;

    private PersistenceTransaction  transaction;
    
    private CacheAccessor cacheAccessor;
    
    //apache package to sink data to database
    //added by yf
    private DataToDatabaseSink sink;

    
    private static Log log = LogFactory.getLog(PersistenceManager.class);

    public PersistenceManager(PersistenceManagerFactory factory) {
        this.factory = factory;
        this.database = factory.getUnitMetaData().getDatabase();
        unitMetaData = factory.getUnitMetaData();
        transaction = MetaDataRepository.getTxFactory().getTranscation(this);
        cacheAccessor = new CacheAccessor(factory);
//        sink = new DataToDatabaseSink(database.getPlatform(), database.getModel());
//        sink.setUseBatchMode(true);
    }

    public PersistenceManager(UnitMetaData unitMetaData) {
        this.database = unitMetaData.getDatabase();
        this.unitMetaData = unitMetaData;
        transaction = MetaDataRepository.getTxFactory().getTranscation(this);
    }

    
    /**
     * persist considered with cache
     * added by yf
     */
    public void persist(Object entity){
    	if(entity == null){
    		log.error("can't persist an empty entity!");
    		return ;
    	} 	
    	//persistenceContext
    	Identity id;
    	
    	if(!getContext().contains(entity)){	
        	/**
        	 * transient entity to persist with no id ,generate id first ;
        	 */
    		id = getIdentity(entity);
    		if(getContext().containsByKey(id)&&getContext().isTransientState(id))
    			cacheAccessor.generateID(entity, this.unitMetaData.getClassMetaData(entity.getClass().getCanonicalName()),false,this);
    		else
    			cacheAccessor.generateID(entity, this.unitMetaData.getClassMetaData(entity.getClass().getCanonicalName()),true,this);
    		id = getIdentity(entity);
    		context.addEntity(id, entity, getHashCode(entity));
    		getContext().setEntityState(id, getContext().getStateManager().getTransientState());
    		getContext().addToOrderList(entity);
    	}
    	else
    	{
    		id = getIdentity(entity);
    		getContext().setEntityState(id, getContext().getStateManager().getDirtyState());
    		getContext().addToOrderList(entity);
    	}

    }
    

    /**
     * Make an instance managed and persistent.
     * @param entity
     * @throws IllegalArgumentException if not an entity or entity is detached
     * @throws TransactionRequiredException if there is no transaction and the
     *         persistence context is of type PersistenceContextType.TRANSACTION
     */
    public void persistToDatabase(Object entity) {
        CallbackMetaData callbacks = unitMetaData.findClassMetaData(entity.getClass().getName()).getCallbacks();
        for (Callback callback : callbacks.getPrePersistCallbacks()) {
            callback.invoke(entity);
        }

        String entityClassName = entity.getClass().getCanonicalName();
        EntityReader accessor = factory.getEntityAccessor(entityClassName);
        Persister persister = new Persister(accessor);
        persister.persist(entity, database,sink);


        for (Callback callback : callbacks.getPostPersistCallbacks()) {
            callback.invoke(entity);
        }
    }

    public Identity getIdentity(Object entity) {
        ClassMetaData entityMetaData = factory.getUnitMetaData().findClassMetaData(entity.getClass().getCanonicalName());
        Object key = null;
        Method[] methods = entity.getClass().getMethods();
        for (FieldMetaData field : entityMetaData.getDeclaredFields()) {
            if (field.isId()) {
                String getMethodName = NamingStrategy.getFieldAccessMethod(field.getFieldName(), "get");
                for (Method method : methods) {
                    if (method.getName().equals(getMethodName)) {
                        try {
                            key = method.invoke(entity, new Object[] {});
                        } catch (Exception e) {
                        }
                        break;
                    }
                }
                break;
            }
        }
        Identity id = new Identity(key, entity.getClass().getCanonicalName());
        return id;
    }

    
    
    /**
     * Merge the state of the given entity into the current persistence context.
     * @param entity
     * @return the instance that the state was merged to
     * @throws IllegalArgumentException if instance is not an entity or is a
     *         removed entity
     * @throws TransactionRequiredException if there is no transaction and the
     *         persistence context is of type PersistenceContextType.TRANSACTION
     */
    public <T> T merge(T entity) {
        // TODO:
        persist(entity);
        flush();
        return entity;
    
    }

    /**
     * remove with cache
     * added by yf
     */
    public void remove(Object entity){
    	Identity id = getIdentity(entity);
    	
    	//if already deleted from database
    	if(getContext().isGoneState(id)){
    		log.debug("entity has already deleted from database!");
    		return ;
    	}
    	
    	if(!getContext().containsByKey(id)){
    		getContext().addEntity(id, entity, getHashCode(entity));
    	}
    	getContext().setEntityState(id, getContext().getStateManager().getDeletedState());
    }
    
    
    
    /**
     * Remove the entity instance.
     * @param entity
     * @throws IllegalArgumentException if not an entity or if a detached entity
     * @throws TransactionRequiredException if there is no transaction and the
     *         persistence context is of type PersistenceContextType.TRANSACTION
     */
    public void removeFromDatabase(Object entity) {
        CallbackMetaData callbacks = unitMetaData.findClassMetaData(entity.getClass().getName()).getCallbacks();
        for (Callback callback : callbacks.getPreRemoveCallbacks()) {
            callback.invoke(entity);
        }

        String entityClassName = entity.getClass().getCanonicalName();
        EntityReader accessor = factory.getEntityAccessor(entityClassName);
        Remover remover = new Remover(accessor);
        remover.remove(entity, database);

        //Identity id = getIdentity(entity);
        //context.removeEntity(id);

        for (Callback callback : callbacks.getPostRemoveCallbacks()) {
            callback.invoke(entity);
        }
    }

    
    /**
     * find with cache
     * added by yf 
     */
    public <T> T find(Class<T> entityClass, Object primaryKey){
    	
    	Identity id = new Identity(primaryKey,entityClass.getCanonicalName());
    	T entity = null;
    	if(getContext().containsByKey(id)){
    		if(getContext().isDeletedState(id)|| getContext().isGoneState(id)){
    			log.debug("can't find an entity which has been deleted or removed !");
    			return null;
    		}
    		entity = (T)getContext().getEntity(id);
    		return entity;
    	}
    	else if(this.factory.haveCache()){
    		entity = (T)this.factory.getCache().get(id, this.transaction.getTimeStamper());
    		if(entity != null){
    			log.debug("cache hits!");
    			return (T) cacheAccessor.CacheHits(entity,this);
    		}
    	}
    	entity = findFromDatabase(entityClass,primaryKey);
    	if(entity!= null){
    		getContext().addEntity(id, entity, getHashCode(entity));
    		getContext().setEntityState(id, getContext().getStateManager().getCleanState());
    		if(this.factory.haveCache()){
    			cacheAccessor.CacheMiss(entity, this);
    		}
    	}
		return entity ;
    	
    }
    
    
    
    
    /**
     * Find by primary key.
     * @param entityClass
     * @param primaryKey
     * @return the found entity instance or null if the entity does not exist
     * @throws IllegalArgumentException if the first argument does not denote an
     *         entity type or the second argument is not a valid type for that
     *         entity's primary key
     */
    @SuppressWarnings("unchecked")
    public <T> T findFromDatabase(Class<T> entityClass, Object primaryKey) {
        StringBuilder ejbql = new StringBuilder(1024);
        ejbql.append("select object(entity) from ");
        ejbql.append(NamingStrategy.unqualify(entityClass.getCanonicalName()));
        ejbql.append(" entity where entity.");
        ClassMetaData entityMetaData = factory.getUnitMetaData().findClassMetaData(entityClass.getCanonicalName());
        for (FieldMetaData field : entityMetaData.getDeclaredFields()) {
            if (field.isId()) {
                ejbql.append(field.getFieldName());
                break;
            }
        }
        ejbql.append("=\'");
        ejbql.append(primaryKey.toString());
        ejbql.append("\'");
        EJBQLQuery ejbqlQuery = new EJBQLQuery(ejbql.toString(), this);
        T entity = null;
        try {
            entity = (T) ejbqlQuery.getSingleResult();
        } catch (EntityNotFoundException enf) {
        }

        if (entity != null) {
            CallbackMetaData callbacks = unitMetaData.findClassMetaData(entityClass.getName()).getCallbacks();
            for (Callback callback : callbacks.getPostLoadCallbacks()) {
                callback.invoke(entity);
            }
        }
        for(Object id : context.getAllIdentities())
        {
        	if(getContext().isCascadeFindState((Identity)id)){
        		//put into cache
        		cacheAccessor.put(context.getEntity((Identity)id), this);
        		getContext().setEntityState((Identity)id, getContext().getStateManager().getCleanState());
        	}
        }
        return entity;
    }

    /**
     * Get an instance, whose state may be lazily fetched. If the requested
     * instance does not exist in the database, throws EntityNotFoundException
     * when the instance state is first accessed. (The persistence provider
     * runtime is permitted to throw the EntityNotFoundException when
     * getReference is called.) The application should not expect that the
     * instance state will be available upon detachment, unless it was accessed
     * by the application while the entity manager was open.
     * @param entityClass
     * @param primaryKey
     * @return the found entity instance
     * @throws IllegalArgumentException if the first argument does not denote an
     *         entity type or the second argument is not a valid type for that
     *         entity's primary key
     * @throws EntityNotFoundException if the entity state cannot be accessed
     */
    public <T> T getReference(Class<T> entityClass, Object primaryKey) {
        return find(entityClass, primaryKey);
    }

    /**
     * @param entity entity instance
     * @return hashcode
     */
    public Integer getHashCode(Object entity) {
        int hash = 0;
        ClassMetaData entityMetaData = factory.getUnitMetaData().findClassMetaData(entity.getClass().getCanonicalName());
        Method[] methods = entity.getClass().getMethods();
        for (FieldMetaData field : entityMetaData.getDeclaredFields()) {
            String getMethodName = NamingStrategy.getFieldAccessMethod(field.getFieldName(), "get");
            for (Method method : methods) {
                if (method.getName().equals(getMethodName)) {
                    try {
                        Object value = method.invoke(entity, new Object[] {});
                        hash = hash * 17 + value.hashCode();
                    } catch (Exception e) {
                    }
                }
            }
        }
        return new Integer(hash);
    }

    /**
     * Synchronize the persistence context to the underlying database.
     * @throws TransactionRequiredException if there is no transaction
     * @throws PersistenceException if the flush fails
     */

    public void flush() {
//    		sink.start();
    	  for (Object entity : context.getOrderList()){
    		  Identity id = getIdentity(entity);
    		  Integer hashcode = context.getHashCode(id);
    		    if(context.isCleanState(id))
                {
                if ((hashcode == null || (!hashcode.equals(getHashCode(entity)) && hashcode.intValue() != 0))) {
                    /**
                     * do some update on the entity , the cache item should be locked avoid to be used by others
                     */
              	  Lock lock = cacheAccessor.lock(id,this.transaction.getTimeStamper(),this.getContext());
              	  
              	  CallbackMetaData callbacks = unitMetaData.findClassMetaData(entity.getClass().getName()).getCallbacks();
                    for (Callback callback : callbacks.getPreUpdateCallbacks()) {
                        callback.invoke(entity);
                    }

                    // persist(entity);
                    String entityClassName = entity.getClass().getCanonicalName();
                    EntityReader accessor = factory.getEntityAccessor(entityClassName);
                    Persister persister = new Persister(accessor);
                    persister.persist(entity, database,sink);

                   

                    for (Callback callback : callbacks.getPostUpdateCallbacks()) {
                        callback.invoke(entity);
                    }    
                    
                    /**
                     * unlock the item
                     */
                    cacheAccessor.unlock(id, lock);
                }
                }
                else if(getContext().isDeletedState(id)){
              	  
              	
              	Lock lock = cacheAccessor.lock(id,this.transaction.getTimeStamper(),this.getContext());
              	
                	removeFromDatabase(entity);
                	getContext().setEntityState(id, getContext().getStateManager().getGoneState());
                	
                	cacheAccessor.unlock(id, lock);
                }
                else if(getContext().isTransientState(id)){
              	  /**
              	   * for a transient entity instance , id is null and should remove before persist 
              	   * it to database , no lock on cache
              	   */
              	
                	persistToDatabase(entity);
                	
                	getContext().setEntityState(id, getContext().getStateManager().getCleanState());

                  }
                else if(getContext().isDirtyState(id)){
              	  
              	  Lock lock = cacheAccessor.lock(id,this.transaction.getTimeStamper(),this.getContext());
              	  
              	  persistToDatabase(entity);
              	  
              	  getContext().setEntityState(id, getContext().getStateManager().getCleanState());
              	  
              	  cacheAccessor.unlock(id, lock);
                }
    	  }
    	  
    	  for (Object entity : context.getAllEntities()) {
              Identity id = getIdentity(entity);
              Integer hashcode = context.getHashCode(id);

              if(context.isCleanState(id))
              {
              if ((hashcode == null || (!hashcode.equals(getHashCode(entity)) && hashcode.intValue() != 0))) {
                  /**
                   * do some update on the entity , the cache item should be locked avoid to be used by others
                   */
            	  Lock lock = cacheAccessor.lock(id,this.transaction.getTimeStamper(),this.getContext());
            	  
            	  CallbackMetaData callbacks = unitMetaData.findClassMetaData(entity.getClass().getName()).getCallbacks();
                  for (Callback callback : callbacks.getPreUpdateCallbacks()) {
                      callback.invoke(entity);
                  }

                  // persist(entity);
                  String entityClassName = entity.getClass().getCanonicalName();
                  EntityReader accessor = factory.getEntityAccessor(entityClassName);
                  Persister persister = new Persister(accessor);
                  persister.persist(entity, database,sink);

                 

                  for (Callback callback : callbacks.getPostUpdateCallbacks()) {
                      callback.invoke(entity);
                  }    
                  
                  /**
                   * unlock the item
                   */
                  cacheAccessor.unlock(id, lock);
              }
              }
              else if(getContext().isDeletedState(id)){
            	  
            	
            	Lock lock = cacheAccessor.lock(id,this.transaction.getTimeStamper(),this.getContext());
            	
              	removeFromDatabase(entity);
              	getContext().setEntityState(id, getContext().getStateManager().getGoneState());
              	
              	cacheAccessor.unlock(id, lock);
              }
              else if(getContext().isTransientState(id)){
            	  /**
            	   * for a transient entity instance , id is null and should remove before persist 
            	   * it to database , no lock on cache
            	   */
            	
              	persistToDatabase(entity);
              	
              	getContext().setEntityState(id, getContext().getStateManager().getCleanState());
              	

                  	
                }
              else if(getContext().isDirtyState(id)){
            	  
            	  Lock lock = cacheAccessor.lock(id,this.transaction.getTimeStamper(),this.getContext());
            	  
            	  persistToDatabase(entity);
            	  
            	  getContext().setEntityState(id, getContext().getStateManager().getCleanState());
            	  
            	  cacheAccessor.unlock(id, lock);
              }
          }
    	  getContext().clearOrderList();
//    	  sink.end();//DataToDatabaseSink
    }
    
    
    /**
     * Set the flush mode that applies to all objects contained in the
     * persistence context.
     * @param flushMode
     */
    public void setFlushMode(FlushModeType flushMode) {
        this.flushMode = flushMode;
    }

    /**
     * Get the flush mode that applies to all objects contained in the
     * persistence context.
     * @return flushMode
     */
    public FlushModeType getFlushMode() {
        return flushMode;
    }

    /**
     * Set the lock mode for an entity object contained in the persistence
     * context.
     * @param entity
     * @param lockMode
     * @throws PersistenceException if an unsupported lock call is made
     * @throws IllegalArgumentException if the instance is not an entity or is a
     *         detached entity
     * @throws TransactionRequiredException if there is no transaction
     */
    public void lock(Object entity, LockModeType lockMode) {

    }

    /**
     * Refresh the state of the instance from the database, overwriting changes
     * made to the entity, if any.
     * @param entity
     * @throws IllegalArgumentException if not an entity or entity is not
     *         managed
     * @throws TransactionRequiredException if there is no transaction and the
     *         persistence context is of type PersistenceContextType.TRANSACTION
     * @throws EntityNotFoundException if the entity no longer exists in the
     *         database
     */
    public void refresh(Object entity) {
        Object primaryKey = null;
        ClassMetaData entityMetaData = factory.getUnitMetaData().findClassMetaData(entity.getClass().getCanonicalName());
        for (FieldMetaData field : entityMetaData.getDeclaredFields()) {
            if (field.isId()) {
                String getMethodName = NamingStrategy.getFieldAccessMethod(field.getFieldName(), "GET");
                try {
                    Method get = entity.getClass().getMethod(getMethodName, new Class[] {});
                    primaryKey = get.invoke(entity, new Object[] {});
                } catch (Exception e) {
                    e.printStackTrace();
                }
            }
        }

        Object dbEntity = findFromDatabase(entity.getClass(), primaryKey);
        if(dbEntity == null){
        	log.debug("can't refresh an entity non't in database!");
        	return ;
        }


        for (FieldMetaData field : entityMetaData.getDeclaredFields()) {
            String getMethodName = NamingStrategy.getFieldAccessMethod(field.getFieldName(), "GET");
            String setMethodName = NamingStrategy.getFieldAccessMethod(field.getFieldName(), "SET");
            try {
                Method getMethod = entity.getClass().getMethod(getMethodName, new Class[] {});
                Object value = getMethod.invoke(dbEntity, new Object[] {});
                Method setMethod = entity.getClass().getMethod(setMethodName, value.getClass());
                setMethod.invoke(entity, value);
            } catch (Exception e) {
            }
        }
        Identity id = getIdentity(entity);
        getContext().addEntity(id, entity, getHashCode(entity));
        getContext().setEntityState(id, getContext().getStateManager().getCleanState());
    }

    /**
     * Clear the persistence context, causing all managed entities to become
     * detached. Changes made to entities that have not been flushed to the
     * database will not be persisted.
     */
    public void clear() {
    	flush();
        context.clear();
    }

    /**
     * Check if the instance belongs to the current persistence context.
     * @param entity
     * @return
     * @throws IllegalArgumentException if not an entity
     */
    public boolean contains(Object entity) {
        return context.contains(entity);
    }

    /**
     * Create an instance of Query for executing an EJB QL statement.
     * @param ejbqlString an EJB QL query string
     * @return the new query instance
     * @throws IllegalArgumentException if query string is not valid
     */
    public Query createQuery(String ejbqlString) {
        if (flushMode == FlushModeType.AUTO)
            flush();
        EJBQLQuery ejbqlQuery = new EJBQLQuery(ejbqlString, this);
        return ejbqlQuery;
    }

    /**
     * Create an instance of Query for executing a named query (in EJB QL or
     * native SQL).
     * @param name the name of a query defined in metadata
     * @return the new query instance
     * @throws IllegalArgumentException if a query has not been defined with the
     *         given name
     */
    public Query createNamedQuery(String name) {
        if (flushMode == FlushModeType.AUTO)
            flush();
        NamedQueryMapping namedQuery = unitMetaData.getNamedQuery(name);
        if (namedQuery != null) {
            String ejbql = namedQuery.getQuery();
            return new EJBQLQuery(ejbql, this);
        }
        NamedNativeQueryMapping namedNativeQuery = unitMetaData.getNamedNativeQuery(name);
        if (namedNativeQuery != null) {
            String sql = namedNativeQuery.getQuery();
            SQLQuery sqlQuery = new SQLQuery(sql, this);
            String resultSetMappingName = namedNativeQuery.getResultSetMapping();
            if (resultSetMappingName != null) {
                sqlQuery.setResultSetMapping(unitMetaData.getResultSet(resultSetMappingName));
            }
            Class resultClass = namedNativeQuery.getResultClass();
            if (resultClass != null) {
                sqlQuery.setResultClass(resultClass);
            }
            return sqlQuery;
        }
        return null;
    }

    /**
     * Create an instance of Query for executing a native SQL statement, e.g.,
     * for update or delete.
     * @param sqlString a native SQL query string
     * @return the new query instance
     */
    public Query createNativeQuery(String sqlString) {
        if (flushMode == FlushModeType.AUTO)
            flush();
        SQLQuery sqlQuery = new SQLQuery(sqlString, this);
        return sqlQuery;
    }

    /**
     * Create an instance of Query for executing a native SQL query.
     * @param sqlString a native SQL query string
     * @param resultClass the class of the resulting instance(s)
     * @return the new query instance
     */
    public Query createNativeQuery(String sqlString, Class resultClass) {
        if (flushMode == FlushModeType.AUTO)
            flush();
        SQLQuery sqlQuery = new SQLQuery(sqlString, this);
        sqlQuery.setResultClass(resultClass);
        return sqlQuery;
    }

    /**
     * Create an instance of Query for executing a native SQL query.
     * @param sqlString a native SQL query string
     * @param resultSetMapping the name of the result set mapping
     * @return the new query instance
     */
    public Query createNativeQuery(String sqlString, String resultSetMapping) {
        if (flushMode == FlushModeType.AUTO)
            flush();
        SQLQuery sqlQuery = new SQLQuery(sqlString, this);
        sqlQuery.setResultSetMapping(unitMetaData.getResultSet(resultSetMapping));
        return sqlQuery;
    }

    /**
     * Close an application-managed EntityManager. After an EntityManager has
     * been closed, all methods on the EntityManager instance will throw the
     * IllegalStateException except for isOpen, which will return false. This
     * method can only be called when the EntityManager is not associated with
     * an active transaction.
     * @throws IllegalStateException if the EntityManager is associated with an
     *         active transaction or if the EntityManager is container-managed.
     */
    public void close() {
        getContext().clear();
        open = false;
    }

    /**
     * Determine whether the EntityManager is open.
     * @return true until the EntityManager has been closed.
     */
    public boolean isOpen() {
        return open;
    }

    /**
     * Return the resource-level transaction object. The EntityTransaction
     * instance may be used serially to begin and commit multiple transactions.
     * @return EntityTransaction instance
     * @throws IllegalStateException if invoked on a JTA EntityManager or an
     *         EntityManager that has been closed.
     */
    public PersistenceTransaction getTransaction() {
        return (PersistenceTransaction)transaction;
    }

    public synchronized PersistenceContext getContext() {
        return context;
    }

    public UnitMetaData getUnitMetaData() {
        return unitMetaData;
    }

    public void joinTransaction() {
        // TODO Auto-generated method stub

    }

    public Object getDelegate() {
        // TODO Auto-generated method stub
        return null;
    }

	public PersistenceManagerFactory getFactory() {
		return factory;
	}

	public void setFactory(PersistenceManagerFactory factory) {
		this.factory = factory;
	}
}
