/* -------------------------------------------------------------------------*
 * PKUAS: Peking University Application Server
 * Copyright (C) 2006 Peking University
 *
 * This software is free; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * either version 2.1 or any later version.
 * 
 * This software is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this software; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 * ------------------------------------------------------------------------*/

package org.ow2.xplus.broker;

import java.util.HashMap;
import java.util.List;
import java.util.Map;

import javax.persistence.EntityManager;
import javax.persistence.EntityManagerFactory;
import javax.persistence.PersistenceContextType;

import org.ow2.xplus.cache.CacheGenerate;
import org.ow2.xplus.metadata.UnitMetaData;
import org.ow2.xplus.metadata.object.ClassMetaData;
import org.ow2.xplus.operator.EntityReader;
import org.ow2.xplus.setting.CacheConfigInfo;
import org.ow2.xplus.setting.CacheSetting;
import pku.as.cache.Cache;
import pku.as.cache.CacheFactory;
import pku.as.cache.impl.Timestamper;

/**
 * @author Lin Liang
 */

public class PersistenceManagerFactory implements EntityManagerFactory {

    private Map<String, EntityReader> accessors = new HashMap<String, EntityReader>();

    /**
     * the metadata of unit
     */
    private UnitMetaData unitMetaData;
	private Cache cache=null;
	private CacheConfigInfo info = new CacheConfigInfo();


    public PersistenceManagerFactory(UnitMetaData unitMetaData) {
        this.unitMetaData = unitMetaData;
        List<ClassMetaData> classMetaDatas = unitMetaData.getClassMetaDatas();
        for (ClassMetaData classMetaData : classMetaDatas) {
            EntityReader accessor = new EntityReader(classMetaData);
            accessors.put(classMetaData.getEntityName(), accessor);
        }
        new CacheSetting(info);
        if(info.isUseCache())
        {
        	CacheGenerate generate = new CacheGenerate(info);
        	cache = generate.createSecondLevelCache();
        } 
    }

    /**
     * added by yf
     * @return
     */
	public boolean haveCache(){
		if(cache == null)return false;
		else 
			return true;
	}
	public Cache getCache(){
		return cache;
	}
  
    
    /**
     * Create a new EntityManager of of type PersistenceContextType.TRANSACTION.
     * This method returns a new application-managed PersistenceSession instance
     * (with a new stand-alone persistence context) each time it is invoked. The
     * isOpen method will return true on the returned instance.
     */
    public EntityManager createEntityManager() {
        return new PersistenceManager(this);
    }

    /**
     * Create a new EntityManager of the specified persistence context type.
     * This method returns a new application-managed EntityManager instance
     * (with a new stand-alone persistence context) each time it is invoked. The
     * isOpen method will return true on the returned instance.
     */
    public EntityManager createEntityManager(PersistenceContextType type) {
        // TODO:
        return new PersistenceManager(this);
    }

    /**
     * Get an EntityManager instance whose persistence context is propagated
     * with the current JTA transaction. If there is no persistence context
     * bound to the current JTA transaction, a new transaction-scoped
     * persistence context is created and associated with the transaction and
     * the EntityManager instance that is created and returned. If no JTA
     * transaction is in progress, an EntityManager instance is created for
     * which the persistence context will be propagated with subsequent JTA
     * transactions. Throws IllegalStateException if called on an
     * PersistenceSessionFactory that does not provide JTA PersistenceSessions.
     */
    public EntityManager getEntityManager() {
        // TODO:
        return new PersistenceManager(this);
    }

    /**
     * Close the factory, releasing any resources that it holds. After a factory
     * instance is closed, all methods invoked on it will throw an
     * IllegalStateException, except for isOpen, which will return false.
     */
    public void close() {

    }

    /**
     * Indicates whether the factory is open. Returns true until the factory has
     * been closed.
     */
    public boolean isOpen() {
        // TODO:
        return true;
    }

    public UnitMetaData getUnitMetaData() {
        return unitMetaData;
    }

    public EntityReader getEntityAccessor(String entityClassName) {
        return accessors.get(entityClassName);
    }

    public EntityManager createEntityManager(Map arg0) {
        // TODO Auto-generated method stub
        return new PersistenceManager(this);
    }

}
