package org.ow2.xplus.cache;

import java.lang.reflect.InvocationTargetException;
import java.lang.reflect.Method;
import java.util.List;

import org.jdom.Element;

import pku.as.cache.Cache;
import pku.as.cache.impl.Lock;
import org.ow2.xplus.broker.PersistenceManager;
import org.ow2.xplus.broker.PersistenceManagerFactory;
import org.ow2.xplus.context.Identity;
import org.ow2.xplus.context.PersistenceContext;
import org.ow2.xplus.exception.PropertyAccessException;
import org.ow2.xplus.metadata.MetaDataRepository;
import org.ow2.xplus.metadata.UnitMetaData;
import org.ow2.xplus.metadata.object.ClassMetaData;
import org.ow2.xplus.metadata.object.FieldMetaData;
import org.ow2.xplus.metadata.relational.RdbTable;
import org.ow2.xplus.metadata.strategy.GeneratorStrategy;
import org.ow2.xplus.operator.accessor.Accessor;
import org.ow2.xplus.query.SQLQuery;
import org.ow2.xplus.transaction.PersistenceTransaction;
import org.ow2.xplus.util.NamingStrategy;
import org.ow2.xplus.cache.ID4Transient;
/**
 * 
 * @author yf
 *
 */
public class CacheAccessor {
private Cache cache;
private PersistenceManagerFactory factory;

public CacheAccessor(PersistenceManagerFactory factory){
	this.cache = factory.getCache();
	this.factory = factory;
}

/**
 * take a copy of the hit entity
 * @param <T>
 * @param entity
 * @param pm
 * @return
 */
public<T> T CacheHits(T entity,PersistenceManager pm){
	DeepCopy copy = new DeepCopy(pm.getUnitMetaData().getClassMetaData(entity.getClass().getCanonicalName()),pm);
	T newEntity = (T) copy.deepCopy(entity);
	Identity id = pm.getIdentity(newEntity);
	//put into the new map
	pm.getContext().addEntity(id, newEntity, pm.getHashCode(newEntity));
	pm.getContext().setEntityState(id,pm.getContext().getStateManager().getCleanState());
	//put into the old map
	pm.getContext().addOEntity(id, entity, null);
	return newEntity;
}

public void CacheMiss(Object entity,PersistenceManager pm){
	DeepCopy copy = new DeepCopy(pm.getUnitMetaData().getClassMetaData(entity.getClass().getCanonicalName()),pm);
	Object newEntity = copy.deepCopy(entity);
	Identity id = pm.getIdentity(newEntity);
	this.factory.getCache().put(id, newEntity, ((PersistenceTransaction)(pm.getTransaction())).getTimeStamper(), null, null, false);
	pm.getContext().addOEntity(id, newEntity, pm.getHashCode(newEntity));
}

public void put(Object entity,PersistenceManager pm){
	DeepCopy copy = new DeepCopy(pm.getUnitMetaData().getClassMetaData(entity.getClass().getCanonicalName()),pm);
	Object newEntity = copy.deepCopy(entity);
	Identity id = pm.getIdentity(newEntity);
	this.factory.getCache().put(id, entity, pm.getTransaction().getTimeStamper(), null, null, false);
	pm.getContext().addOEntity(id, newEntity, pm.getHashCode(newEntity));
}



public Lock lock(Identity id,long timeStamper,PersistenceContext context){
	if(!this.factory.haveCache())
		return null;
	if(context.containsByKeyO(id)){
		return this.cache.lock(id, null);
	}
	return null;
}

public void unlock(Identity id,Lock lock){
	if(!this.factory.haveCache())
		return ;
	if(lock == null)
		return ;
	this.cache.release(id, lock);
}

/**
 * 
 * @param entity
 * @param classMetaData
 * @param first whether it's the first entity with the default id ;
 */
public void generateID(Object entity,ClassMetaData classMetaData,boolean first,PersistenceManager pm){
	ID4Transient id4ts = new ID4Transient(pm);
	id4ts.getID(entity, classMetaData,first);
}

public boolean isIDNull(Object entity,ClassMetaData classMetaData){
	UnitMetaData unit = MetaDataRepository.getUnitMetaData(classMetaData.getUnitName());
	
    String tableName = classMetaData.getPrimaryTableName();

    RdbTable primaryTable = unit.getTable(tableName);
    for (FieldMetaData field : classMetaData.getAllFields()) {
        Object fieldData = getFieldValue(entity, field.getFieldName());
        if (field.isId()) {
            GeneratorStrategy generator = unit.getTableIdGen(primaryTable.getName()).getGeneratorStrategy();
            if (generator != null) {
            	if (fieldData.equals(generator.getDefaultValue())||fieldData.toString().equalsIgnoreCase(generator.getDefaultValue().toString())) {
            		return true;
            	}
            }
        }
    }
	  return false;
}

protected Object getFieldValue(Object entityData, String propertyName) {
    Method[] methods = entityData.getClass().getMethods();
    Object value = null;

    String getMethodName = NamingStrategy.getFieldAccessMethod(propertyName, "get");
    String isMethodName = NamingStrategy.getFieldAccessMethod(propertyName, "is");
    for (Method method : methods) {
        if (method.getName().equals(getMethodName) || method.getName().equals(isMethodName)) {
            try {
                value = method.invoke(entityData, new Object[] {});
            } catch (IllegalArgumentException iae) {
                PropertyAccessException ex = new PropertyAccessException(iae, PropertyAccessException.ILLEGAL_ARGUMENT_ERROR);
                ex.showMessage();
            } catch (IllegalAccessException iace) {
                PropertyAccessException ex = new PropertyAccessException(iace, PropertyAccessException.ILLEGAL_ACCESS_ERROR);
                ex.showMessage();
            } catch (InvocationTargetException ite) {
                PropertyAccessException ex = new PropertyAccessException(ite, PropertyAccessException.INVOCATION_TARGET_ERROR);
                ex.showMessage();
            }
        }
    }
    	return value;
}

protected void setFieldValue(Object entityData, String propertyName, Object value){
	   Method[] methods = entityData.getClass().getMethods();

    String setMethodName = NamingStrategy.getFieldAccessMethod(propertyName, "set");
    for (Method method : methods) {
        if (method.getName().equals(setMethodName)) {
            try {
                    method.invoke(entityData, new Object[] { value });
            } catch (Exception ex) {
                ex.printStackTrace();
            }
        }
    }
}

public Cache getCache() {
	return cache;
}
public void setCache(Cache cache) {
	this.cache = cache;
}

}
