package org.ow2.xplus.cache;

import java.lang.reflect.InvocationTargetException;
import java.lang.reflect.Method;
import java.util.List;

import org.ow2.xplus.broker.PersistenceManager;
import org.ow2.xplus.exception.PropertyAccessException;
import org.ow2.xplus.metadata.MetaDataRepository;
import org.ow2.xplus.metadata.UnitMetaData;
import org.ow2.xplus.metadata.object.ClassMetaData;
import org.ow2.xplus.metadata.object.FieldMetaData;
import org.ow2.xplus.metadata.relational.RdbTable;
import org.ow2.xplus.metadata.strategy.GeneratorStrategy;
import org.ow2.xplus.operator.accessor.ClassAccessor;
import org.ow2.xplus.query.SQLQuery;
import org.ow2.xplus.util.NamingStrategy;

public class ID4Transient {
	
	PersistenceManager pm;
	public ID4Transient(PersistenceManager pm){
		this.pm=pm;
	}
	public void generateID(Object entity,ClassMetaData classMetaData,boolean first){
		UnitMetaData unit = MetaDataRepository.getUnitMetaData(classMetaData.getUnitName());
		
	    String tableName = classMetaData.getPrimaryTableName();

	    RdbTable primaryTable = unit.getTable(tableName);
	 
	    // process value
	    for (FieldMetaData field : classMetaData.getAllFields()) {
	            Object fieldData = getFieldValue(entity, field.getFieldName());
	            if (field.isId()&& field.isGeneratedID()) {
	                GeneratorStrategy generator = unit.getTableIdGen(primaryTable.getName()).getGeneratorStrategy();
	                if (generator != null) {
	                	//fieldData.equals(generator.getDefaultValue())||fieldData.toString().equalsIgnoreCase(generator.getDefaultValue().toString())
	                    if (fieldData.equals(generator.getDefaultValue())||fieldData.toString().equalsIgnoreCase(generator.getDefaultValue().toString())) {
	                        List result = null;
	                        try {
	                            String query = "SELECT " + primaryTable.getColumns().get(0).getName() + " FROM \""
	                                    + primaryTable.getName() + "\"";
	                            SQLQuery sqlQuery = new SQLQuery(query, null);
	                            sqlQuery.setUnit(unit);
	                            result = sqlQuery.getSQLResult();
	                        } catch (Exception ex) {
	                            String query = "SELECT " + primaryTable.getColumns().get(0).getName() + " FROM "
	                                    + primaryTable.getName();
	                            SQLQuery sqlQuery = new SQLQuery(query, null);
	                            sqlQuery.setUnit(unit);
	                            result = sqlQuery.getSQLResult();
	                        }
	                        if ((!first)||((result != null) && (result.size() > 0))) {
	                            fieldData = generator.generate();
	                            setFieldValue(entity, field.getFieldName(), fieldData);
	                        }
	                    }
	                }
	            }
	        }
	}

	public void getID(Object entity,ClassMetaData classMetaData,boolean first){
		UnitMetaData unit = MetaDataRepository.getUnitMetaData(classMetaData.getUnitName());
		
		generateID(entity,classMetaData,first);
		
		if(classMetaData.getParent()!=null){
		    ClassMetaData parentMetaData = unit.getClassMetaData(classMetaData.getParent().getCanonicalName());
		    getID(entity,parentMetaData,first);
		}
	}
	
	protected Object getFieldValue(Object entityData, String propertyName) {
	    Method[] methods = entityData.getClass().getMethods();
	    Object value = null;

	    String getMethodName = NamingStrategy.getFieldAccessMethod(propertyName, "get");
	    String isMethodName = NamingStrategy.getFieldAccessMethod(propertyName, "is");
	    for (Method method : methods) {
	        if (method.getName().equals(getMethodName) || method.getName().equals(isMethodName)) {
	            try {
	                value = method.invoke(entityData, new Object[] {});
	            } catch (IllegalArgumentException iae) {
	                PropertyAccessException ex = new PropertyAccessException(iae, PropertyAccessException.ILLEGAL_ARGUMENT_ERROR);
	                ex.showMessage();
	            } catch (IllegalAccessException iace) {
	                PropertyAccessException ex = new PropertyAccessException(iace, PropertyAccessException.ILLEGAL_ACCESS_ERROR);
	                ex.showMessage();
	            } catch (InvocationTargetException ite) {
	                PropertyAccessException ex = new PropertyAccessException(ite, PropertyAccessException.INVOCATION_TARGET_ERROR);
	                ex.showMessage();
	            }
	        }
	    }
	    	return value;
	}

	protected void setFieldValue(Object entityData, String propertyName, Object value){
		   Method[] methods = entityData.getClass().getMethods();

	    String setMethodName = NamingStrategy.getFieldAccessMethod(propertyName, "set");
	    for (Method method : methods) {
	        if (method.getName().equals(setMethodName)) {
	            try {
	                    method.invoke(entityData, new Object[] { value });
	            } catch (Exception ex) {
	                ex.printStackTrace();
	            }
	        }
	    }
	}
}
