/* -------------------------------------------------------------------------*
 * PKUAS: Peking University Application Server
 * Copyright (C) 2006 Peking University
 *
 * This software is free; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * either version 2.1 or any later version.
 * 
 * This software is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this software; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 * ------------------------------------------------------------------------*/

package org.ow2.xplus.callback;

import java.lang.reflect.Method;

/**
 * Entity lifecycle callback methods can be defined on an entity listener class
 * and/or directly on an entity class or mapped superclass.
 * @author Lin Liang
 */

public class ListenerCallback implements Callback {

    /**
     * Lifecycle callback methods
     */
    private Method callbackMethod;

    /**
     * entity listener
     */
    private Class listenerClass;

    public ListenerCallback(Method callbackMethod, Class listenerClass) {
        this.callbackMethod = callbackMethod;
        this.listenerClass = listenerClass;
    }

    public void invoke(Object bean) {
        if (bean == null)
            return;
        try {
            Object listener = listenerClass.newInstance();
            callbackMethod.invoke(listener, new Object[] { bean });
        } catch (Exception e) {
            throw new RuntimeException(e);
        }
    }
}
