/* -------------------------------------------------------------------------*
 * PKUAS: Peking University Application Server
 * Copyright (C) 2006 Peking University
 *
 * This software is free; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * either version 2.1 or any later version.
 * 
 * This software is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this software; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 * ------------------------------------------------------------------------*/

package org.ow2.xplus.datasvc;

import java.io.PrintWriter;
import java.sql.Connection;
import java.sql.SQLException;
import java.util.Properties;

import javax.naming.InitialContext;
import javax.naming.NamingException;
import javax.sql.DataSource;

import org.ow2.xplus.deployer.PersistenceXMLParser;

/**
 * A connection provider that uses a DataSource registered with JNDI.
 * @author Lin Liang
 */

public class JNDIDataSource extends AbstractDataSource {

    public JNDIDataSource() {
        super();
    }

    public void configure(Properties props) {
        try {
            InitialContext initialContext = new InitialContext();
            dataSource = (DataSource) initialContext.lookup(props.getProperty(PersistenceXMLParser.PERSISTENCE_DS_NAME));
        } catch (NamingException ne) {
            ne.printStackTrace();
        }
        props.put(PersistenceXMLParser.PERSISTENCE_DB_NAME, getDatabaseName());
        // props.put(PersistenceXMLParser.PERSISTENCE_DB_DIALECT,
        // getDatabaseDialect(getDriverClassName()));
    }

    public Connection getConnection() throws SQLException {
        if (dataSource != null)
            return dataSource.getConnection();
        return null;
    }

    public Connection getConnection(String username, String password) throws SQLException {
        if (dataSource != null)
            return dataSource.getConnection(username, password);
        return null;
    }

    public PrintWriter getLogWriter() throws SQLException {
        if (dataSource != null)
            return dataSource.getLogWriter();
        return null;
    }

    public void setLogWriter(PrintWriter out) throws SQLException {
        if (dataSource != null)
            dataSource.setLogWriter(out);
    }

    public void setLoginTimeout(int seconds) throws SQLException {
        if (dataSource != null)
            dataSource.setLoginTimeout(seconds);
    }

    public int getLoginTimeout() throws SQLException {
        if (dataSource != null)
            return dataSource.getLoginTimeout();
        return 0;
    }

	public <T> T unwrap(Class<T> iface) throws SQLException {
		// TODO Auto-generated method stub
		return null;
	}

	public boolean isWrapperFor(Class<?> iface) throws SQLException {
		// TODO Auto-generated method stub
		return false;
	}
}
