package org.ow2.xplus.datasvc.connection;

import java.io.PrintWriter;
import java.sql.Connection;
import java.sql.DriverManager;
import java.sql.SQLException;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;
import java.util.Properties;

import javax.sql.DataSource;

import org.ow2.xplus.deployer.PersistenceXMLParser;
import org.ow2.xplus.logger.LogFactory;
import org.ow2.xplus.logger.Logger;

public class DriverManagerDataSource implements DataSource {
	
	private String driverClass;
	
	private String url;
	
	private String userName;
	
	private String password;
	
	private List<Connection> pool = new ArrayList<Connection>();
	
	private int maxPoolSize = 50000;
	
	private Integer isolation = Connection.TRANSACTION_READ_COMMITTED;

	private boolean autocommit;
	
	private Logger logger = LogFactory.getLogger(DriverManagerDataSource.class);
	
	public DriverManagerDataSource(Properties props){
		driverClass = props.getProperty(PersistenceXMLParser.PERSISTENCE_CONNECTION_DRIVERCLASS);
		
		if (driverClass==null) {
			logger.error("no JDBC Driver class was specified.");
		}
		else {
			try {
				Class.forName(driverClass);
			}
			catch (Exception e) {
				e.printStackTrace();
			}
		}

		url = props.getProperty(PersistenceXMLParser.PERSISTENCE_CONNECTION_URL);
		if (url==null) {
			logger.error("JDBC URL was not specified.");
		}
		
		userName = props.getProperty(PersistenceXMLParser.PERSISTENCE_CONNECTION_USER);
		password = props.getProperty(PersistenceXMLParser.PERSISTENCE_CONNECTION_PASSWORD);

	}

	public Connection getConnection() throws SQLException {
		synchronized (pool) {
			if ( !pool.isEmpty() ) {
				int last = pool.size() - 1;
				Connection pooled = (Connection) pool.remove(last);
				if (isolation!=null) pooled.setTransactionIsolation( isolation.intValue() );
				if ( pooled.getAutoCommit()!=autocommit ) pooled.setAutoCommit(autocommit);
				return pooled;
			}
		}
		Connection conn = DriverManager.getConnection(url, userName, password);
		if (isolation!=null) conn.setTransactionIsolation( isolation.intValue() );
		if ( conn.getAutoCommit()!=autocommit ) conn.setAutoCommit(autocommit);
		return conn;
	}

	public Connection getConnection(String username, String pass) throws SQLException {
		synchronized (pool) {
			if ( !pool.isEmpty() ) {
				int last = pool.size() - 1;
				Connection pooled = (Connection) pool.remove(last);
				if (isolation!=null) pooled.setTransactionIsolation( isolation.intValue() );
				if ( pooled.getAutoCommit()!=autocommit ) pooled.setAutoCommit(autocommit);
				return pooled;
			}
		}
		Connection conn = DriverManager.getConnection(url, username, pass);
		if (isolation!=null) conn.setTransactionIsolation( isolation.intValue() );
		if ( conn.getAutoCommit()!=autocommit ) conn.setAutoCommit(autocommit);
		return conn;
	}
	
	public void closeConnection(Connection conn) throws SQLException {
		synchronized (pool) {
			int currentSize = pool.size();
			if ( currentSize < maxPoolSize ) {
				pool.add(conn);
				return;
			}
		}
		conn.close();
	}

	public void close() {
		Iterator iter = pool.iterator();
		while ( iter.hasNext() ) {
			try {
				( (Connection) iter.next() ).close();
			}
			catch (SQLException sqle) {
				sqle.printStackTrace();
			}
		}
		pool.clear();

	}

	public PrintWriter getLogWriter() throws SQLException {
		// TODO Auto-generated method stub
		return null;
	}

	public void setLogWriter(PrintWriter out) throws SQLException {
		// TODO Auto-generated method stub
		
	}

	public void setLoginTimeout(int seconds) throws SQLException {
		// TODO Auto-generated method stub
		
	}

	public int getLoginTimeout() throws SQLException {
		// TODO Auto-generated method stub
		return 0;
	}

}
