/* -------------------------------------------------------------------------*
 * PKUAS: Peking University Application Server
 * Copyright (C) 2006 Peking University
 *
 * This software is free; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * either version 2.1 or any later version.
 * 
 * This software is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this software; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 * ------------------------------------------------------------------------*/

package org.ow2.xplus.deployer;

import java.sql.SQLException;
import java.util.HashMap;
import java.util.Map;
import java.util.Properties;

import javax.persistence.spi.PersistenceUnitInfo;
import javax.sql.DataSource;

import org.ow2.xplus.descriptor.DatabaseDescriptor;
import org.ow2.xplus.exception.UnitDeployingException;
import org.ow2.xplus.logger.LogFactory;
import org.ow2.xplus.logger.Logger;
import org.ow2.xplus.mapping.MappingProcessor;
import org.ow2.xplus.metadata.MetaDataRepository;
import org.ow2.xplus.metadata.PersistenceInfo;
import org.ow2.xplus.setting.CacheConfigInfo;
import org.ow2.xplus.setting.Setting;
import org.ow2.xplus.transaction.TransactionFactory;

/**
 * Abstract implementation of the Deployer.
 * @author Lin Liang
 */

public abstract class AbstractDeployer implements Deployer {

    /**
     * the name of application directory
     */
    protected String appDir;

    /**
     * the name of deploy directory
     */
    protected String deployDir;

    protected Logger logger = LogFactory.getLogger(Deployer.class);

    /**
     * Properties of the persistence layer
     */
    protected static Properties properties = null;
    

    /**
     * persistence unit list Map<persistence unit name, persistence application
     * manager>
     */
    private static Map<String, PersistenceInfo> unitList = new HashMap<String, PersistenceInfo>();

    /**
     * default constructor
     */
    public AbstractDeployer() {
        Setting setting = new Setting();
        properties = setting.getProperties();

    }

    public void start() {
    }

    /**
     * deploy the specific persistence unit
     * @param persistenceUnitName the name of the persistence unit
     */
    public void deploy(String persistenceUnitName) {
        PersistenceInfo persistenceInfo = unitList.get(persistenceUnitName);
        deploy(persistenceInfo);
    }

    /**
     * deploy the specific persistence unit
     * @param persistenceInfo the info of the persistence unit
     */
    public void deploy(PersistenceUnitInfo persistenceInfo) {
        if (persistenceInfo == null) {
            throw new UnitDeployingException("unit doesn't existed", UnitDeployingException.UNIT_NOT_EXISTED_ERROR);
        }

        String persistenceUnitName = persistenceInfo.getPersistenceUnitName();

        PackageVistor vistor = new PackageVistor(persistenceInfo);
        vistor.travel();

        MappingProcessor processor = new MappingProcessor(persistenceInfo);
        processor.process();

        DatabaseDescriptor dbDescriptor = new DatabaseDescriptor(MetaDataRepository.getUnitMetaData(persistenceUnitName));

        String dbDialect = persistenceInfo.getProperties().getProperty(PersistenceXMLParser.PERSISTENCE_CONNECTION_DIALECT);
        DatabaseManager dbManager = new DatabaseManager(dbDialect);
        MetaDataRepository.getUnitMetaData(persistenceUnitName).setDatabaseDialect(dbDialect);
        DataSource dataSource = persistenceInfo.getJtaDataSource();
        if (dataSource == null)
            dataSource = persistenceInfo.getNonJtaDataSource();
        dbManager.setDataSource(dataSource);
        String dbName = "";
        try {
            String url = dataSource.getConnection().getMetaData().getURL();
            dbName = dbManager.getPlatform().getDatabaseName(url);
        } catch (SQLException e) {
        }
        MetaDataRepository.getUnitMetaData(persistenceUnitName).setDatabaseName(dbName);

        String dbSchema = dbDescriptor.toXML();
        dbManager.buildModel(dbSchema);

        boolean dropTablesFirst = true;
        boolean continueOnError = true;
        String createTableStrategy = persistenceInfo.getProperties().getProperty(PersistenceXMLParser.PERSISTENCE_CREATE_TABLE);
        if (createTableStrategy != null) {
            if (createTableStrategy.equalsIgnoreCase("CREATE_DROP")) {
                dropTablesFirst = true;
                continueOnError = true;
                dbManager.createTable(dbSchema, dropTablesFirst, continueOnError);
            } else if (createTableStrategy.equalsIgnoreCase("CREATE_ONLY")) {
                dropTablesFirst = false;
                continueOnError = true;
                dbManager.createTable(dbSchema, dropTablesFirst, continueOnError);
            }
        }
        MetaDataRepository.getUnitMetaData(persistenceUnitName).setDatabase(dbManager);
        String txFactoryName = persistenceInfo.getProperties().getProperty(
                PersistenceXMLParser.PERSISTENCE_TRANSACTION_MANAGER);
        if (txFactoryName == null) txFactoryName = "org.ow2.xplus.transaction.txmanager.XplusTxManager";
        MetaDataRepository.setTxFactory(new TransactionFactory(txFactoryName));

    }

    protected static Map<String, PersistenceInfo> getUnitList() {
        return unitList;
    }


}
