/* -------------------------------------------------------------------------*
 * PKUAS: Peking University Application Server
 * Copyright (C) 2006 Peking University
 *
 * This software is free; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * either version 2.1 or any later version.
 * 
 * This software is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this software; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 * ------------------------------------------------------------------------*/

package org.ow2.xplus.deployer;

import java.io.StringReader;
import java.io.StringWriter;

import javax.sql.DataSource;

import org.apache.ddlutils.Platform;
import org.apache.ddlutils.PlatformFactory;
import org.apache.ddlutils.io.DatabaseIO;
import org.apache.ddlutils.model.Database;

/**
 * @author Lin Liang
 */

public class DatabaseManager {
    /**
     * database platform
     */
    private Platform platform;

    /**
     * The database model.
     */
    private Database model;

    /**
     * default constructor
     * @param dbName the name of the database
     * @param dbSchema the xml schema of the database
     */
    public DatabaseManager(String dbName) {
        StringWriter writer = new StringWriter();
        platform = PlatformFactory.createNewPlatformInstance(dbName);
        platform.getSqlBuilder().setWriter(writer);
        if (platform.getPlatformInfo().isDelimitedIdentifiersSupported()) {
            platform.setDelimitedIdentifierModeOn(true);
        }
    }

    /**
     * build database model
     * @param dbSchema the xml schema of the database
     */
    public void buildModel(String dbSchema) {
        DatabaseIO dbIO = new DatabaseIO();
        dbIO.setUseInternalDtd(true);
        dbIO.setValidateXml(false);
        model = dbIO.read(new StringReader(dbSchema));
    }

    /**
     * Creates a new database from the given schema
     * @param dbSchema the xml schema of the database
     */
    public void createTable(String dbSchema, boolean dropTablesFirst, boolean continueOnError) {
        platform.setSqlCommentsOn(false);
        platform.createTables(model, dropTablesFirst, continueOnError);
    }

    public void setDataSource(DataSource dataSource) {
        platform.setDataSource(dataSource);
    }

    public Database getModel() {
        return model;
    }

    public Platform getPlatform() {
        return platform;
    }
}
