/* -------------------------------------------------------------------------*
 * PKUAS: Peking University Application Server
 * Copyright (C) 2006 Peking University
 *
 * This software is free; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * either version 2.1 or any later version.
 * 
 * This software is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this software; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 * ------------------------------------------------------------------------*/

package org.ow2.xplus.deployer;

import java.io.File;
import java.util.List;

import javax.persistence.Embeddable;
import javax.persistence.Entity;
import javax.persistence.MappedSuperclass;
import javax.persistence.spi.PersistenceUnitInfo;

import org.ow2.xplus.exception.UnitDeployingException;
import org.ow2.xplus.metadata.PersistenceInfo;

/**
 * Parse a JAR of any form (zip file, exploded directory, ...) and build the
 * persistence info
 * @author Lin Liang
 */

public class PackageVistor {

    private ClassLoader classLoader;

    private PersistenceInfo persistenceInfo;

    /**
     * constructor
     * @param persistenceInfo
     */
    public PackageVistor(PersistenceUnitInfo persistenceUnitInfo) {
        this.persistenceInfo = (PersistenceInfo) persistenceUnitInfo;

        // URL[] urls = {persistenceInfo.getPersistenceUnitRootUrl()};
        // ClassLoader currentClassLoader =
        // Thread.currentThread().getContextClassLoader();
        // classLoader = new EntityClassLoader(urls, currentClassLoader);
        // Thread.currentThread().setContextClassLoader(classLoader);
        classLoader = this.getClass().getClassLoader();
        persistenceInfo.setClassLoader(this.getClass().getClassLoader());
    }

    /**
     * travel through the package, load classes, resources
     */
    public void travel() {
        if (persistenceInfo.excludeUnlistedClasses()) {
            List<String> classNames = persistenceInfo.getManagedClassNames();
            for (String className : classNames) {
                if (!isClassPersistent(className)) {
                    // not Persistent Class
                }
            }
        } else {
            if (persistenceInfo.getPersistenceUnitRootUrl() != null) {
                File directory = new File(persistenceInfo.getPersistenceUnitRootUrl().getFile());
                loadClassesInDirectory(directory, null);
            }
        }
    }

    /**
     * load persistence classes in the directory
     * @param directory the directory of persistence classes
     * @param prefix the prefix name of class file
     */
    private void loadClassesInDirectory(File directory, String prefix) {
        File[] files = directory.listFiles();
        prefix = prefix == null ? "" : prefix + ".";
        for (File file : files) {
            if (!file.isDirectory()) {
                String entityName = file.getName();
                if (entityName.endsWith(".class")) {
                    entityName = entityName.substring(0, entityName.length() - ".class".length()).replace('/', '.');
                    loadClass(prefix + entityName);
                }
            } else {
                loadClassesInDirectory(file, prefix + file.getName());
            }
        }
    }

    private void loadClass(String entityName) {
        if (isClassPersistent(entityName)) {
            persistenceInfo.addManagedClassName(entityName);
        }
    }

    /**
     * Check whether the class with the given name is persistent. A class is
     * persistent if it is annotated with
     * @Entity,
     * @Embeddable or
     * @MappedSuperclass
     * @return whether the class with the given name is persistent
     */
    @SuppressWarnings("unchecked")
    private boolean isClassPersistent(String className) {
        Class entityClass = null;
        try {
            entityClass = classLoader.loadClass(className);
        } catch (ClassNotFoundException cnfe) {
            UnitDeployingException ex = new UnitDeployingException(cnfe, UnitDeployingException.PERSISTENCE_CLASS_NOT_FOUND_ERROR);
            ex.showMessage();
        }

        if (entityClass.isAnnotationPresent(Entity.class) || entityClass.isAnnotationPresent(Embeddable.class)
                || entityClass.isAnnotationPresent(MappedSuperclass.class)) {
            return true;
        }
        return false;
    }

}
