/* -------------------------------------------------------------------------*
 * PKUAS: Peking University Application Server
 * Copyright (C) 2006 Peking University
 *
 * This software is free; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * either version 2.1 or any later version.
 * 
 * This software is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this software; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 * ------------------------------------------------------------------------*/

package org.ow2.xplus.deployer;

import java.io.File;
import java.io.IOException;
import java.util.List;

import org.ow2.xplus.exception.UnitDeployingException;
import org.ow2.xplus.metadata.PersistenceInfo;
import org.ow2.xplus.setting.CacheConfigInfo;
import org.ow2.xplus.setting.Setting;
import org.ow2.xplus.util.JarMaker;

/**
 * Standalone Deployer for persistece package
 * @author Lin Liang
 */

public class StandaloneDeployer extends AbstractDeployer {

    public StandaloneDeployer() {
        super();
    }

    /**
     * read properties from setting file
     */
    @Override
    public void start() {
        appDir = properties.getProperty(Setting.APP_DIR);
        deployDir = properties.getProperty(Setting.DEPLOY_DIR);
        deployDir(appDir);
    }

    /**
     * deploy a directory of persistence application
     * @param appDir a directory containing persistene file
     */
    private void deployDir(String appDir) {
        // Read applications from application directory, and load them.
        File dir = new File(appDir);
        File[] appjars = dir.listFiles(new java.io.FileFilter() {
            public boolean accept(File pathname) {
                String name = pathname.getName();
                return ((name.endsWith(".jar")) || (name.endsWith(".ear")) || (name.endsWith(".war")));
            }
        });

        // deploying applications
        for (int i = 0; appjars != null && i < appjars.length; i++) {
            File appjar = appjars[i];
            unpack(appjar.getName());
            install(appjar.getName());
        }
    }

    /**
     * intall the persistence package
     * @param fileName the application package's filename(full name without
     *        path)
     */
    protected void install(String fileName) {

        String appPath = deployDir + File.separator + fileName;

        ApplicationManager appManager = new ApplicationManager(appPath);
        appManager.parsePersistenceUnit();
        List<PersistenceInfo> persistenceInfos = appManager.getPersistenceInfos();

        if (persistenceInfos != null) {
            for (PersistenceInfo info : persistenceInfos)
                getUnitList().put(info.getPersistenceUnitName(), info);
        }
    }

    /**
     * unpack the app archive.
     * @param p_fileName : the jar file name, e.g., test.jar
     */
    private void unpack(String fileName) {
        String sourcePath = appDir + File.separator + fileName;
        String targetPath = deployDir + File.separator + fileName;
        File targetFile = new File(targetPath);

        try {
            // log.debug("Unpack all the jars in " + sourcePath);
            targetFile.mkdirs();
            JarMaker.unpackJar(sourcePath, targetPath);
        } catch (IOException ioe) {
            UnitDeployingException ex = new UnitDeployingException(ioe, UnitDeployingException.UNPACKING_ERROR);
            ex.showMessage();
        }
    }
    
}
