package org.ow2.xplus.deployer;

import java.net.URL;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import javax.persistence.Embeddable;
import javax.persistence.Entity;
import javax.persistence.EntityManagerFactory;
import javax.persistence.MappedSuperclass;
import javax.persistence.spi.PersistenceProvider;

import pku.as.application.EJBClassLoader;
import pku.as.application.InjectionDeployer;
import pku.as.metadata.ApplicationInfo;
import org.ow2.xplus.XplusPersistence;
import org.ow2.xplus.metadata.PersistenceInfo;

public class XplusPersistenceDeployer implements InjectionDeployer {

	public static final String PERSISTENCE_XML_FILENAME = "META-INF/persistence.xml";
	
	public void deploy(ApplicationInfo appInfo, String moduleName, EJBClassLoader ejbCL, List classList) throws Throwable {
		
		/** Mapping: PersistenceUnit -> EntityManagerFactory*/
		Map<String, EntityManagerFactory>  persistenceUnitMap = new HashMap<String, EntityManagerFactory> ();
		
		URL persistenceUrl = ejbCL.findResource(PERSISTENCE_XML_FILENAME);
		
		if (persistenceUrl != null) {
			PersistenceXMLParser parser = new PersistenceXMLParser();
			
			parser.setXmlFileStream(persistenceUrl.openStream());
			
			parser.parse();
			List<PersistenceInfo> infos = parser.getPersistenceInfos();

			ClassLoader cl = Thread.currentThread().getContextClassLoader();
			Thread.currentThread().setContextClassLoader(ejbCL);

			for (PersistenceInfo info : infos) {
				String className = info.getPersistenceProviderClassName();
				PersistenceProvider persistence = null;
				if (className != null) {
					try {
						persistence = (PersistenceProvider) Class.forName(
								className).newInstance();
					} catch (InstantiationException e) {
						e.printStackTrace();
					} catch (IllegalAccessException e) {
						e.printStackTrace();
					} catch (ClassNotFoundException e) {
						e.printStackTrace();
					}
				}
				if (classList != null) {
					for (Object clazz : classList){
						if (((Class)clazz).isAnnotationPresent(Entity.class)
								|| ((Class)clazz).isAnnotationPresent(Embeddable.class)
								|| ((Class)clazz).isAnnotationPresent(MappedSuperclass.class))
							info.addManagedClassName(((Class)clazz).getCanonicalName());
					}
				}
				
				if (persistence == null)
					persistence = new XplusPersistence();

				EntityManagerFactory emf = persistence
						.createContainerEntityManagerFactory(info, null);
				String unitName = info.getPersistenceUnitName();
				
				persistenceUnitMap.put(unitName, emf);

			}

			Thread.currentThread().setContextClassLoader(cl);
		}
		appInfo.addPersistenceUnitMap( moduleName , persistenceUnitMap ) ;
	}

}
