/* -------------------------------------------------------------------------*
 * PKUAS: Peking University Application Server
 * Copyright (C) 2006 Peking University
 *
 * This software is free; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * either version 2.1 or any later version.
 * 
 * This software is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this software; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 * ------------------------------------------------------------------------*/

package org.ow2.xplus.descriptor;

import org.jdom.Element;

import org.ow2.xplus.metadata.relational.RdbColumn;

/**
 * descriptor used to generate column schema
 * @author Lin Liang
 */

public class ColumnDescriptor extends AbstractDescriptor {

    private RdbColumn column;

    public ColumnDescriptor(RdbColumn column) {
        this.column = column;
    }

    /**
     * append the column description to table element
     * @param tableElem the element for a table
     */
    public void append(Element tableElem) {
        // build column element
        Element columnElem = new Element(DescriptorTag.COLUMN);
        columnElem.setAttribute(DescriptorTag.NAME, column.getName().toUpperCase());
        String primaryKey = column.isPrimaryKey() ? "true" : "false";
        columnElem.setAttribute(DescriptorTag.PRIMARYKEY, primaryKey);
        String required = column.isPrimaryKey() ? "true" : "false";
        columnElem.setAttribute(DescriptorTag.REQUIRED, required);
        String type = column.getSqlType();
        columnElem.setAttribute(DescriptorTag.TYPE, type);
        String size = column.getLength().toString();
        columnElem.setAttribute(DescriptorTag.SIZE, size);
        String defaultValue = column.getDefaultValue().toString();
        String autoIncrement = column.isAutoIncrement() ? "true" : "false";
        columnElem.setAttribute(DescriptorTag.AUTOINCREMENT, autoIncrement);
        if (!defaultValue.equals(""))
            columnElem.setAttribute(DescriptorTag.DEFAULTVALUE, defaultValue);

        // add to corresponding table element
        String tableName = column.getTableName();
        if (!tableName.equals(tableElem.getAttribute(DescriptorTag.NAME).getValue())) {
            Element databaseElem = tableElem.getParentElement();
            Element secTableElem = getTableElement(databaseElem, tableName);
            secTableElem.addContent(columnElem);
        } else {
            tableElem.addContent(columnElem);
        }
    }

}
