/* -------------------------------------------------------------------------*
 * PKUAS: Peking University Application Server
 * Copyright (C) 2006 Peking University
 *
 * This software is free; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * either version 2.1 or any later version.
 * 
 * This software is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this software; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 * ------------------------------------------------------------------------*/

package org.ow2.xplus.descriptor;

import org.jdom.Element;

import org.ow2.xplus.metadata.relational.RdbColumn;
import org.ow2.xplus.metadata.relational.RdbUniqueConstraint;

/**
 * descriptor used to generate constraint
 * @author Lin Liang
 */

public class ConstraintDescriptor extends AbstractDescriptor {

    private RdbUniqueConstraint unique = null;

    private RdbColumn column = null;

    public ConstraintDescriptor(RdbUniqueConstraint unique) {
        this.unique = unique;
    }

    public ConstraintDescriptor(RdbColumn column) {
        this.column = column;
    }

    /**
     * append the constraint description to table element
     * @param tableElem the element for a table
     */
    public void append(Element tableElem) {
        Element constraintElem = new Element(DescriptorTag.UNIQUE);
        if (unique != null) {
            for (String columnName : unique.getUniqueColumns()) {
                Element uniqueColumnElem = new Element(DescriptorTag.UNIQUECOLUMN);
                uniqueColumnElem.setAttribute(DescriptorTag.NAME, columnName);
                constraintElem.addContent(uniqueColumnElem);
            }
        }

        if (column != null) {
            Element uniqueColumnElem = new Element(DescriptorTag.UNIQUECOLUMN);
            uniqueColumnElem.setAttribute(DescriptorTag.NAME, column.getName());
            constraintElem.addContent(uniqueColumnElem);
        }

        tableElem.addContent(constraintElem);
    }

}
