/* -------------------------------------------------------------------------*
 * PKUAS: Peking University Application Server
 * Copyright (C) 2006 Peking University
 *
 * This software is free; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * either version 2.1 or any later version.
 * 
 * This software is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this software; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 * ------------------------------------------------------------------------*/

package org.ow2.xplus.descriptor;

import org.jdom.Document;
import org.jdom.Element;
import org.jdom.output.XMLOutputter;

import org.ow2.xplus.metadata.UnitMetaData;
import org.ow2.xplus.metadata.relational.RdbTable;

/**
 * descriptor used to generate database schema
 * @author Lin Liang
 */

public class DatabaseDescriptor extends AbstractDescriptor {

    /**
     * the metadata for persistence unit
     */
    private UnitMetaData unitMetaData;

    public DatabaseDescriptor(UnitMetaData unitMetaData) {
        this.unitMetaData = unitMetaData;
    }

    /**
     * get database schema in xml format
     * @return database schema in xml format
     */
    public String toXML() {
        Element databaseElem = new Element(DescriptorTag.DATABASE);
        String databaseName = unitMetaData.getDatabaseName();
        databaseElem.setAttribute(DescriptorTag.NAME, databaseName);

        for (RdbTable table : unitMetaData.getTables()) {
            if (table.isExisted()) {
                TableDescriptor tableDescriptor = new TableDescriptor(table);
                tableDescriptor.append(databaseElem);
            }
        }

        Document doc = new Document(databaseElem);
        XMLOutputter XMLOut = new XMLOutputter();
        return XMLOut.outputString(doc);
    }

}
