/* -------------------------------------------------------------------------*
 * PKUAS: Peking University Application Server
 * Copyright (C) 2006 Peking University
 *
 * This software is free; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * either version 2.1 or any later version.
 * 
 * This software is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this software; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 * ------------------------------------------------------------------------*/

package org.ow2.xplus.descriptor;

import java.util.List;

import org.jdom.Element;

import org.ow2.xplus.metadata.relational.RdbForeignKey;
import org.ow2.xplus.metadata.relational.RdbRelation;
import org.ow2.xplus.metadata.relational.RdbRelationTable;

/**
 * a Descriptor used to generate relation schema
 * @author Lin Liang
 */

public class RelationDescriptor extends AbstractDescriptor {

    private RdbRelation relation;

    public RelationDescriptor(RdbRelation relation) {
        this.relation = relation;
    }

    public void append(Element databaseElem) {
        if (relation instanceof RdbForeignKey)
            appendForeignKey(databaseElem);

        if (relation instanceof RdbRelationTable)
            appendRelationTable(databaseElem);
    }

    private void appendForeignKey(Element tableElem) {
        RdbForeignKey foreignKey = (RdbForeignKey) relation;

        Element foreignKeyElem = null;
        for (Object elem : tableElem.getChildren(DescriptorTag.FOREIGNKEY)) {
            Element fkElem = (Element) elem;
            if (fkElem.getAttribute(DescriptorTag.FOREIGNTABLE).equals(foreignKey.getReferencedTableName())) {
                foreignKeyElem = fkElem;
                break;
            }
        }
        if (foreignKeyElem == null)
            foreignKeyElem = new Element(DescriptorTag.FOREIGNKEY);
        foreignKeyElem.setAttribute(DescriptorTag.FOREIGNTABLE, foreignKey.getReferencedTableName());

        List<String> joinColumnNames = foreignKey.getJoinColumnNames();
        List<String> referencedColumnNames = foreignKey.getReferencedColumnNames();

        for (int i = 0; i < joinColumnNames.size(); ++i) {
            Element referenceElem = new Element(DescriptorTag.REFERENCE);
            referenceElem.setAttribute(DescriptorTag.LOCAL, joinColumnNames.get(i).toUpperCase());
            referenceElem.setAttribute(DescriptorTag.FOREIGN, referencedColumnNames.get(i).toUpperCase());
            foreignKeyElem.addContent(referenceElem);
        }

        // add to corresponding table element
        String tableName = relation.getOwnerTableName();
        if (!tableName.equals(tableElem.getAttribute(DescriptorTag.NAME).getValue())) {
            Element databaseElem = tableElem.getParentElement();
            Element secTableElem = getTableElement(databaseElem, tableName);
            secTableElem.addContent(foreignKeyElem);
        } else {
            tableElem.addContent(foreignKeyElem);
        }
    }

    @SuppressWarnings("unchecked")
    private void appendRelationTable(Element tableElem) {
        RdbRelationTable relationTable = (RdbRelationTable) relation;

        if (relationTable.isInverse())
            return;

        String tableName = relationTable.getRelationTable().getName();
        Element databaseElem = tableElem.getParentElement();
        Element relationTableElem = getTableElement(databaseElem, tableName);

        List<Element> elements = relationTableElem.getChildren(DescriptorTag.UNIQUE);
        for (Element elem : elements) {
            elem.removeContent();
        }

        List<String> joinColumnNames = relationTable.getSourceColumnNames();
        List<String> referencedColumnNames = relationTable.getSourceRelationColumnNames();

        Element ownerForeignKeyElem = new Element(DescriptorTag.FOREIGNKEY);
        ownerForeignKeyElem.setAttribute(DescriptorTag.FOREIGNTABLE, relationTable.getOwnerTableName());
        for (int i = 0; i < joinColumnNames.size(); ++i) {
            Element referenceElem = new Element(DescriptorTag.REFERENCE);
            referenceElem.setAttribute(DescriptorTag.LOCAL, referencedColumnNames.get(i).toUpperCase());
            referenceElem.setAttribute(DescriptorTag.FOREIGN, joinColumnNames.get(i).toUpperCase());
            ownerForeignKeyElem.addContent(referenceElem);
        }
        relationTableElem.addContent(ownerForeignKeyElem);

        joinColumnNames = relationTable.getTargetColumnNames();
        referencedColumnNames = relationTable.getTargetRelationColumnNames();

        Element referencedForeignKeyElem = new Element(DescriptorTag.FOREIGNKEY);
        referencedForeignKeyElem.setAttribute(DescriptorTag.FOREIGNTABLE, relationTable.getReferencedTableName());
        for (int i = 0; i < joinColumnNames.size(); ++i) {
            Element referenceElem = new Element(DescriptorTag.REFERENCE);
            referenceElem.setAttribute(DescriptorTag.LOCAL, referencedColumnNames.get(i).toUpperCase());
            referenceElem.setAttribute(DescriptorTag.FOREIGN, joinColumnNames.get(i).toUpperCase());
            referencedForeignKeyElem.addContent(referenceElem);
        }
        relationTableElem.addContent(referencedForeignKeyElem);

    }
}
