/* -------------------------------------------------------------------------*
 * PKUAS: Peking University Application Server
 * Copyright (C) 2006 Peking University
 *
 * This software is free; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * either version 2.1 or any later version.
 * 
 * This software is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this software; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 * ------------------------------------------------------------------------*/

package org.ow2.xplus.descriptor;

import org.jdom.Element;

import org.ow2.xplus.metadata.relational.RdbColumn;
import org.ow2.xplus.metadata.relational.RdbRelation;
import org.ow2.xplus.metadata.relational.RdbTable;
import org.ow2.xplus.metadata.relational.RdbUniqueConstraint;

/**
 * a Descriptor used to generate table schema
 * @author Lin Liang
 */

public class TableDescriptor extends AbstractDescriptor {

    private RdbTable table;

    public TableDescriptor(RdbTable table) {
        this.table = table;
    }

    public void append(Element databaseElem) {
        Element tableElem = getTableElement(databaseElem, table.getCanonicalName());

        for (RdbColumn column : table.getColumns()) {
            ColumnDescriptor columnDescriptor = new ColumnDescriptor(column);
            columnDescriptor.append(tableElem);
            if (column.isUnique() && !column.isPrimaryKey()) {
                ConstraintDescriptor constraintDescriptor = new ConstraintDescriptor(column);
                constraintDescriptor.append(tableElem);
            }
        }

        for (RdbRelation relation : table.getRelations()) {
            RelationDescriptor relationDescriptor = new RelationDescriptor(relation);
            relationDescriptor.append(tableElem);
        }

        for (RdbUniqueConstraint unique : table.getUniqueConstraints()) {
            ConstraintDescriptor constraintDescriptor = new ConstraintDescriptor(unique);
            constraintDescriptor.append(tableElem);
        }

        //databaseElem.addContent(tableElem);
    }
}