/* -------------------------------------------------------------------------*
 * PKUAS: Peking University Application Server
 * Copyright (C) 2006 Peking University
 *
 * This software is free; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * either version 2.1 or any later version.
 * 
 * This software is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this software; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 * ------------------------------------------------------------------------*/

package org.ow2.xplus.mapping.annotation;

import java.lang.reflect.Method;

import javax.persistence.EntityListeners;
import javax.persistence.ExcludeSuperclassListeners;
import javax.persistence.PostLoad;
import javax.persistence.PostPersist;
import javax.persistence.PostRemove;
import javax.persistence.PostUpdate;
import javax.persistence.PrePersist;
import javax.persistence.PreRemove;
import javax.persistence.PreUpdate;

import org.ow2.xplus.callback.Callback;
import org.ow2.xplus.callback.ListenerCallback;
import org.ow2.xplus.callback.MethodCallback;
import org.ow2.xplus.metadata.UnitMetaData;
import org.ow2.xplus.metadata.callback.CallbackMetaData;
import org.ow2.xplus.metadata.object.ClassMetaData;

/**
 * @author Lin Liang
 */

public class CallbackProcessor {

    private UnitMetaData unit;

    private CallbackMetaData callbacks;

    public CallbackProcessor(UnitMetaData unit) {
        this.unit = unit;
    }

    public void process(Class entityClass) {
        callbacks = new CallbackMetaData(unit, entityClass);

        processExcludeSuperclassListeners(entityClass);
        processListenerCallback(entityClass);

        Method[] methods = entityClass.getDeclaredMethods();
        for (Method method : methods) {
            processMethodCallback(method);
        }

        ClassMetaData entityMetaData = unit.getClassMetaData(entityClass.getCanonicalName());
        entityMetaData.setCallbacks(callbacks);
    }

    @SuppressWarnings("unchecked")
    private void processExcludeSuperclassListeners(Class entityClass) {
        if (entityClass.isAnnotationPresent(ExcludeSuperclassListeners.class)) {
            callbacks.setExcludeSuperclassListeners(true);
        }
    }

    @SuppressWarnings("unchecked")
    private void processListenerCallback(Class entityClass) {
        if (entityClass.isAnnotationPresent(EntityListeners.class)) {
            EntityListeners listeners = (EntityListeners) entityClass.getAnnotation(EntityListeners.class);
            Class[] listenerClasses = listeners.value();

            for (Class listenerClass : listenerClasses) {
                Method[] methods = listenerClass.getDeclaredMethods();
                for (Method method : methods) {
                    processListenerMethodCallback(method, listenerClass);
                }
            }
        }
    }

    private void processListenerMethodCallback(Method method, Class listenerClass) {
        ListenerCallback callback = new ListenerCallback(method, listenerClass);

        addCallback(method, callback, false);
    }

    private void addCallback(Method method, Callback callback, boolean override) {
        if (method.isAnnotationPresent(PrePersist.class)) {
            callbacks.addPrePersistCallback(callback, override);
        }
        if (method.isAnnotationPresent(PostPersist.class)) {
            callbacks.addPostPersistCallback(callback, override);
        }
        if (method.isAnnotationPresent(PreRemove.class)) {
            callbacks.addPreRemoveCallback(callback, override);
        }
        if (method.isAnnotationPresent(PostRemove.class)) {
            callbacks.addPostRemoveCallback(callback, override);
        }
        if (method.isAnnotationPresent(PreUpdate.class)) {
            callbacks.addPreUpdateCallback(callback, override);
        }
        if (method.isAnnotationPresent(PostUpdate.class)) {
            callbacks.addPostUpdateCallback(callback, override);
        }
        if (method.isAnnotationPresent(PostLoad.class)) {
            callbacks.addPostLoadCallback(callback, override);
        }
    }

    private void processMethodCallback(Method method) {
        MethodCallback callback = new MethodCallback(method);

        //TODO: check override
        boolean override = true;
        //boolean override = false;
        //if (method.isAnnotationPresent(Override.class)) override = true;

        addCallback(method, callback, override);
    }
}
