/* -------------------------------------------------------------------------*
 * PKUAS: Peking University Application Server
 * Copyright (C) 2006 Peking University
 *
 * This software is free; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * either version 2.1 or any later version.
 * 
 * This software is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this software; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 * ------------------------------------------------------------------------*/

package org.ow2.xplus.mapping.annotation;

import java.lang.annotation.Annotation;

import javax.persistence.ColumnResult;
import javax.persistence.EntityResult;
import javax.persistence.FieldResult;
import javax.persistence.NamedNativeQueries;
import javax.persistence.NamedNativeQuery;
import javax.persistence.NamedQueries;
import javax.persistence.NamedQuery;
import javax.persistence.SqlResultSetMapping;
import javax.persistence.SqlResultSetMappings;

import org.ow2.xplus.metadata.UnitMetaData;
import org.ow2.xplus.metadata.query.ColumnMapping;
import org.ow2.xplus.metadata.query.EntityMapping;
import org.ow2.xplus.metadata.query.FieldMapping;
import org.ow2.xplus.metadata.query.NamedNativeQueryMapping;
import org.ow2.xplus.metadata.query.NamedQueryMapping;
import org.ow2.xplus.metadata.query.ResultSetMapping;

/**
 * Query Processor
 * @author Lin Liang
 */

public class QueryProcessor {

    private UnitMetaData unit;

    public QueryProcessor(UnitMetaData unit) {
        this.unit = unit;
    }

    public void process(Class entityClass) {
        Annotation[] anns = entityClass.getAnnotations();

        for (Annotation ann : anns) {
            if (ann.annotationType() == NamedQuery.class) {
                processNamedQuery((NamedQuery) ann);
            }
            if (ann.annotationType() == NamedQueries.class) {
                processNamedQueries((NamedQueries) ann);
            }
            if (ann.annotationType() == NamedNativeQuery.class) {
                processNamedNativeQuery((NamedNativeQuery) ann);
            }
            if (ann.annotationType() == NamedNativeQueries.class) {
                processNamedNativeQueries((NamedNativeQueries) ann);
            }
            if (ann.annotationType() == SqlResultSetMappings.class) {
                processSqlResultSetMappings((SqlResultSetMappings) ann);
            }
            if (ann.annotationType() == SqlResultSetMapping.class) {
                processSqlResultSetMapping((SqlResultSetMapping) ann);
            }
        }
    }

    private void processSqlResultSetMappings(SqlResultSetMappings sqlResultSetMappingsAnn) {
        for (SqlResultSetMapping sqlResultSetMappingAnn : sqlResultSetMappingsAnn.value()) {
            processSqlResultSetMapping(sqlResultSetMappingAnn);
        }
    }

    private void processSqlResultSetMapping(SqlResultSetMapping ann) {
        ResultSetMapping resultSetMapping = new ResultSetMapping(ann.name());

        for (EntityResult entity : ann.entities()) {
            Class entityClass = entity.entityClass();
            EntityMapping entityResult = new EntityMapping(entityClass);
            entityResult.setDiscriminatorColumn(entity.discriminatorColumn());

            for (FieldResult field : entity.fields()) {
                FieldMapping fieldResult = new FieldMapping(field.name(), field.column());
                entityResult.addFieldMapping(fieldResult);
            }

            resultSetMapping.addEntityMapping(entityResult);
        }

        for (ColumnResult column : ann.columns()) {
            ColumnMapping columnResult = new ColumnMapping(column.name());

            resultSetMapping.addColumnMapping(columnResult);
        }

        unit.addResultSet(resultSetMapping.getName(), resultSetMapping);

    }

    private void processNamedNativeQueries(NamedNativeQueries namedNativeQueriesAnn) {
        for (NamedNativeQuery namedNativeQueryAnn : namedNativeQueriesAnn.value()) {
            processNamedNativeQuery(namedNativeQueryAnn);
        }
    }

    private void processNamedNativeQuery(NamedNativeQuery namedNativeQueryAnn) {
        NamedNativeQueryMapping mapping = new NamedNativeQueryMapping();
        mapping.setName(namedNativeQueryAnn.name());
        mapping.setQuery(namedNativeQueryAnn.query());
        mapping.setResultClass(namedNativeQueryAnn.resultClass());
        mapping.setResultSetMapping(namedNativeQueryAnn.resultSetMapping());
        unit.addNamedNativeQuery(mapping.getName(), mapping);
    }

    private void processNamedQueries(NamedQueries namedQueriesAnn) {
        for (NamedQuery namedQueryAnn : namedQueriesAnn.value()) {
            processNamedQuery(namedQueryAnn);
        }
    }

    private void processNamedQuery(NamedQuery namedQueryAnn) {
        NamedQueryMapping mapping = new NamedQueryMapping();
        mapping.setName(namedQueryAnn.name());
        mapping.setQuery(namedQueryAnn.query());
        unit.addNamedQuery(mapping.getName(), mapping);
    }
}
