/* -------------------------------------------------------------------------*
 * PKUAS: Peking University Application Server
 * Copyright (C) 2006 Peking University
 *
 * This software is free; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * either version 2.1 or any later version.
 * 
 * This software is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this software; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 * ------------------------------------------------------------------------*/

package org.ow2.xplus.metadata;

import java.util.HashMap;
import java.util.Map;

import org.ow2.xplus.transaction.TransactionFactory;

/**
 * Repository for all persistence information
 * @author Lin Liang
 */

public class MetaDataRepository {

    /**
     * all persistence unit meta data map<the name of the persistence unit,
     * unit metameta>
     */
    private static Map<String, UnitMetaData> unitMetaDatas = new HashMap<String, UnitMetaData>();

    /**
     * factory for Transaction instances. Concrete implementations are specified
     * by pkuas.transaction.manager
     */
    private static TransactionFactory txFactory = null;

    /**
     * singleton pattern
     */
    static private MetaDataRepository repository = new MetaDataRepository();

    private MetaDataRepository() {
    }

    public static MetaDataRepository getRepository() {
        return repository;
    }

    public static UnitMetaData getUnitMetaData(String unitName) {
        UnitMetaData unitMetaData = unitMetaDatas.get(unitName);
        if (unitMetaData == null) {
            unitMetaData = new UnitMetaData(unitName);
            unitMetaDatas.put(unitName, unitMetaData);
        }
        return unitMetaData;
    }

    public static TransactionFactory getTxFactory() {
        return txFactory;
    }

    public static void setTxFactory(TransactionFactory factory) {
        txFactory = factory;
    }

}
