/* -------------------------------------------------------------------------*
 * PKUAS: Peking University Application Server
 * Copyright (C) 2006 Peking University
 *
 * This software is free; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * either version 2.1 or any later version.
 * 
 * This software is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this software; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 * ------------------------------------------------------------------------*/

package org.ow2.xplus.metadata.callback;

import java.util.ArrayList;
import java.util.List;

import org.ow2.xplus.callback.Callback;
import org.ow2.xplus.callback.ListenerCallback;
import org.ow2.xplus.callback.MethodCallback;
import org.ow2.xplus.metadata.UnitMetaData;
import org.ow2.xplus.metadata.object.ClassMetaData;

/**
 * @author Lin Liang
 */

public class CallbackMetaData {

    private List<Callback> prePersistListenerCallbacks = new ArrayList<Callback>();

    private List<Callback> prePersistMethodCallbacks = new ArrayList<Callback>();

    private List<Callback> postPersistListenerCallbacks = new ArrayList<Callback>();

    private List<Callback> postPersistMethodCallbacks = new ArrayList<Callback>();

    private List<Callback> preRemoveListenerCallbacks = new ArrayList<Callback>();

    private List<Callback> preRemoveMethodCallbacks = new ArrayList<Callback>();

    private List<Callback> postRemoveListenerCallbacks = new ArrayList<Callback>();

    private List<Callback> postRemoveMethodCallbacks = new ArrayList<Callback>();

    private List<Callback> preUpdateListenerCallbacks = new ArrayList<Callback>();

    private List<Callback> preUpdateMethodCallbacks = new ArrayList<Callback>();

    private List<Callback> postUpdateListenerCallbacks = new ArrayList<Callback>();

    private List<Callback> postUpdateMethodCallbacks = new ArrayList<Callback>();

    private List<Callback> postLoadListenerCallbacks = new ArrayList<Callback>();

    private List<Callback> postLoadMethodCallbacks = new ArrayList<Callback>();

    private boolean excludeSuperclassListeners = false;

    private boolean excludeDefaultListeners = false;

    private List<String> prePersistOverrideMethods = new ArrayList<String>();

    private List<String> postPersistOverrideMethods = new ArrayList<String>();

    private List<String> preRemoveOverrideMethods = new ArrayList<String>();

    private List<String> postRemoveOverrideMethods = new ArrayList<String>();

    private List<String> preUpdateOverrideMethods = new ArrayList<String>();

    private List<String> postUpdateOverrideMethods = new ArrayList<String>();

    private List<String> postLoadOverrideMethods = new ArrayList<String>();

    private UnitMetaData unit;

    private Class entityClass;

    public CallbackMetaData(UnitMetaData unit, Class entityClass) {
        this.unit = unit;
        this.entityClass = entityClass;
    }

    // PostLoad
    public void addPostLoadCallback(Callback postLoad, boolean override) {
        if (override)
            handlePostLoadOverride(postLoad);
        if (postLoad instanceof ListenerCallback)
            postLoadListenerCallbacks.add(postLoad);
        else
            postLoadMethodCallbacks.add(postLoad);
    }

    private void handlePostLoadOverride(Callback callback) {
        if (callback == null)
            return;
        if (callback instanceof MethodCallback) {
            postLoadOverrideMethods.add(((MethodCallback) callback).getMethodName());
        }
    }

    private List<Callback> getPostLoadListenerCallbacks() {
        if (excludeSuperclassListeners) {
            return postLoadListenerCallbacks;
        }
        List<Callback> callbacks = new ArrayList<Callback>();

        ClassMetaData classMetaData = unit.getClassMetaData(entityClass.getCanonicalName());
        Class parentEntity = classMetaData.getParent();
        if (parentEntity != null) {
            ClassMetaData parentMetaData = unit.getClassMetaData(parentEntity.getCanonicalName());
            callbacks.addAll(parentMetaData.getCallbacks().getPostLoadListenerCallbacks());
        }

        callbacks.addAll(postLoadListenerCallbacks);

        return callbacks;
    }

    private List<Callback> getPostLoadMethodCallbacks() {
        if (excludeSuperclassListeners) {
            return postLoadMethodCallbacks;
        }
        List<Callback> callbacks = new ArrayList<Callback>();

        ClassMetaData classMetaData = unit.getClassMetaData(entityClass.getCanonicalName());
        Class parentEntity = classMetaData.getParent();
        if (parentEntity != null) {
            ClassMetaData parentMetaData = unit.getClassMetaData(parentEntity.getCanonicalName());
            for (Callback callback : parentMetaData.getCallbacks().getPostLoadMethodCallbacks()) {
                if (!postLoadOverrideMethods.contains(((MethodCallback) callback).getMethodName()))
                    callbacks.add(callback);
            }
        }

        callbacks.addAll(postLoadMethodCallbacks);

        return callbacks;
    }

    public List<Callback> getPostLoadCallbacks() {
        List<Callback> callbacks = new ArrayList<Callback>();
        callbacks.addAll(getPostLoadListenerCallbacks());
        callbacks.addAll(getPostLoadMethodCallbacks());
        return callbacks;
    }

    // PostPersist
    public void addPostPersistCallback(Callback postPersist, boolean override) {
        if (override)
            handlePersistOverride(postPersist);
        if (postPersist instanceof ListenerCallback)
            postPersistListenerCallbacks.add(postPersist);
        else
            postPersistMethodCallbacks.add(postPersist);
    }

    private void handlePersistOverride(Callback callback) {
        if (callback == null)
            return;
        if (callback instanceof MethodCallback) {
            postPersistOverrideMethods.add(((MethodCallback) callback).getMethodName());
        }
    }

    private List<Callback> getPostPersistListenerCallbacks() {
        if (excludeSuperclassListeners) {
            return postPersistListenerCallbacks;
        }
        List<Callback> callbacks = new ArrayList<Callback>();

        ClassMetaData classMetaData = unit.getClassMetaData(entityClass.getCanonicalName());
        Class parentEntity = classMetaData.getParent();
        if (parentEntity != null) {
            ClassMetaData parentMetaData = unit.getClassMetaData(parentEntity.getCanonicalName());
            callbacks.addAll(parentMetaData.getCallbacks().getPostPersistListenerCallbacks());
        }

        callbacks.addAll(postPersistListenerCallbacks);

        return callbacks;
    }

    private List<Callback> getPostPersistMethodCallbacks() {
        if (excludeSuperclassListeners) {
            return postPersistMethodCallbacks;
        }
        List<Callback> callbacks = new ArrayList<Callback>();

        ClassMetaData classMetaData = unit.findClassMetaData(entityClass.getCanonicalName());
        Class parentEntity = classMetaData.getParent();
        if (parentEntity != null) {
            ClassMetaData parentMetaData = unit.findClassMetaData(parentEntity.getCanonicalName());
            for (Callback callback : parentMetaData.getCallbacks().getPostPersistMethodCallbacks()) {
                if (!postLoadOverrideMethods.contains(((MethodCallback) callback).getMethodName()))
                    callbacks.add(callback);
            }
        }

        callbacks.addAll(postPersistMethodCallbacks);

        return callbacks;
    }

    public List<Callback> getPostPersistCallbacks() {
        List<Callback> callbacks = new ArrayList<Callback>();
        callbacks.addAll(getPostPersistListenerCallbacks());
        callbacks.addAll(getPostPersistMethodCallbacks());
        return callbacks;
    }

    // Post Remove
    public void addPostRemoveCallback(Callback postRemove, boolean override) {
        if (override)
            handlePostRemoveOverride(postRemove);
        if (postRemove instanceof ListenerCallback)
            postRemoveListenerCallbacks.add(postRemove);
        else
            postRemoveMethodCallbacks.add(postRemove);
    }

    private void handlePostRemoveOverride(Callback callback) {
        if (callback == null)
            return;
        if (callback instanceof MethodCallback) {
            postRemoveOverrideMethods.add(((MethodCallback) callback).getMethodName());
        }
    }

    private List<Callback> getPostRemoveListenerCallbacks() {
        if (excludeSuperclassListeners) {
            return postRemoveListenerCallbacks;
        }
        List<Callback> callbacks = new ArrayList<Callback>();

        ClassMetaData classMetaData = unit.findClassMetaData(entityClass.getCanonicalName());
        Class parentEntity = classMetaData.getParent();
        if (parentEntity != null) {
            ClassMetaData parentMetaData = unit.findClassMetaData(parentEntity.getCanonicalName());
            callbacks.addAll(parentMetaData.getCallbacks().getPostRemoveListenerCallbacks());
        }
        callbacks.addAll(postRemoveListenerCallbacks);
        return callbacks;
    }

    private List<Callback> getPostRemoveMethodCallbacks() {
        if (excludeSuperclassListeners) {
            return postRemoveMethodCallbacks;
        }
        List<Callback> callbacks = new ArrayList<Callback>();

        ClassMetaData classMetaData = unit.findClassMetaData(entityClass.getCanonicalName());
        Class parentEntity = classMetaData.getParent();
        if (parentEntity != null) {
            ClassMetaData parentMetaData = unit.findClassMetaData(parentEntity.getCanonicalName());
            for (Callback callback : parentMetaData.getCallbacks().getPostRemoveMethodCallbacks()) {
                if (!postRemoveOverrideMethods.contains(((MethodCallback) callback).getMethodName()))
                    callbacks.add(callback);
            }
        }

        callbacks.addAll(postRemoveMethodCallbacks);

        return callbacks;
    }

    public List<Callback> getPostRemoveCallbacks() {
        List<Callback> callbacks = new ArrayList<Callback>();
        callbacks.addAll(getPostRemoveListenerCallbacks());
        callbacks.addAll(getPostRemoveMethodCallbacks());
        return callbacks;
    }

    // Post Update
    public void addPostUpdateCallback(Callback postUpdate, boolean override) {
        if (override)
            handlePostUpdateOverride(postUpdate);
        if (postUpdate instanceof ListenerCallback)
            postUpdateListenerCallbacks.add(postUpdate);
        else
            postUpdateMethodCallbacks.add(postUpdate);
    }

    private void handlePostUpdateOverride(Callback callback) {
        if (callback == null)
            return;
        if (callback instanceof MethodCallback) {
            postUpdateOverrideMethods.add(((MethodCallback) callback).getMethodName());
        }
    }

    private List<Callback> getPostUpdateListenerCallbacks() {
        if (excludeSuperclassListeners) {
            return postUpdateListenerCallbacks;
        }
        List<Callback> callbacks = new ArrayList<Callback>();

        ClassMetaData classMetaData = unit.findClassMetaData(entityClass.getCanonicalName());
        Class parentEntity = classMetaData.getParent();
        if (parentEntity != null) {
            ClassMetaData parentMetaData = unit.findClassMetaData(parentEntity.getCanonicalName());
            callbacks.addAll(parentMetaData.getCallbacks().getPostUpdateListenerCallbacks());
        }
        callbacks.addAll(postUpdateListenerCallbacks);
        return callbacks;
    }

    private List<Callback> getPostUpdateMethodCallbacks() {
        if (excludeSuperclassListeners) {
            return postUpdateMethodCallbacks;
        }
        List<Callback> callbacks = new ArrayList<Callback>();

        ClassMetaData classMetaData = unit.findClassMetaData(entityClass.getCanonicalName());
        Class parentEntity = classMetaData.getParent();
        if (parentEntity != null) {
            ClassMetaData parentMetaData = unit.findClassMetaData(parentEntity.getCanonicalName());
            for (Callback callback : parentMetaData.getCallbacks().getPostUpdateMethodCallbacks()) {
                if (!postUpdateOverrideMethods.contains(((MethodCallback) callback).getMethodName()))
                    callbacks.add(callback);
            }
        }

        callbacks.addAll(postUpdateMethodCallbacks);

        return callbacks;
    }

    public List<Callback> getPostUpdateCallbacks() {
        List<Callback> callbacks = new ArrayList<Callback>();
        callbacks.addAll(getPostUpdateListenerCallbacks());
        callbacks.addAll(getPostUpdateMethodCallbacks());
        return callbacks;
    }

    // PrePersist
    public void addPrePersistCallback(Callback PrePersist, boolean override) {
        if (override)
            handlePrePersistOverride(PrePersist);
        if (PrePersist instanceof ListenerCallback)
            prePersistListenerCallbacks.add(PrePersist);
        else
            prePersistMethodCallbacks.add(PrePersist);
    }

    private void handlePrePersistOverride(Callback callback) {
        if (callback == null)
            return;
        if (callback instanceof MethodCallback) {
            prePersistOverrideMethods.add(((MethodCallback) callback).getMethodName());
        }
    }

    private List<Callback> getPrePersistListenerCallbacks() {
        if (excludeSuperclassListeners) {
            return prePersistListenerCallbacks;
        }
        List<Callback> callbacks = new ArrayList<Callback>();

        ClassMetaData classMetaData = unit.findClassMetaData(entityClass.getCanonicalName());
        Class parentEntity = classMetaData.getParent();
        if (parentEntity != null) {
            ClassMetaData parentMetaData = unit.findClassMetaData(parentEntity.getCanonicalName());
            callbacks.addAll(parentMetaData.getCallbacks().getPrePersistListenerCallbacks());
        }
        callbacks.addAll(prePersistListenerCallbacks);
        return callbacks;
    }

    private List<Callback> getPrePersistMethodCallbacks() {
        if (excludeSuperclassListeners) {
            return prePersistMethodCallbacks;
        }
        List<Callback> callbacks = new ArrayList<Callback>();

        ClassMetaData classMetaData = unit.findClassMetaData(entityClass.getCanonicalName());
        Class parentEntity = classMetaData.getParent();
        if (parentEntity != null) {
            ClassMetaData parentMetaData = unit.findClassMetaData(parentEntity.getCanonicalName());
            for (Callback callback : parentMetaData.getCallbacks().getPrePersistMethodCallbacks()) {
                if (!prePersistOverrideMethods.contains(((MethodCallback) callback).getMethodName()))
                    callbacks.add(callback);
            }
        }

        callbacks.addAll(prePersistMethodCallbacks);

        return callbacks;
    }

    public List<Callback> getPrePersistCallbacks() {
        List<Callback> callbacks = new ArrayList<Callback>();
        callbacks.addAll(getPrePersistListenerCallbacks());
        callbacks.addAll(getPrePersistMethodCallbacks());
        return callbacks;
    }

    // PreRemove
    public void addPreRemoveCallback(Callback PreRemove, boolean override) {
        if (override)
            handlePreRemoveOverride(PreRemove);
        if (PreRemove instanceof ListenerCallback)
            preRemoveListenerCallbacks.add(PreRemove);
        else
            preRemoveMethodCallbacks.add(PreRemove);
    }

    private void handlePreRemoveOverride(Callback callback) {
        if (callback == null)
            return;
        if (callback instanceof MethodCallback) {
            preRemoveOverrideMethods.add(((MethodCallback) callback).getMethodName());
        }
    }

    private List<Callback> getPreRemoveListenerCallbacks() {
        if (excludeSuperclassListeners) {
            return preRemoveListenerCallbacks;
        }
        List<Callback> callbacks = new ArrayList<Callback>();

        ClassMetaData classMetaData = unit.findClassMetaData(entityClass.getCanonicalName());
        Class parentEntity = classMetaData.getParent();
        if (parentEntity != null) {
            ClassMetaData parentMetaData = unit.findClassMetaData(parentEntity.getCanonicalName());
            callbacks.addAll(parentMetaData.getCallbacks().getPreRemoveListenerCallbacks());
        }
        callbacks.addAll(preRemoveListenerCallbacks);
        return callbacks;
    }

    private List<Callback> getPreRemoveMethodCallbacks() {
        if (excludeSuperclassListeners) {
            return preRemoveMethodCallbacks;
        }
        List<Callback> callbacks = new ArrayList<Callback>();

        ClassMetaData classMetaData = unit.findClassMetaData(entityClass.getCanonicalName());
        Class parentEntity = classMetaData.getParent();
        if (parentEntity != null) {
            ClassMetaData parentMetaData = unit.findClassMetaData(parentEntity.getCanonicalName());
            for (Callback callback : parentMetaData.getCallbacks().getPreRemoveMethodCallbacks()) {
                if (!preRemoveOverrideMethods.contains(((MethodCallback) callback).getMethodName()))
                    callbacks.add(callback);
            }
        }
        callbacks.addAll(preRemoveMethodCallbacks);
        return callbacks;
    }

    public List<Callback> getPreRemoveCallbacks() {
        List<Callback> callbacks = new ArrayList<Callback>();
        callbacks.addAll(getPreRemoveListenerCallbacks());
        callbacks.addAll(getPreRemoveMethodCallbacks());
        return callbacks;
    }

    // PreUpdate
    public void addPreUpdateCallback(Callback PreUpdate, boolean override) {
        if (override)
            handlePreUpdateOverride(PreUpdate);
        if (PreUpdate instanceof ListenerCallback)
            preUpdateListenerCallbacks.add(PreUpdate);
        else
            preUpdateMethodCallbacks.add(PreUpdate);
    }

    private void handlePreUpdateOverride(Callback callback) {
        if (callback == null)
            return;
        if (callback instanceof MethodCallback) {
            preUpdateOverrideMethods.add(((MethodCallback) callback).getMethodName());
        }
    }

    private List<Callback> getPreUpdateListenerCallbacks() {
        if (excludeSuperclassListeners) {
            return preUpdateListenerCallbacks;
        }
        List<Callback> callbacks = new ArrayList<Callback>();

        ClassMetaData classMetaData = unit.findClassMetaData(entityClass.getCanonicalName());
        Class parentEntity = classMetaData.getParent();
        if (parentEntity != null) {
            ClassMetaData parentMetaData = unit.findClassMetaData(parentEntity.getCanonicalName());
            callbacks.addAll(parentMetaData.getCallbacks().getPreUpdateListenerCallbacks());
        }
        callbacks.addAll(preUpdateListenerCallbacks);
        return callbacks;
    }

    private List<Callback> getPreUpdateMethodCallbacks() {
        if (excludeSuperclassListeners) {
            return preUpdateMethodCallbacks;
        }
        List<Callback> callbacks = new ArrayList<Callback>();

        ClassMetaData classMetaData = unit.findClassMetaData(entityClass.getCanonicalName());
        Class parentEntity = classMetaData.getParent();
        if (parentEntity != null) {
            ClassMetaData parentMetaData = unit.findClassMetaData(parentEntity.getCanonicalName());
            for (Callback callback : parentMetaData.getCallbacks().getPreUpdateMethodCallbacks()) {
                if (!preUpdateOverrideMethods.contains(((MethodCallback) callback).getMethodName()))
                    callbacks.add(callback);
            }
        }
        callbacks.addAll(preUpdateMethodCallbacks);
        return callbacks;
    }

    public List<Callback> getPreUpdateCallbacks() {
        List<Callback> callbacks = new ArrayList<Callback>();
        callbacks.addAll(getPreUpdateListenerCallbacks());
        callbacks.addAll(getPreUpdateMethodCallbacks());
        return callbacks;
    }

    public boolean isExcludeDefaultListeners() {
        return excludeDefaultListeners;
    }

    public void setExcludeDefaultListeners(boolean excludeDefaultListeners) {
        this.excludeDefaultListeners = excludeDefaultListeners;
    }

    public boolean isExcludeSuperclassListeners() {
        return excludeSuperclassListeners;
    }

    public void setExcludeSuperclassListeners(boolean excludeSuperclassListeners) {
        this.excludeSuperclassListeners = excludeSuperclassListeners;
    }

}
