/* -------------------------------------------------------------------------*
 * PKUAS: Peking University Application Server
 * Copyright (C) 2006 Peking University
 *
 * This software is free; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * either version 2.1 or any later version.
 * 
 * This software is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this software; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 * ------------------------------------------------------------------------*/

package org.ow2.xplus.metadata.object;

import java.util.ArrayList;
import java.util.List;

import javax.persistence.CascadeType;
import javax.persistence.FetchType;

/**
 * Abstract implement of relationship
 * @author Lin Liang
 */

public abstract class AbstractRelation implements Relationship {
    /**
     * The entity class that is the target of the association.
     */
    protected Class targetEntity;

    /**
     * The operations that must be cascaded to the target of the association.
     */
    protected List<CascadeType> cascade = new ArrayList<CascadeType>();

    /**
     * Whether the association should be lazily loaded or must be eagerly
     * fetched.
     */
    protected FetchType fetch = FetchType.EAGER;

    /**
     * The field that owns the relationship.
     */
    protected String mappedBy = "";

    /**
     * Whether the association is optional.
     */
    protected boolean optional = true;

    public Class getTargetEntity() {
        return targetEntity;
    }

    public List<CascadeType> getCascade() {
        return cascade;
    }

    public boolean isOptional() {
        return optional;
    }

    public String getMappedBy() {
        return mappedBy;
    }

    public FetchType getFetch() {
        return fetch;
    }

}
