/* -------------------------------------------------------------------------*
 * PKUAS: Peking University Application Server
 * Copyright (C) 2006 Peking University
 *
 * This software is free; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * either version 2.1 or any later version.
 * 
 * This software is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this software; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 * ------------------------------------------------------------------------*/

package org.ow2.xplus.metadata.object;

import javax.persistence.FetchType;
import javax.persistence.FlushModeType;

import org.ow2.xplus.operator.accessor.Accessor;

/**
 * the metadata of field in the entity class
 * @author Lin Liang
 */

public class FieldMetaData {
    /**
     * the class type of this field
     */
    private Class type;

    /**
     * the collection type of this field
     */
    private Class collectionType;

    /**
     * the name of the field
     */
    private String fieldName;

    /**
     * the metadata of value stored in this field
     */
    private ValueMetaData value;

    /**
     * the relation stored in this field
     */
    private Relationship relation;

    /**
     * Whether the association should be lazily loaded or must be eagerly
     * fetched.
     */
    private FetchType fetch = FetchType.EAGER;

    /**
     * flushMode type
     */
    private FlushModeType flushMode = FlushModeType.AUTO;

    /**
     * Whether the association is optional.
     */
    private boolean optional = true;

    /**
     * the accessor of the field data
     */
    private Accessor accessor;

    private boolean id = false;
     
    private boolean generatedID = true;

    public FieldMetaData(String fieldName) {
        this.fieldName = fieldName;
    }

    public ValueMetaData getValue() {
        return value;
    }

    public void setValue(ValueMetaData value) {
        this.value = value;
    }

    public Relationship getRelation() {
        return relation;
    }

    public void setRelation(Relationship relation) {
        this.relation = relation;
    }

    public String getFieldName() {
        return fieldName;
    }

    public void setFieldName(String fieldName) {
        this.fieldName = fieldName;
    }

    public FetchType getFetch() {
        return fetch;
    }

    public void setFetch(FetchType fetch) {
        this.fetch = fetch;
    }

    public boolean isOptional() {
        return optional;
    }

    public void setOptional(boolean optional) {
        this.optional = optional;
    }

    public FlushModeType getFlushMode() {
        return flushMode;
    }

    public void setFlushMode(FlushModeType flushMode) {
        this.flushMode = flushMode;
    }

    public Class getType() {
        return type;
    }

    public void setType(Class type) {
        this.type = type;
    }

    public boolean isId() {
        return id;
    }

    public void setId(boolean id) {
        this.id = id;
    }

    public Accessor getAccessor() {
        return accessor;
    }

    public void setAccessor(Accessor accessor) {
        this.accessor = accessor;
    }

    public Class getCollectionType() {
        return collectionType;
    }

    public void setCollectionType(Class collectionType) {
        this.collectionType = collectionType;
    }

	public boolean isGeneratedID() {
		return generatedID;
	}

	public void setGeneratedID(boolean generatedID) {
		this.generatedID = generatedID;
	}
}
