/* -------------------------------------------------------------------------*
 * PKUAS: Peking University Application Server
 * Copyright (C) 2006 Peking University
 *
 * This software is free; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * either version 2.1 or any later version.
 * 
 * This software is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this software; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 * ------------------------------------------------------------------------*/

package org.ow2.xplus.metadata.object;

import javax.persistence.CascadeType;
import javax.persistence.ManyToMany;

import org.ow2.xplus.mapping.annotation.Member;
import org.ow2.xplus.metadata.MetaDataRepository;
import org.ow2.xplus.metadata.relational.RdbRelation;
import org.ow2.xplus.metadata.relational.RdbRelationTable;
import org.ow2.xplus.metadata.relational.RdbTable;

/**
 * @author Lin Liang
 */

public class ManyToManyRelation extends AbstractRelation {

    /**
     * constructor
     * @param member member this relation associated with
     * @param manyToMany ManyToMany annotation
     */
    public ManyToManyRelation(Member member, ManyToMany manyToMany) {
        targetEntity = manyToMany.targetEntity().equals(void.class) ? member.getParameterizedType() : manyToMany.targetEntity();

        if (manyToMany.cascade().length != 0) {
            for (CascadeType cascadeType : manyToMany.cascade()) {
                cascade.add(cascadeType);
            }
        }

        fetch = manyToMany.fetch();
        mappedBy = manyToMany.mappedBy();
    }

    /**
     * build relationship on the target side
     */
    public RdbRelation buildInverseRelation(String unitName, String proprtyTypeName) {

        if (mappedBy.equals(""))
            return null;
        // get target class metadata
        ClassMetaData mappedClassMetaData = MetaDataRepository.getUnitMetaData(unitName).getClassMetaData(proprtyTypeName);
        String mappedTableName = mappedClassMetaData.getPrimaryTableName();
        RdbTable mappedTable = MetaDataRepository.getUnitMetaData(unitName).getTable(mappedTableName);
        RdbRelationTable ownerRelationTable = (RdbRelationTable) mappedTable.getRelation(mappedBy);
        // build Relation Table
        RdbRelationTable relationTable = new RdbRelationTable();
        relationTable.setRelationTable(ownerRelationTable.getRelationTable());
        relationTable.setSourceColumnNames(ownerRelationTable.getTargetColumnNames());
        relationTable.setSourceRelationColumnNames(ownerRelationTable.getTargetRelationColumnNames());
        relationTable.setTargetColumnNames(ownerRelationTable.getSourceColumnNames());
        relationTable.setTargetRelationColumnNames(ownerRelationTable.getSourceRelationColumnNames());

        relationTable.setReferencedEntityName(proprtyTypeName);
        relationTable.setReferencedTableName(mappedTableName);
        relationTable.setInverse(true);

        return relationTable;
    }
}
