/* -------------------------------------------------------------------------*
 * PKUAS: Peking University Application Server
 * Copyright (C) 2006 Peking University
 *
 * This software is free; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * either version 2.1 or any later version.
 * 
 * This software is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this software; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 * ------------------------------------------------------------------------*/

package org.ow2.xplus.metadata.object;

import javax.persistence.CascadeType;
import javax.persistence.OneToMany;

import org.ow2.xplus.mapping.annotation.Member;
import org.ow2.xplus.metadata.MetaDataRepository;
import org.ow2.xplus.metadata.relational.RdbForeignKey;
import org.ow2.xplus.metadata.relational.RdbRelation;
import org.ow2.xplus.metadata.relational.RdbRelationTable;
import org.ow2.xplus.metadata.relational.RdbTable;

/**
 * @author Lin Liang
 */

public class OneToManyRelation extends AbstractRelation {
    /**
     * constructor
     * @param member
     * @param oneToMany OneToMany annotation
     */
    public OneToManyRelation(Member member, OneToMany oneToMany) {
        targetEntity = oneToMany.targetEntity().equals(void.class) ? member.getParameterizedType() : oneToMany.targetEntity();

        if (oneToMany.cascade().length != 0) {
            for (CascadeType cascadeType : oneToMany.cascade()) {
                cascade.add(cascadeType);
            }
        }

        fetch = oneToMany.fetch();
        mappedBy = oneToMany.mappedBy();
    }

    /**
     * constructor
     * @param member
     */
    public OneToManyRelation(Member member) {
        targetEntity = member.getParameterizedType();
    }

    /**
     * build relation on the target side
     */
    public RdbRelation buildInverseRelation(String unitName, String proprtyTypeName) {
        if (mappedBy.equals(""))
            return null;
        // get target metadata
        ClassMetaData mappedClassMetaData = MetaDataRepository.getUnitMetaData(unitName).getClassMetaData(proprtyTypeName);
        String mappedTableName = mappedClassMetaData.getPrimaryTableName();
        RdbTable mappedTable = MetaDataRepository.getUnitMetaData(unitName).getTable(mappedTableName);
        RdbForeignKey ownerForeignKey = (RdbForeignKey) mappedTable.getRelation(mappedBy);

        // build Relation Table
        RdbRelationTable relationTable = new RdbRelationTable();
        relationTable.setRelationTable(mappedTable);
        relationTable.setSourceColumnNames(ownerForeignKey.getReferencedColumnNames());
        relationTable.setSourceRelationColumnNames(ownerForeignKey.getJoinColumnNames());
        relationTable.setTargetColumnNames(mappedTable.getPrimaryKey().getColumnNames());
        relationTable.setTargetRelationColumnNames(mappedTable.getPrimaryKey().getColumnNames());

        relationTable.setReferencedEntityName(proprtyTypeName);
        relationTable.setReferencedTableName(mappedTableName);
        relationTable.setInverse(true);

        return relationTable;
    }
}
