/* -------------------------------------------------------------------------*
 * PKUAS: Peking University Application Server
 * Copyright (C) 2006 Peking University
 *
 * This software is free; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * either version 2.1 or any later version.
 * 
 * This software is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this software; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 * ------------------------------------------------------------------------*/

package org.ow2.xplus.metadata.object;

import javax.persistence.CascadeType;
import javax.persistence.OneToOne;

import org.ow2.xplus.mapping.annotation.Member;
import org.ow2.xplus.metadata.MetaDataRepository;
import org.ow2.xplus.metadata.relational.RdbForeignKey;
import org.ow2.xplus.metadata.relational.RdbRelation;
import org.ow2.xplus.metadata.relational.RdbTable;

/**
 * @author Lin Liang
 */

public class OneToOneRelation extends AbstractRelation {
    /**
     * constructor
     * @param member field or method hold this relation
     * @param oneToOne OneToOne annotation
     */
    public OneToOneRelation(Member member, OneToOne oneToOne) {
        targetEntity = oneToOne.targetEntity().equals(void.class) ? member.getParameterizedType() : oneToOne.targetEntity();

        if (oneToOne.cascade().length != 0) {
            for (CascadeType cascadeType : oneToOne.cascade()) {
                cascade.add(cascadeType);
            }
        }

        fetch = oneToOne.fetch();
        optional = oneToOne.optional();
        mappedBy = oneToOne.mappedBy();
    }

    /**
     * build relation on the target side
     */
    public RdbRelation buildInverseRelation(String unitName, String proprtyTypeName) {
        if (mappedBy.equals(""))
            return null;
        // get target side metadata
        ClassMetaData mappedClassMetaData = MetaDataRepository.getUnitMetaData(unitName).getClassMetaData(proprtyTypeName);
        String mappedTableName = mappedClassMetaData.getPrimaryTableName();
        RdbTable mappedTable = MetaDataRepository.getUnitMetaData(unitName).getTable(mappedTableName);
        RdbForeignKey ownerForeignKey = (RdbForeignKey) mappedTable.getRelation(mappedBy);
        // build foreign key
        RdbForeignKey foreignKey = new RdbForeignKey();
        foreignKey.setJoinColumnNames(ownerForeignKey.getReferencedColumnNames());
        foreignKey.setReferencedColumnNames(ownerForeignKey.getJoinColumnNames());

        foreignKey.setReferencedEntityName(proprtyTypeName);
        foreignKey.setReferencedTableName(mappedTableName);
        foreignKey.setInverse(true);

        return foreignKey;
    }
}
