/* -------------------------------------------------------------------------*
 * PKUAS: Peking University Application Server
 * Copyright (C) 2006 Peking University
 *
 * This software is free; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * either version 2.1 or any later version.
 * 
 * This software is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this software; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 * ------------------------------------------------------------------------*/

package org.ow2.xplus.metadata.object;

import java.util.HashMap;
import java.util.Map;

import org.ow2.xplus.metadata.relational.RdbColumn;
import org.ow2.xplus.metadata.strategy.ValueStrategy;
import org.ow2.xplus.metadata.strategy.VersionStrategy;

/**
 * @author Lin Liang
 */

public class ValueMetaData {
    /**
     * collection value
     */
    private CollectionValue collectionValue;

    /**
     * simple value
     */
    private SimpleValue simpleValue;

    /**
     * whether it is a collection mapping
     */
    boolean collectionMapping = false;

    /**
     * strategy to store value
     */
    private ValueStrategy valueStrategy;

    /**
     * the version of the value which serves as its optimistic lock value.
     */
    private VersionStrategy versionStrategy;

    /**
     * Map<the name of column field, the info of override column>
     */
    private Map<String, RdbColumn> attributeOverrides = new HashMap<String, RdbColumn>();

    public CollectionValue getCollectionValue() {
        return collectionValue;
    }

    public void setCollectionValue(CollectionValue collection) {
        this.collectionValue = collection;
    }

    public boolean isCollectionMapping() {
        return collectionMapping;
    }

    public void setCollectionMapping(boolean collectionMapping) {
        this.collectionMapping = collectionMapping;
    }

    public SimpleValue getSimpleValue() {
        return simpleValue;
    }

    public void setSimpleValue(SimpleValue simple) {
        this.simpleValue = simple;
    }

    public ValueStrategy getValueStrategy() {
        return valueStrategy;
    }

    public void setValueStrategy(ValueStrategy valueStrategy) {
        this.valueStrategy = valueStrategy;
    }

    public VersionStrategy getVersionStrategy() {
        return versionStrategy;
    }

    public void setVersionStrategy(VersionStrategy versionStrategy) {
        this.versionStrategy = versionStrategy;
    }

    public void addAttributeOverride(String fieldName, RdbColumn overrrideColumnInfo) {
        attributeOverrides.put(fieldName, overrrideColumnInfo);
    }

    /**
     * return Attribute Override info, if there is no Attribute Override, return
     * null
     * @param columnName the name of the column
     * @return
     */
    public RdbColumn getAttributeOverride(String fieldName) {
        return attributeOverrides.get(fieldName);
    }
}
