/* -------------------------------------------------------------------------*
 * PKUAS: Peking University Application Server
 * Copyright (C) 2006 Peking University
 *
 * This software is free; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * either version 2.1 or any later version.
 * 
 * This software is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this software; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 * ------------------------------------------------------------------------*/

package org.ow2.xplus.metadata.relational;

/**
 * @author Lin Liang
 */

public class RdbColumn {

    /**
     * column name value.
     */
    private String name = "";

    /**
     * the name of field corresponding to the column
     */
    private String canonicalFieldName = "";

    /**
     * Whether the database column is nullable.
     */
    private boolean nullable = true;

    /**
     * Whether the property is a unique key..
     */
    private boolean unique = false;

    /**
     * Whether the column is included in SQL INSERT statements
     */
    private boolean insertable = true;

    /**
     * Whether the column is included in SQL UPDATE statements
     */
    private boolean updatable = true;

    /**
     * column default value
     */
    private String defaultValue = "";

    /**
     * The name of the table that contains the column
     */
    private String tableName = "";

    /**
     * length tag value.
     */
    private Integer length = new Integer(255);

    /**
     * scale tag value.
     */
    private Integer scale = new Integer(0);

    /**
     * precision tag value.
     */
    private Integer precision = new Integer(0);

    /**
     * java type.
     */
    private Class javaType;

    /**
     * sql type.
     */
    private String sqlType = "VARCHAR";

    /**
     * the SQL fragment for the column.
     */
    private String columnDefinition = "";

    /**
     * whether it is a primary key
     */
    private boolean isPrimaryKey = false;

    /**
     * whether to auto increment the value
     */
    private boolean autoIncrement = false;

    public RdbColumn(String name) {
        this.name = name.toUpperCase();
    }

    public RdbColumn(RdbColumn column) {
        this.name = column.getName();
        this.columnDefinition = column.getColumnDefinition();
        this.defaultValue = column.getDefaultValue();
        this.insertable = column.isInsertable();
        this.length = column.getLength();
        this.nullable = column.isNullable();
        this.precision = column.getPrecision();
        this.scale = column.getScale();
        this.javaType = column.getJavaType();
        this.sqlType = column.getSqlType();
        this.unique = column.isUnique();
        this.updatable = column.isUpdatable();
    }

    public String getName() {
        return name;
    }

    public void setName(String name) {
        this.name = name.toUpperCase();
    }

    public String getColumnDefinition() {
        return columnDefinition;
    }

    public void setColumnDefinition(String columnDefinition) {
        this.columnDefinition = columnDefinition;
    }

    public String getDefaultValue() {
        return defaultValue;
    }

    public void setDefaultValue(String defaultValue) {
        this.defaultValue = defaultValue;
    }

    public boolean isInsertable() {
        return insertable;
    }

    public void setInsertable(boolean insertable) {
        this.insertable = insertable;
    }

    public Integer getLength() {
        return length;
    }

    public void setLength(Integer length) {
        this.length = length;
    }

    public boolean isNullable() {
        return nullable;
    }

    public void setNullable(boolean nullable) {
        this.nullable = nullable;
    }

    public Integer getPrecision() {
        return precision;
    }

    public void setPrecision(Integer precision) {
        this.precision = precision;
    }

    public Integer getScale() {
        return scale;
    }

    public void setScale(Integer scale) {
        this.scale = scale;
    }

    public String getSqlType() {
        return sqlType;
    }

    public boolean isUnique() {
        return unique;
    }

    public void setUnique(boolean unique) {
        this.unique = unique;
    }

    public boolean isUpdatable() {
        return updatable;
    }

    public void setUpdatable(boolean updatable) {
        this.updatable = updatable;
    }

    public String getTableName() {
        return tableName;
    }

    public void setTableName(String table) {
        this.tableName = table;
    }

    public String getCanonicalFieldName() {
        return canonicalFieldName;
    }

    public void setCanonicalFieldName(String canonicalFieldName) {
        this.canonicalFieldName = canonicalFieldName;
    }

    public Class getJavaType() {
        return javaType;
    }

    public void setJavaType(Class javaType) {
        this.javaType = javaType;
    }

    public void setSqlType(String sqlType) {
        this.sqlType = sqlType;
    }

    public boolean isPrimaryKey() {
        return isPrimaryKey;
    }

    public void setPrimaryKey(boolean isPrimaryKey) {
        this.isPrimaryKey = isPrimaryKey;
        this.unique = isPrimaryKey;
    }

    public boolean isAutoIncrement() {
        return autoIncrement;
    }

    public void setAutoIncrement(boolean autoIncrement) {
        this.autoIncrement = autoIncrement;
    }

}
