/* -------------------------------------------------------------------------*
 * PKUAS: Peking University Application Server
 * Copyright (C) 2006 Peking University
 *
 * This software is free; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * either version 2.1 or any later version.
 * 
 * This software is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this software; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 * ------------------------------------------------------------------------*/

package org.ow2.xplus.metadata.relational;

import java.util.ArrayList;
import java.util.List;

import javax.persistence.JoinColumn;

import org.ow2.xplus.metadata.MetaDataRepository;
import org.ow2.xplus.util.NamingStrategy;

/**
 * foreign key meta of relation db
 * @author Lin Liang
 */

public class RdbForeignKey extends RdbAbstractRelation {
    /**
     * the name join column
     */
    private List<String> joinColumnNames = new ArrayList<String>();

    /**
     * the name of referenced column
     */
    private List<String> referencedColumnNames = new ArrayList<String>();

    /**
     * the primarykey of target entity
     */
    private RdbPrimaryKey joinPrimaryKey = null;

    /**
     * join PrimaryKey Columns
     */
    private List<JoinColumn> joinPrimaryKeyColumns = new ArrayList<JoinColumn>();

    /**
     * construcor
     */
    public RdbForeignKey() {
    }

    /**
     * add joinColumnName to referencedColumnName Mapping
     * @param joinColumnName the name of join column
     * @param referencedColumnName the name of referenced column
     */
    public void addcolumnMaping(String joinColumnName, String referencedColumnName) {
        joinColumnNames.add(joinColumnName);
        referencedColumnNames.add(referencedColumnName);
    }

    public List<String> getJoinColumnNames() {
        return joinColumnNames;
    }

    public void setJoinColumnNames(List<String> joinColumnNames) {
        this.joinColumnNames = joinColumnNames;
    }

    public List<String> getReferencedColumnNames() {
        return referencedColumnNames;
    }

    public void setReferencedColumnNames(List<String> referencedColumnNames) {
        this.referencedColumnNames = referencedColumnNames;
    }

    public RdbPrimaryKey getJoinPrimaryKey() {
        return joinPrimaryKey;
    }

    public void setJoinPrimaryKey(RdbPrimaryKey joinPrimaryKey) {
        this.joinPrimaryKey = joinPrimaryKey;
    }

    public List<JoinColumn> getJoinPrimaryKeyColumn() {
        return joinPrimaryKeyColumns;
    }

    public void addJoinPrimaryKeyColumn(int index, JoinColumn joinColumnAnn) {
        joinPrimaryKeyColumns.add(index, joinColumnAnn);
    }

    /**
     * bind columns from embedded entity
     */
    public void bindEmbeddedColumn(String unitName, String fieldName, String ownerTableName) {
        if ((joinPrimaryKey == null) || (joinPrimaryKey.getColumns().isEmpty()))
            return;

        int index = 0;
        for (RdbColumn primaryKeyColumn : joinPrimaryKey.getColumns()) {
            JoinColumn joinColumnAnn = null;
            if (joinPrimaryKeyColumns.size() > index) {
                joinColumnAnn = joinPrimaryKeyColumns.get(index++);
            }
            bindJoinColumn(unitName, fieldName, ownerTableName, primaryKeyColumn, joinColumnAnn);
        }
    }

    /**
     * bind join column to foreign key
     * @param unitName the name of persistence unit contains this entity
     * @param fieldName the field holds the relation
     * @param ownerTableName the name of relation owner table
     * @param referencedColumn the referenced column
     * @param joinColumnAnn JoinColumn annotation
     */
    public void bindJoinColumn(String unitName, String fieldName, String ownerTableName, RdbColumn referencedColumn,
            JoinColumn joinColumnAnn) {

        String joinColumnName = NamingStrategy.getJoinColumnName(fieldName, referencedColumn.getName());

        if ((joinColumnAnn != null) && (!joinColumnAnn.name().equals(""))) {
            joinColumnName = joinColumnAnn.name();
        }

        RdbColumn column = new RdbColumn(referencedColumn);
        column.setName(joinColumnName);
        column.setUnique(false);
        column.setPrimaryKey(false);

        if (joinColumnAnn != null) {
            column.setUnique(joinColumnAnn.unique());
            column.setNullable(joinColumnAnn.nullable());
            column.setInsertable(joinColumnAnn.insertable());
            column.setColumnDefinition(joinColumnAnn.columnDefinition());
        }

        column.setTableName(ownerTableName);

        RdbTable table = MetaDataRepository.getUnitMetaData(unitName).getTable(ownerTableName);
        table.addColumn(column);

        addcolumnMaping(joinColumnName, referencedColumn.getName());
    }

}
