/* -------------------------------------------------------------------------*
 * PKUAS: Peking University Application Server
 * Copyright (C) 2006 Peking University
 *
 * This software is free; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * either version 2.1 or any later version.
 * 
 * This software is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this software; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 * ------------------------------------------------------------------------*/

package org.ow2.xplus.metadata.relational;

import java.util.ArrayList;
import java.util.Collection;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * @author Lin Liang
 */

public class RdbPrimaryKey {
    /**
     * the name of primaryKey class
     */
    private String pkClassName = null;

    /**
     * primary key columns Map<the name of column, rdb column>
     */
    private Map<String, RdbColumn> keyColumns = new HashMap<String, RdbColumn>();

    /**
     * rdb id generator
     */
    private RdbIdGenerator idgen = null;

    /**
     * @return all key columns
     */
    public List<RdbColumn> getColumns() {
        Collection<RdbColumn> allColumns = keyColumns.values();
        List<RdbColumn> columns = new ArrayList<RdbColumn>();
        columns.addAll(allColumns);
        return columns;
    }

    public List<String> getColumnNames() {
        Collection<RdbColumn> allColumns = keyColumns.values();
        List<String> columnNames = new ArrayList<String>();
        for (RdbColumn column : allColumns) {
            columnNames.add(column.getName());
        }
        return columnNames;
    }

    public void addColumn(RdbColumn column) {
        keyColumns.put(column.getName(), column);
    }

    public RdbIdGenerator getIdgen() {
        return idgen;
    }

    public void setIdgen(RdbIdGenerator idgen) {
        this.idgen = idgen;
    }

    public String getPkClassName() {
        return pkClassName;
    }

    public void setPkClassName(String pkClassName) {
        this.pkClassName = pkClassName;
    }

    public RdbColumn getColumn(String primaryColumnName) {
        return keyColumns.get(primaryColumnName);
    }

}
