/* -------------------------------------------------------------------------*
 * PKUAS: Peking University Application Server
 * Copyright (C) 2006 Peking University
 *
 * This software is free; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * either version 2.1 or any later version.
 * 
 * This software is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this software; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 * ------------------------------------------------------------------------*/

package org.ow2.xplus.metadata.relational;

import java.util.ArrayList;
import java.util.List;

import javax.persistence.JoinColumn;

import org.ow2.xplus.util.NamingStrategy;

/**
 * the metadata of relation table
 * @author Lin Liang
 */

public class RdbRelationTable extends RdbAbstractRelation {
    /**
     * the relation table
     */
    private RdbTable relationTable;

    /**
     * the name of the source column from relation owner entity
     */
    private List<String> sourceColumnNames = new ArrayList<String>();

    /**
     * the name of the column mapped from source column
     */
    private List<String> sourceRelationColumnNames = new ArrayList<String>();

    /**
     * the name of the target column from relation target entity
     */
    private List<String> targetColumnNames = new ArrayList<String>();

    /**
     * the name of the column mapped from target column
     */
    private List<String> targetRelationColumnNames = new ArrayList<String>();

    /**
     * the primary key from relation owner
     */
    private RdbPrimaryKey sourceJoinPrimaryKey = null;

    /**
     * the primary key join columns from relation owner
     */
    private List<JoinColumn> sourceJoinPrimaryKeyColumns = new ArrayList<JoinColumn>();

    /**
     * the primary key from relation target
     */
    private RdbPrimaryKey targetJoinPrimaryKey = null;

    /**
     * the primary key join columns from relation target
     */
    private List<JoinColumn> targetJoinPrimaryKeyColumns = new ArrayList<JoinColumn>();

    public RdbRelationTable() {

    }

    public RdbTable getRelationTable() {
        return relationTable;
    }

    public String getRelationTableName() {
        return relationTable.getCanonicalName();
    }

    public void setRelationTable(RdbTable relationTable) {
        this.relationTable = relationTable;
    }

    public void addSourceRelationMapping(String relationColumnName, String sourceColumnName) {
        sourceColumnNames.add(sourceColumnName);
        sourceRelationColumnNames.add(relationColumnName);
    }

    public void addTargetRelationMapping(String relationColumnName, String targetColumnName) {
        targetColumnNames.add(targetColumnName);
        targetRelationColumnNames.add(relationColumnName);
    }

    public List<String> getSourceColumnNames() {
        return sourceColumnNames;
    }

    public List<String> getSourceRelationColumnNames() {
        return sourceRelationColumnNames;
    }

    public String getSourceRelationColumnName(String sourceColumnName) {
        for (int index = 0; index < sourceColumnNames.size(); ++index) {
            if (sourceColumnNames.get(index).equals(sourceColumnName)) {
                return sourceRelationColumnNames.get(index);
            }
        }
        return null;
    }

    public List<String> getTargetColumnNames() {
        return targetColumnNames;
    }

    public List<String> getTargetRelationColumnNames() {
        return targetRelationColumnNames;
    }

    public String getTargetRelationColumnName(String targetColumnName) {
        for (int index = 0; index < targetColumnNames.size(); ++index) {
            if (targetColumnNames.get(index).equals(targetColumnName)) {
                return targetRelationColumnNames.get(index);
            }
        }
        return null;
    }

    public void setSourceColumnNames(List<String> sourceColumnNames) {
        this.sourceColumnNames = sourceColumnNames;
    }

    public void setSourceRelationColumnNames(List<String> sourceRelationColumnNames) {
        this.sourceRelationColumnNames = sourceRelationColumnNames;
    }

    public void setTargetColumnNames(List<String> targetColumnNames) {
        this.targetColumnNames = targetColumnNames;
    }

    public void setTargetRelationColumnNames(List<String> targetRelationColumnNames) {
        this.targetRelationColumnNames = targetRelationColumnNames;
    }

    public RdbPrimaryKey getSourceJoinPrimaryKey() {
        return sourceJoinPrimaryKey;
    }

    public void setSourceJoinPrimaryKey(RdbPrimaryKey sourceJoinPrimaryKey) {
        this.sourceJoinPrimaryKey = sourceJoinPrimaryKey;
    }

    public RdbPrimaryKey getTargetJoinPrimaryKey() {
        return targetJoinPrimaryKey;
    }

    public void setTargetJoinPrimaryKey(RdbPrimaryKey targetJoinPrimaryKey) {
        this.targetJoinPrimaryKey = targetJoinPrimaryKey;
    }

    public List<JoinColumn> getSourceJoinPrimaryKeyColumns() {
        return sourceJoinPrimaryKeyColumns;
    }

    public void addSourceJoinPrimaryKeyColumn(int index, JoinColumn joinColumnAnn) {
        sourceJoinPrimaryKeyColumns.add(index, joinColumnAnn);
    }

    public List<JoinColumn> getTargetJoinPrimaryKeyColumns() {
        return targetJoinPrimaryKeyColumns;
    }

    public void addTargetJoinPrimaryKeyColumn(int index, JoinColumn joinColumnAnn) {
        targetJoinPrimaryKeyColumns.add(index, joinColumnAnn);
    }

    /**
     * bind embedded column from PrimaryKey
     * @param fieldName the name of field holds the relation
     * @param ownerTableName the name of relation owner's table
     */
    public void bindEmbeddedColumn(String unitName, String fieldName, String ownerTableName) {
        if (!(sourceJoinPrimaryKey == null) && !(sourceJoinPrimaryKey.getColumns().isEmpty())) {
            int index = 0;
            for (RdbColumn primaryKeyColumn : sourceJoinPrimaryKey.getColumns()) {
                JoinColumn joinColumnAnn = null;
                if (sourceJoinPrimaryKeyColumns.size() > index) {
                    joinColumnAnn = sourceJoinPrimaryKeyColumns.get(index++);
                }
                bindJoinColumn(fieldName, primaryKeyColumn, joinColumnAnn, false);
            }
        }

        if (!(targetJoinPrimaryKey == null) && !(targetJoinPrimaryKey.getColumns().isEmpty())) {
            int index = 0;
            for (RdbColumn primaryKeyColumn : targetJoinPrimaryKey.getColumns()) {
                JoinColumn joinColumnAnn = null;
                if (targetJoinPrimaryKeyColumns.size() > index) {
                    joinColumnAnn = targetJoinPrimaryKeyColumns.get(index++);
                }
                bindJoinColumn(fieldName, primaryKeyColumn, joinColumnAnn, true);
            }
        }
    }

    /**
     * bind join column to relation table
     * @param fieldName the name of field holds the relation
     * @param referencedColumn the referenced column
     * @param joinColumnAnn the JoinColumn annotation
     * @param inverse if it is a inverse joinColumn
     */
    public void bindJoinColumn(String fieldName, RdbColumn referencedColumn, JoinColumn joinColumnAnn, boolean inverse) {

        String relationColumnName = NamingStrategy.getJoinColumnName(fieldName, referencedColumn.getName());

        if ((joinColumnAnn != null) && (!joinColumnAnn.name().equals(""))) {
            relationColumnName = joinColumnAnn.name();
        }

        RdbColumn column = null;
        if (referencedColumn != null) {
            column = new RdbColumn(referencedColumn);
            column.setUnique(false);
            column.setPrimaryKey(false);
        } else {
            column = new RdbColumn(relationColumnName);
        }

        column.setName(relationColumnName);

        if (joinColumnAnn != null) {
            column.setUnique(joinColumnAnn.unique());
            column.setNullable(joinColumnAnn.nullable());
            column.setInsertable(joinColumnAnn.insertable());
            column.setColumnDefinition(joinColumnAnn.columnDefinition());
        }

        column.setTableName(relationTable.getCanonicalName());
        relationTable.addColumn(column);

        if (inverse) {
            addTargetRelationMapping(relationColumnName, referencedColumn.getName());
        } else {
            addSourceRelationMapping(relationColumnName, referencedColumn.getName());
        }
    }

}
