/* -------------------------------------------------------------------------*
 * PKUAS: Peking University Application Server
 * Copyright (C) 2006 Peking University
 *
 * This software is free; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * either version 2.1 or any later version.
 * 
 * This software is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this software; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 * ------------------------------------------------------------------------*/

package org.ow2.xplus.metadata.strategy.generator;

import java.util.List;

import org.ow2.xplus.broker.PersistenceManager;
import org.ow2.xplus.metadata.UnitMetaData;
import org.ow2.xplus.metadata.strategy.GeneratorStrategy;
import org.ow2.xplus.query.SQLQuery;

/**
 * @author Lin Liang
 */

public class TableGeneratorStrategy implements GeneratorStrategy {

    private final static String PK_COLUMN_NAME = "KEY";

    private final static String PK_COLUMN_VALUE = "KEY";

    private final static String VALUE_COLUMN_NAME = "VALUE";

    private String name;

    private String tableName;

    private String query;

    private UnitMetaData unitMetaData;

    private String pkColumnName = PK_COLUMN_NAME;

    private String valueColumnName = VALUE_COLUMN_NAME;

    private String pkColumnValue = PK_COLUMN_VALUE;

    private int initialValue = 0;

    private int allocationSize = 50;

    public TableGeneratorStrategy(UnitMetaData unitMetaData) {
        query = "select " + VALUE_COLUMN_NAME + " from " + tableName;
        this.unitMetaData = unitMetaData;
    }

    public int getAllocationSize() {
        return allocationSize;
    }

    public void setAllocationSize(int allocationSize) {
        this.allocationSize = allocationSize;
    }

    public int getInitialValue() {
        return initialValue;
    }

    public void setInitialValue(int initialValue) {
        this.initialValue = initialValue;
    }

    public String getPkColumnName() {
        return pkColumnName;
    }

    public void setPkColumnName(String pkColumnName) {
        if ((pkColumnName == null) || (pkColumnName.equals("")))
            return;
        this.pkColumnName = pkColumnName;
    }

    public String getPkColumnValue() {
        return pkColumnValue;
    }

    public void setPkColumnValue(String pkColumnValue) {
        if ((pkColumnValue == null) || (pkColumnValue.equals("")))
            return;
        this.pkColumnValue = pkColumnValue;
    }

    public String getTableName() {
        return tableName;
    }

    public void setTableName(String tableName) {
        this.tableName = tableName;
    }

    public String getValueColumnName() {
        return valueColumnName;
    }

    public void setValueColumnName(String valueColumnName) {
        if ((valueColumnName == null) || (valueColumnName.equals("")))
            return;
        this.valueColumnName = valueColumnName;
    }

    public String getName() {
        return name;
    }

    public void setName(String name) {
        this.name = name;
    }

    public Object generate() {
        PersistenceManager pm = new PersistenceManager(unitMetaData);
        query = "select " + VALUE_COLUMN_NAME + " from " + tableName;
        SQLQuery sqlQuery = new SQLQuery(query, pm);
        sqlQuery.setPropertyName(VALUE_COLUMN_NAME);
        List result = sqlQuery.getSQLResult();
        Integer keyValue = new Integer(initialValue);

        if (result.isEmpty()) {
            String initQuery = "INSERT INTO " + tableName + " VALUES (\'" + PK_COLUMN_NAME + "\', "
                    + getDefaultValue().toString() + ")";
            SQLQuery initSqlQuery = new SQLQuery(initQuery, pm);
            initSqlQuery.executeUpdate();
        } else {
            keyValue = Integer.valueOf(result.get(0).toString());
        }

        Integer newValue = new Integer((keyValue.intValue()) + 1);
        String updateQuery = "UPDATE " + tableName + " SET " + VALUE_COLUMN_NAME + " = " + newValue.toString() + " WHERE "
                + PK_COLUMN_NAME + " = \"" + PK_COLUMN_VALUE + "\"";
        SQLQuery updateSqlQuery = new SQLQuery(updateQuery, pm);
        updateSqlQuery.executeUpdate();
        return keyValue;
    }

    public Object getDefaultValue() {
        return new Integer(initialValue);
    }

}
