/* -------------------------------------------------------------------------*
 * PKUAS: Peking University Application Server
 * Copyright (C) 2006 Peking University
 *
 * This software is free; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * either version 2.1 or any later version.
 * 
 * This software is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this software; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 * ------------------------------------------------------------------------*/

package org.ow2.xplus.metadata.strategy.inheritance;

import java.util.List;

import org.ow2.xplus.metadata.MetaDataRepository;
import org.ow2.xplus.metadata.object.ClassMetaData;
import org.ow2.xplus.metadata.relational.RdbColumn;
import org.ow2.xplus.metadata.relational.RdbInheritanceJoinedKey;
import org.ow2.xplus.metadata.relational.RdbPrimaryKey;
import org.ow2.xplus.metadata.relational.RdbTable;
import org.ow2.xplus.metadata.strategy.InheritanceStrategy;
import org.ow2.xplus.util.NamingStrategy;

/**
 * @author Lin Liang
 */

public class JoinedInheritanceStrategy implements InheritanceStrategy {

    private static JoinedInheritanceStrategy strategy = new JoinedInheritanceStrategy();

    private JoinedInheritanceStrategy() {
        super();
    }

    public static JoinedInheritanceStrategy getInstance() {
        return strategy;
    }

    public String getTableName(String unitName, String className) {
        return NamingStrategy.unqualify(className);
    }

    public boolean ignoreTableAnn(String unitName, String className) {
        return false;
    }

    public void buildTable(String unitName, String className) {
        ClassMetaData metadata = MetaDataRepository.getUnitMetaData(unitName).getClassMetaData(className);
        if (metadata.isInheritanceRoot())
            return;

        RdbTable table = MetaDataRepository.getUnitMetaData(unitName).getTable(metadata.getPrimaryTableName());
        RdbInheritanceJoinedKey inheritanceJoinedKey = table.getInheritanceJoinedKey();

        String inheritanceEntityName = metadata.getParent().getName();
        inheritanceJoinedKey.setInheritanceEntityName(inheritanceEntityName);
        String inheritanceEntityTableName = MetaDataRepository.getUnitMetaData(unitName).getClassMetaData(inheritanceEntityName)
                .getPrimaryTableName();
        inheritanceJoinedKey.setInheritanceEntityTableName(inheritanceEntityTableName);

        RdbTable inheritanceEntityTable = MetaDataRepository.getUnitMetaData(unitName).getTable(inheritanceEntityTableName);
        RdbPrimaryKey inheritancePrimaryKey = inheritanceEntityTable.getPrimaryKey();

        List<String> joinColumnNames = inheritanceJoinedKey.getJoinColumnNames();
        List<String> referencedColumnNames = inheritanceJoinedKey.getReferencedColumnNames();

        if (inheritancePrimaryKey.getColumns() != null) {
            for (RdbColumn inheritancePrimaryColumn : inheritancePrimaryKey.getColumns()) {
                String joinColumnName = inheritancePrimaryColumn.getName();
                int index = referencedColumnNames.indexOf(joinColumnName);
                if (index != -1)
                    joinColumnName = joinColumnNames.get(index);
                RdbColumn joinColumn = new RdbColumn(inheritancePrimaryColumn);
                joinColumn.setName(joinColumnName);
                joinColumn.setTableName(table.getCanonicalName());
                joinColumn.setPrimaryKey(true);
                table.addColumn(joinColumn);
            }
        }
    }

}
