/* -------------------------------------------------------------------------*
 * PKUAS: Peking University Application Server
 * Copyright (C) 2006 Peking University
 *
 * This software is free; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * either version 2.1 or any later version.
 * 
 * This software is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this software; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 * ------------------------------------------------------------------------*/

package org.ow2.xplus.metadata.strategy.inheritance;

import org.ow2.xplus.metadata.MetaDataRepository;
import org.ow2.xplus.metadata.object.ClassMetaData;
import org.ow2.xplus.metadata.relational.RdbColumn;
import org.ow2.xplus.metadata.relational.RdbDiscriminator;
import org.ow2.xplus.metadata.relational.RdbTable;
import org.ow2.xplus.metadata.strategy.DiscriminatorStrategy;
import org.ow2.xplus.metadata.strategy.InheritanceStrategy;
import org.ow2.xplus.metadata.strategy.discriminator.StringDiscriminatorStrategy;

/**
 * @author Lin Liang
 */

public class SingleTableInheritanceStrategy implements InheritanceStrategy {

    private static SingleTableInheritanceStrategy strategy = new SingleTableInheritanceStrategy();

    private SingleTableInheritanceStrategy() {
        super();
    }

    public static SingleTableInheritanceStrategy getInstance() {
        return strategy;
    }

    @SuppressWarnings("unchecked")
    public String getTableName(String unitName, String className) {
        ClassMetaData metaData = MetaDataRepository.getUnitMetaData(unitName).getClassMetaData(className);
        Class root = metaData.getInheritanceRootClass();
        ClassMetaData rootMetaData = MetaDataRepository.getUnitMetaData(unitName).getClassMetaData(root.getName());
        return rootMetaData.getPrimaryTableName();
    }

    public boolean ignoreTableAnn(String unitName, String className) {
        ClassMetaData metadata = MetaDataRepository.getUnitMetaData(unitName).getClassMetaData(className);
        if (metadata.isInheritanceRoot())
            return false;
        return true;
    }

    public void buildTable(String unitName, String className) {
        ClassMetaData metadata = MetaDataRepository.getUnitMetaData(unitName).getClassMetaData(className);
        RdbTable primaryTable = MetaDataRepository.getUnitMetaData(unitName).getTable(metadata.getPrimaryTableName());
        if (metadata.isInheritanceRoot()) {
            metadata.setHasTable(true);
            if (!metadata.getSubClasseNames().isEmpty()) {
                addDiscriminator(primaryTable);
            }
        }
    }

    private void addDiscriminator(RdbTable primaryTable) {
        if (primaryTable.getDiscriminator() != null)
            return;
        RdbDiscriminator discriminator = new RdbDiscriminator();
        RdbColumn discriminatorColumn = new RdbColumn("DType");
        discriminatorColumn.setTableName(primaryTable.getCanonicalName());
        discriminator.setDiscriminatorColumnName("DType");
        DiscriminatorStrategy discriminatorStrategy = new StringDiscriminatorStrategy();
        discriminator.setDiscriminatorStrategy(discriminatorStrategy);
        discriminatorColumn.setJavaType(discriminatorStrategy.getJavaType());
        discriminatorColumn.setSqlType(discriminatorStrategy.getSqlType());

        primaryTable.setDiscriminator(discriminator);
        primaryTable.addColumn(discriminatorColumn);
    }

}
