/* -------------------------------------------------------------------------*
 * PKUAS: Peking University Application Server
 * Copyright (C) 2006 Peking University
 *
 * This software is free; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * either version 2.1 or any later version.
 * 
 * This software is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this software; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 * ------------------------------------------------------------------------*/

package org.ow2.xplus.metadata.strategy.value;

import java.util.ArrayList;
import java.util.List;

/**
 * @author Lin Liang
 */

public class EnumeratedValueStrategy extends AbstractValueStrategy {

    private boolean ordinal = false;

    private Class<? extends Enum> valueType;

    private List<Enum> enums = new ArrayList<Enum>();

    public EnumeratedValueStrategy() {

    }

    public void addEnum(Enum enumElem) {
        enums.add(enumElem);
    }

    @Override
    public Object getDataValue(Object value) {
        if (value == null)
            return null;
        if (ordinal) {
            return new Integer(((Enum) value).ordinal());
        }
        return ((Enum) value).name();
    }

    @Override
    @SuppressWarnings("unchecked")
    public Object getObjectValue(Object value) {
        if (value == null) {
            return null;
        }
        if (ordinal) {
            return enums.get(((Number) value).intValue());
        }
        return Enum.valueOf(valueType, (String) value);
    }

    public Class<? extends Enum> getValueType() {
        return valueType;
    }

    public void setValueType(Class<? extends Enum> ValueType) {
        this.valueType = ValueType;
    }

    public boolean isOrdinal() {
        return ordinal;
    }

    public void setOrdinal(boolean ordinal) {
        this.ordinal = ordinal;
        if (ordinal)
            this.dataType = Integer.class;
        else
            this.dataType = String.class;
    }

}
