/* -------------------------------------------------------------------------*
 * PKUAS: Peking University Application Server
 * Copyright (C) 2006 Peking University
 *
 * This software is free; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * either version 2.1 or any later version.
 * 
 * This software is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this software; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 * ------------------------------------------------------------------------*/

package org.ow2.xplus.operator;

import java.util.ArrayList;
import java.util.List;

import org.apache.commons.beanutils.DynaBean;
import org.apache.ddlutils.platform.ModelBasedResultSetIterator;

import org.ow2.xplus.context.PersistenceContext;
import org.ow2.xplus.metadata.UnitMetaData;
import org.ow2.xplus.metadata.object.ClassMetaData;
import org.ow2.xplus.operator.accessor.ClassAccessor;

/**
 * a EntityCreator used to create a entity based on ResultSet from query
 * @author Lin Liang
 */

public class EntityCreator {

    /**
     * the metadata of a persistence unit
     */
    private UnitMetaData unit;

    private PersistenceContext context;

    public EntityCreator(UnitMetaData unit, PersistenceContext context) {
        this.unit = unit;
        this.context = context;
    }

    /**
     * create a list of entities from ResultSet
     * @param type the class type of entity to be created
     * @param resultIter the ResultSet Iterator
     * @return a list of entities
     */
    @SuppressWarnings("unchecked")
    public List create(Class type, ModelBasedResultSetIterator resultIter) {
        List result = new ArrayList();

        while (resultIter.hasNext()) {
            DynaBean bean = (DynaBean) resultIter.next();
            ClassMetaData classMetaData = unit.getClassMetaData(type.getName());

            ClassAccessor constructor = new ClassAccessor(classMetaData);
            try {
                Object entity = constructor.create(bean, context,null);
                result.add(entity);
            } catch (Exception ex) {
            }
        }

        return result;
    }

}
