/* -------------------------------------------------------------------------*
 * PKUAS: Peking University Application Server
 * Copyright (C) 2006 Peking University
 *
 * This software is free; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * either version 2.1 or any later version.
 * 
 * This software is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this software; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 * ------------------------------------------------------------------------*/

package org.ow2.xplus.operator;

import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Set;

import javax.persistence.CascadeType;

import org.jdom.Document;
import org.jdom.Element;
import org.jdom.output.XMLOutputter;

import org.ow2.xplus.metadata.object.ClassMetaData;
import org.ow2.xplus.operator.accessor.ClassAccessor;

/**
 * Read the properties of a entity, and return its XML represent
 * @author Lin Liang
 */

public class EntityReader {

    private ClassMetaData classMetaData;

    /** the data in xml format */
    private String dataXML = null;

    /** the relation table datas */
    private String relationDataXML = null;

    /**
     * whether do a copy for the data
     */
    private static boolean doCopy = false;
    
    /**
     * the names of primary key columns per table
     */
    private Map<String, List<String>> keyColumns = new HashMap<String, List<String>>();

    public EntityReader(ClassMetaData classMetaData) {
        this.classMetaData = classMetaData;
    }

    /**
     * read object data
     * @param data the entity instance
     * @param cascade whether to cascade
     * @param whether to generate id while reading
     */
    public void read(Object data, CascadeType cascade, boolean generateid) {
        Element dataElem = new Element("data");
        Element relationDataElem = new Element("data");

        ClassAccessor classAccessor = new ClassAccessor(classMetaData);
        classAccessor.setNeedCascadeType(cascade);

        classAccessor.getDataXML(dataElem, relationDataElem, data, keyColumns, generateid);

        Document doc = new Document(dataElem);
        XMLOutputter XMLOut = new XMLOutputter();
        dataXML = XMLOut.outputString(doc);

        doc = new Document(relationDataElem);
        XMLOut = new XMLOutputter();
        relationDataXML = XMLOut.outputString(doc);
    }

    /**
     * get all table affected when reading
     * @param cascade whether to cascade
     * @return all the names of tables
     */
    public Set<String> getAffectTableNames(boolean cascade) {
        ClassAccessor classAccessor = new ClassAccessor(classMetaData);
        return classAccessor.getAffectTableNames(cascade);
    }

    /**
     * the xml represent of the entity data
     * @return xml data
     */
    public String getDataXML() {
        return dataXML;
    }

    /**
     * get the names of primary key columns per table
     * @return the names of primary key columns per table
     */
    public Map<String, List<String>> getKeyColumns() {
        return keyColumns;
    }

    /**
     * the relation table datas
     * @return the xml
     */
    public String getRelationDataXML() {
        return relationDataXML;
    }

	public static boolean isDoCopy() {
		return doCopy;
	}

	public static void setDoCopy(boolean doCopy) {
		EntityReader.doCopy = doCopy;
	}

}
