/* -------------------------------------------------------------------------*
 * PKUAS: Peking University Application Server
 * Copyright (C) 2006 Peking University
 *
 * This software is free; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * either version 2.1 or any later version.
 * 
 * This software is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this software; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 * ------------------------------------------------------------------------*/

package org.ow2.xplus.operator;

import java.io.StringReader;
import java.util.List;
import java.util.Map;

import javax.persistence.CascadeType;

import org.apache.ddlutils.io.DataReader;
import org.apache.ddlutils.io.DataToDatabaseSink;
import org.apache.ddlutils.io.DelSetFromDatabaseSink;

import org.ow2.xplus.deployer.DatabaseManager;

/**
 * persist entities to database
 * @author Lin Liang
 */

public class Persister {

    private EntityReader reader;

    public Persister(EntityReader reader) {
        this.reader = reader;
    }

    /**
     * persistence given entities to a database
     * @param entity entity to save
     * @param database the DatabaseManager of the database
     */
    public void persist(Object entity, DatabaseManager database,DataToDatabaseSink sink) {
        reader.read(entity, CascadeType.PERSIST, true);
        String dataXml = reader.getDataXML();
        String relationDataXml = reader.getRelationDataXML();
        Map<String, List<String>> keyColumns = reader.getKeyColumns();

        // delete all of old relations
        DataReader dataReader = new DataReader();
        dataReader.setModel(database.getModel());
        dataReader.setSink(new DelSetFromDatabaseSink(database.getPlatform(), database.getModel(), keyColumns));
        try {
            dataReader.parse(new StringReader(relationDataXml));
        } catch (Exception e) {
            e.printStackTrace();
        }

        // persister entities
        dataReader = new DataReader();
        dataReader.setModel(database.getModel());
        dataReader.setSink(new DataToDatabaseSink(database.getPlatform(), database.getModel()));
        try {
            dataReader.parse(new StringReader(dataXml));
        } catch (Exception e) {
            e.printStackTrace();
        }

    }

}
