/* -------------------------------------------------------------------------*
 * PKUAS: Peking University Application Server
 * Copyright (C) 2006 Peking University
 *
 * This software is free; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * either version 2.1 or any later version.
 * 
 * This software is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this software; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 * ------------------------------------------------------------------------*/

package org.ow2.xplus.operator.accessor;

import java.util.Collection;
import java.util.Map;

import org.ow2.xplus.metadata.object.ClassMetaData;
import org.ow2.xplus.metadata.object.FieldMetaData;

/**
 * @author Lin Liang
 */

public abstract class AbstractCollectionAccessor extends AbstractAccessor {

    protected Visitor visitor;

    public AbstractCollectionAccessor(ClassMetaData classMetaData, FieldMetaData field) {
        super(classMetaData, field);
        if (Collection.class.isAssignableFrom(field.getCollectionType())) {
            visitor = new CollectionVisitor();
        } else if (Map.class.isAssignableFrom(field.getCollectionType())) {
            visitor = new MapVisitor();
        } else if (field.getCollectionType().isArray()) {
            visitor = new ArrayVisitor();
        }
    }

    /**
     * save the value of the field to the collection field
     */
    public boolean save(Object entityData, Object fieldValue) {
        if (fieldValue == null)
            return false;

        ClassMetaData valueClassMetaData = unit.getClassMetaData(fieldValue.getClass().getCanonicalName());

        Object value = getFieldValue(entityData, field.getFieldName());
        if (!fieldValue.equals(value)) {
            visitor.addToCollection(entityData, field, fieldValue, valueClassMetaData);
            return true;
        }
        return false;
    }
    
    public boolean save4Copy(Object entityData, Object fieldValue) {
    	return save(entityData,fieldValue);
    }
}