/* -------------------------------------------------------------------------*
 * PKUAS: Peking University Application Server
 * Copyright (C) 2006 Peking University
 *
 * This software is free; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * either version 2.1 or any later version.
 * 
 * This software is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this software; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 * ------------------------------------------------------------------------*/

package org.ow2.xplus.operator.accessor;

import java.util.List;
import java.util.Set;

import javax.persistence.CascadeType;

import org.apache.commons.beanutils.DynaBean;
import org.jdom.Element;

import org.ow2.xplus.context.PersistenceContext;
import org.ow2.xplus.metadata.relational.RdbColumn;

/**
 * property accessor
 * @author Lin Liang
 */

public interface Accessor {

    /**
     * get the value, build columns and the value stored
     * @param data the entity data
     */
    public void obtainValue(Object data);

    /**
     * @return the columns representing this property
     */
    public List<RdbColumn> getColumns();

    /**
     * @return the values stored in the columns representing this property
     */
    public List getValues();

    /**
     * get data form object, write it in xml format
     * @param root the root of xml doc
     * @param data the data
     * @param generateid whether to generate id while reading
     */
    public void generateXMLData(Element root, Object data, boolean generateid);

    /**
     * get cascade entities infomation, write it in xml format
     * @param root the root of xml doc
     * @param data the data
     * @param generateid whether to generate id while reading
     */
    public void generateCascadeXMLData(Element root, Object data, boolean generateid);

    /**
     * the cascade type, used to determine whether to generate cascade data
     */
    public void setNeedCascadeType(CascadeType cascadeType);

    /**
     * the name of tables affected while accessing
     */
    public Set<String> getAffectTableNames(boolean cascade);

    /**
     * create the entity based on dynabean, store to the context
     * @param bean the dynabean store the value of properties
     * @param context the persistence context
     * @return the entity created
     */
    public Object create(DynaBean bean, PersistenceContext context,Object entityData);

    
 /**
  * create the entity based on copy 
  * @param bean
  * @param context
  * @return
  */
    public Object create(Object oentity, PersistenceContext context,Object entityData);
    
    
    /**
     * save the value to the field of the entity
     * @param entityData the entity data
     * @param fieldValue the value to store in the field
     * @return whether the value has changed
     */
    public boolean save(Object entityData, Object fieldValue);
    
    /**
     * save the value to the field of the entity
     * @param entityData the entity data
     * @param fieldValue the value to store in the field
     * @return whether the value has changed
     */
    public boolean save4Copy(Object entityData, Object fieldValue);
}
