/* -------------------------------------------------------------------------*
 * PKUAS: Peking University Application Server
 * Copyright (C) 2006 Peking University
 *
 * This software is free; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * either version 2.1 or any later version.
 * 
 * This software is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this software; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 * ------------------------------------------------------------------------*/

package org.ow2.xplus.operator.accessor;

import java.util.ArrayList;
import java.util.Collection;
import java.util.List;

import org.ow2.xplus.metadata.object.ClassMetaData;
import org.ow2.xplus.metadata.object.FieldMetaData;

/**
 * @author Lin Liang
 */

@SuppressWarnings("unchecked")
public class ArrayVisitor extends AbstractVisitor {

    public Collection getCollection(Object data) {
        Object[] dataArray = (Object[]) data;
        int length = dataArray.length;

        Collection allData = new ArrayList();
        for (int index = 0; index < length; ++index) {
            allData.add(dataArray[index]);
        }

        return allData;
    }

    public void addToCollection(Object entityData, FieldMetaData field, Object value, ClassMetaData valueClassMetaData) {

        Object[] dataArray = (Object[]) getFieldValue(entityData, field.getFieldName());
        int length = dataArray.length;

        List allData = new ArrayList();
        for (int index = 0; index < length; ++index) {
            allData.add(dataArray[index]);
        }

        saveWithCheckForExistence(allData, value, valueClassMetaData);

        Object fieldValue = allData.toArray();
        setFieldValue(entityData, field.getFieldName(), fieldValue);
    }


    /**
     * save the value to the collection property, if there exists one with the
     * same identity, override it
     * @param fieldValue the value of the field
     * @param valueToAdd the value to add to the collection
     * @param valueClassMetaData the ClassMetaData of the value
     */
    @SuppressWarnings("unchecked")
    private void saveWithCheckForExistence(Object fieldValue, Object valueToAdd, ClassMetaData valueClassMetaData) {
        Object valueToAddId = getIdValue(valueToAdd, valueClassMetaData);

        for (Object collectionValue : getCollection(fieldValue)) {
            Object collectionValueId = getIdValue(collectionValue, valueClassMetaData);
            if (collectionValueId.equals(valueToAddId)) {
                ((Collection) fieldValue).remove(collectionValue);
                break;
            }
        }
        ((Collection) fieldValue).add(valueToAdd);
    }

}
