/* -------------------------------------------------------------------------*
 * PKUAS: Peking University Application Server
 * Copyright (C) 2006 Peking University
 *
 * This software is free; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * either version 2.1 or any later version.
 * 
 * This software is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this software; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 * ------------------------------------------------------------------------*/

package org.ow2.xplus.operator.accessor;

import java.lang.reflect.Method;
import java.util.HashSet;
import java.util.List;
import java.util.Map;
import java.util.Set;

import javax.persistence.CascadeType;

import org.apache.commons.beanutils.DynaBean;
import org.jdom.Element;

import org.ow2.xplus.context.EntityState;
import org.ow2.xplus.context.Identity;
import org.ow2.xplus.context.PersistenceContext;
import org.ow2.xplus.metadata.object.ClassMetaData;
import org.ow2.xplus.metadata.object.FieldMetaData;
import org.ow2.xplus.metadata.relational.RdbDiscriminator;
import org.ow2.xplus.metadata.relational.RdbTable;
import org.ow2.xplus.metadata.strategy.GeneratorStrategy;
import org.ow2.xplus.query.SQLQuery;
import org.ow2.xplus.util.ClassStrategy;
import org.ow2.xplus.util.NamingStrategy;

/**
 * @author Lin Liang
 */

public class ClassAccessor extends AbstractAccessor {

    public ClassAccessor(ClassMetaData classMetaData) {
        super(classMetaData, null);
    }

    @SuppressWarnings("unchecked")
    public void obtainValue(Object entityData) {
        columns.clear();
        values.clear();
        for (FieldMetaData field : classMetaData.getAllFields()) {
            Accessor accessor = field.getAccessor();
            this.field = field;
            Object fieldData = getFieldValue(entityData, field.getFieldName());
            this.field = null;
            accessor.obtainValue(fieldData);
            columns.addAll(accessor.getColumns());
            values.addAll(accessor.getValues());
        }
    }

    @SuppressWarnings("unchecked")
    public void generatePerClassXMLData(Element root, Object data, CascadeType cascadeType, boolean generateid) {
        String tableName = classMetaData.getPrimaryTableName();
        Element tableElem = root.getChild(tableName);
        if (tableElem == null) {
            tableElem = new Element(tableName);
            root.addContent(tableElem);
        }

        RdbTable primaryTable = unit.getTable(tableName);
        for (String secTableName : primaryTable.getSecondaryTableNames()) {
            Element secTableElem = new Element(secTableName);
            root.addContent(secTableElem);
        }

        // process value
        for (FieldMetaData field : classMetaData.getAllFields()) {
            if (field.getRelation() == null) {
                Accessor accessor = field.getAccessor();
                accessor.setNeedCascadeType(cascadeType);
                this.field = field;
                Object fieldData = getFieldValue(data, field.getFieldName());
                this.field = null;
                if (field.isId()&&field.isGeneratedID()) {
                    GeneratorStrategy generator = unit.getTableIdGen(primaryTable.getName()).getGeneratorStrategy();
                    if (generator != null) {
                        if ((fieldData.equals(generator.getDefaultValue())||fieldData.toString().equalsIgnoreCase(generator.getDefaultValue().toString()))&& generateid) {
                            List result = null;
                            try {
                                String query = "SELECT " + primaryTable.getColumns().get(0).getName() + " FROM \""
                                        + primaryTable.getName() + "\"";
                                SQLQuery sqlQuery = new SQLQuery(query, null);
                                sqlQuery.setUnit(unit);
                                result = sqlQuery.getSQLResult();
                            } catch (Exception ex) {
                                String query = "SELECT " + primaryTable.getColumns().get(0).getName() + " FROM "
                                        + primaryTable.getName();
                                SQLQuery sqlQuery = new SQLQuery(query, null);
                                sqlQuery.setUnit(unit);
                                result = sqlQuery.getSQLResult();
                            }
                            if ((result != null) && (result.size() > 0)) {
                                fieldData = generator.generate();
                                this.field = field;
                                setFieldValue(data, field.getFieldName(), fieldData);
                                this.field = null;
                            }
                        }
                    }
                }
                accessor.generateXMLData(root, fieldData, generateid);
            }
        }

        // process cascade
        Element cascadeElem = new Element("cascade");
        for (FieldMetaData field : classMetaData.getAllFields()) {
            if (field.getRelation() != null) {
                List<CascadeType> cascade = field.getRelation().getCascade();
                if (cascade.contains(CascadeType.ALL) || cascade.contains(cascadeType) || cascade.contains(CascadeType.PERSIST)) {
                    Accessor accessor = field.getAccessor();
                    accessor.setNeedCascadeType(cascadeType);
                    this.field = field;
                    Object fieldData = getFieldValue(data, field.getFieldName());
                    this.field = null;
                    accessor.generateCascadeXMLData(cascadeElem, fieldData, generateid);
                }
            }
        }

        List<Element> cascadeDataElems = (List<Element>) cascadeElem.removeContent();
        for (Element cascadeDataElem : cascadeDataElems) {
            List<Element> dataElems = (List<Element>) cascadeDataElem.removeContent();
            for (Element dataElem : dataElems) {
                root.addContent(dataElem);
            }
        }

        // process relation
        for (FieldMetaData field : classMetaData.getAllFields()) {
            if (field.getRelation() != null) {
                Accessor accessor = field.getAccessor();
                this.field = field;
                Object fieldData = getFieldValue(data, field.getFieldName());
                this.field = null;
                accessor.generateXMLData(root, fieldData, generateid);
            }
        }
    }

    private void generatePerClassRelationXMLData(Element root, Element relationRoot, Object data,
            Map<String, List<String>> keyColumns) {
        for (FieldMetaData field : classMetaData.getAllFields()) {
            if (field.getRelation() != null) {
                Accessor accessor = field.getAccessor();
                this.field = field;
                Object fieldData = getFieldValue(data, field.getFieldName());
                this.field = null;
                if (accessor instanceof RelationTableAccessor) {
                    RelationTableAccessor relationTableAccessor = (RelationTableAccessor) accessor;
                    relationTableAccessor.generateRelationXMLData(root, relationRoot, fieldData, keyColumns);
                }
            }
        }
    }

    public void getDataXML(Element root, Element relationRoot, Object data, Map<String, List<String>> keyColumns,
            boolean generateid) {
        generateXMLData(root, data, generateid);

        generatePerClassRelationXMLData(root, relationRoot, data, keyColumns);

        if (classMetaData.getParent() != null) {
            ClassMetaData parentMetaData = unit.getClassMetaData(classMetaData.getParent().getCanonicalName());
            ClassAccessor parentAccessor = new ClassAccessor(parentMetaData);
            parentAccessor.setNeedCascadeType(needCascadeType);
            parentAccessor.generatePerClassRelationXMLData(root, relationRoot, data, keyColumns);
        }
    }

    @Override
    public void generateXMLData(Element root, Object data, boolean generateid) {
        generatePerClassXMLData(root, data, needCascadeType, generateid);

        if (classMetaData.getParent() != null) {
            ClassMetaData parentMetaData = unit.getClassMetaData(classMetaData.getParent().getCanonicalName());
            ClassAccessor parentAccessor = new ClassAccessor(parentMetaData);
            parentAccessor.setNeedCascadeType(needCascadeType);
            parentAccessor.generateXMLData(root, data, generateid);
        }

        RdbTable primaryTable = unit.getTable(classMetaData.getPrimaryTableName());
        RdbDiscriminator discriminator = primaryTable.getDiscriminator();
        if (discriminator != null) {
            String columnName = discriminator.getDiscriminatorColumnName();
            String discriminatorValue = classMetaData.getDiscriminatorValue();
            if (discriminatorValue == null)
                discriminatorValue = discriminator.getDiscriminatorStrategy().getValue(data.getClass().getName()).toString();

            Element tableElem = root.getChild(primaryTable.getName());
            if (tableElem == null) {
                tableElem = new Element(primaryTable.getName());
                root.addContent(tableElem);
            }

            tableElem.setAttribute(columnName, discriminatorValue);
        }
    }

    @Override
    public Set<String> getAffectTableNames(boolean cascade) {
        Set<String> affectTableNames = new HashSet<String>();
        affectTableNames.add(classMetaData.getPrimaryTableName());
        RdbTable primaryTable = unit.getTable(classMetaData.getPrimaryTableName());
        affectTableNames.addAll(primaryTable.getSecondaryTableNames());

        for (FieldMetaData field : classMetaData.getAllFields()) {
            if (field.getRelation() != null) {
                Accessor accessor = field.getAccessor();
                affectTableNames.addAll(accessor.getAffectTableNames(cascade));
            }
        }

        return affectTableNames;
    }

    private ClassMetaData findClassMetadata(ClassMetaData entityMetaData, DynaBean bean) {
        RdbTable primaryTable = unit.getTable(entityMetaData.getPrimaryTableName());
        RdbDiscriminator discriminator = primaryTable.getDiscriminator();
        if (discriminator != null) {
            String columnName = discriminator.getDiscriminatorColumnName();
            String discriminatorValue = entityMetaData.getDiscriminatorValue();
            if (bean.get(columnName.toUpperCase()).equals(discriminatorValue))
                return entityMetaData;
            return findClassMetadataInsubClasses(entityMetaData, bean);
        }
        return null;
    }

    private ClassMetaData findClassMetadataInsubClasses(ClassMetaData entityMetaData, DynaBean bean) {
        List<String> classNames = entityMetaData.getSubClasseNames();
        for (String className : classNames) {
            ClassMetaData resultMetaData = findClassMetadata(unit.getClassMetaData(className), bean);
            if (resultMetaData != null)
                return resultMetaData;
        }
        return null;
    }

    public Object create(DynaBean bean, PersistenceContext context,Object entityData) {
        ClassMetaData entityMetaData = findClassMetadata(classMetaData, bean);
        if (entityMetaData != null)
            classMetaData = entityMetaData;

        entityData = perClassCreate(bean, context, entityData);

        if (classMetaData.getParent() != null) {
            ClassMetaData parentMetaData = unit.getClassMetaData(classMetaData.getParent().getCanonicalName());
            ClassAccessor parentAccessor = new ClassAccessor(parentMetaData);
            entityData = parentAccessor.createParent(bean, context, entityData);
        }

        return entityData;
    }
    
    public Object createParent(DynaBean bean, PersistenceContext context,Object entityData) {
    	  entityData = perClassCreate(bean, context, entityData);

          if (classMetaData.getParent() != null) {
              ClassMetaData parentMetaData = unit.getClassMetaData(classMetaData.getParent().getCanonicalName());
              ClassAccessor parentAccessor = new ClassAccessor(parentMetaData);
              entityData = parentAccessor.createParent(bean, context, entityData);
          }

          return entityData;
    }
    
    public Object create(Object oentity,PersistenceContext context,Object entityData){
    	
    	if(oentity == null)
    		return null;
    	
    	entityData = perClassCreate(oentity,context,entityData);
    	
    	  if (classMetaData.getParent() != null) {
              ClassMetaData parentMetaData = unit.getClassMetaData(classMetaData.getParent().getCanonicalName());
              ClassAccessor parentAccessor = new ClassAccessor(parentMetaData);
              entityData = parentAccessor.create(oentity, context, entityData); 
          }
    	  return entityData;
    }
    

    private Object perClassCreate(DynaBean bean, PersistenceContext context, Object entityData) {
        if (entityData == null)
            entityData = initialize(bean, context);
        if (entityData == null)
            return null;

        Identity id = getIdentity(bean);
        // save with a 0 hashcode
        context.addEntity(id, entityData, new Integer(0));

        EntityState state = context.getEntityState(id);
        if (!state.equals(context.getStateManager().getFetchState())) {
            context.setEntityState(id, state.PreLoad());
        } else {
            return entityData;
        }

        for (FieldMetaData field : classMetaData.getAllFields()) {
            if (field.getAccessor() != null) {
                field.getAccessor().save(entityData, field.getAccessor().create(bean, context,null));
            }
        }
        
        context.setEntityState(id, context.getStateManager().getCascadeFindState());
        context.addEntity(id, entityData, getHashCode(entityData));

        return entityData;
    }
    
    private Object perClassCreate(Object oentity,PersistenceContext context , Object entityData){
    	if(entityData == null)
    		entityData = initialize(oentity,context);
    	if(entityData == null)
    		return null;
    	for(FieldMetaData field : classMetaData.getAllFields()) {
    		if(field.getAccessor() != null){
    			field.getAccessor().save4Copy(entityData, field.getAccessor().create(oentity,context,null));
    		}
    	}
    	return entityData;
    }
    /**
     * @param entity entity instance
     * @return hashcode
     */
    private Integer getHashCode(Object entity) {
        int hash = 0;
        ClassMetaData entityMetaData = unit.findClassMetaData(entity.getClass().getCanonicalName());
        Method[] methods = entity.getClass().getMethods();
        for (FieldMetaData field : entityMetaData.getDeclaredFields()) {
            String getMethodName = NamingStrategy.getFieldAccessMethod(field.getFieldName(), "get");
            for (Method method : methods) {
                if (method.getName().equals(getMethodName)) {
                    try {
                        Object value = method.invoke(entity, new Object[] {});
                        hash = hash * 17 + value.hashCode();
                    } catch (Exception e) {
                    }
                }
            }
        }
        return new Integer(hash);
    }

    private Identity getIdentity(DynaBean bean) {
        Identity id = null;

        for (FieldMetaData field : classMetaData.getDeclaredFields()) {
            if (field.isId()) {
                Accessor constructor = field.getAccessor();
                Object key = constructor.create(bean, null,null);
                id = new Identity(key, classMetaData.getEntityName());
                break;
            }
        }

        return id;
    }

    public boolean save(Object entityData, Object fieldValue) {
        // nothing
        return false;
    }
    
    public boolean save4Copy(Object entityData,Object fieldValue){
    	return false;
    }

    private Object initialize(DynaBean bean, PersistenceContext context) {
        Object entityData = null;

        Identity id = getIdentity(bean);
        entityData = context.getEntity(id);

        if (entityData == null) {
            Class entityClass = ClassStrategy.forName(classMetaData.getEntityName());
            try {
                entityData = entityClass.newInstance();
            } catch (InstantiationException e) {
                e.printStackTrace();
            } catch (IllegalAccessException e) {
                e.printStackTrace();
            }
        }

        return entityData;
    }
    private Object initialize(Object oentity,PersistenceContext context){
    	Object entityData = null;
    	if(entityData == null) {
    		Class entityClass = ClassStrategy.forName(classMetaData.getEntityName());
    			try {
					entityData = entityClass.newInstance();
				} catch (InstantiationException e) {
					// TODO Auto-generated catch block
					e.printStackTrace();
				} catch (IllegalAccessException e) {
					// TODO Auto-generated catch block
					e.printStackTrace();
				}
    	}
    	return entityData;
    }
    
}
