/* -------------------------------------------------------------------------*
 * PKUAS: Peking University Application Server
 * Copyright (C) 2006 Peking University
 *
 * This software is free; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * either version 2.1 or any later version.
 * 
 * This software is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this software; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 * ------------------------------------------------------------------------*/

package org.ow2.xplus.operator.accessor;

import java.util.HashSet;
import java.util.Set;

import org.apache.commons.beanutils.DynaBean;

import org.ow2.xplus.context.PersistenceContext;
import org.ow2.xplus.metadata.object.ClassMetaData;
import org.ow2.xplus.metadata.object.FieldMetaData;
import org.ow2.xplus.metadata.relational.RdbTable;

/**
 * @author Lin Liang
 */

public class DirectCollectionAccessor extends AbstractCollectionAccessor {

    public DirectCollectionAccessor(ClassMetaData classMetaData, FieldMetaData field) {
        super(classMetaData, field);
    }

    @SuppressWarnings("unchecked")
    public void obtainValue(Object fieldData) {
        if (fieldData == null)
            return;
        columns.clear();
        values.clear();
        Class referencedEntity = field.getType();
        ClassMetaData referencedEntityMetaData = unit.getClassMetaData(referencedEntity.getCanonicalName());
        String collectionTableName = field.getValue().getCollectionValue().getCollectionTableName();
        RdbTable collectionTable = unit.getTable(collectionTableName);

        for (FieldMetaData field : referencedEntityMetaData.getAllFields()) {
            if (field.isId()) {
                Accessor accessor = field.getAccessor();
                accessor.obtainValue(getFieldValue(fieldData, field.getFieldName()));
                overrideColumn(accessor);
                columns.clear();
                values.clear();
                columns.addAll(accessor.getColumns());
                values.addAll(accessor.getValues());
            }
        }

        for (Object collectionData : visitor.getCollection(fieldData)) {
            columns.add(collectionTable.getColumn(field.getFieldName()));
            values.add(collectionData.toString());
        }
    }

    @Override
    public Set<String> getAffectTableNames(boolean cascade) {
        Set<String> affectTableNames = new HashSet<String>();
        String collectionTableName = field.getValue().getCollectionValue().getCollectionTableName();
        affectTableNames.add(collectionTableName);
        return affectTableNames;
    }

    public Object create(DynaBean bean, PersistenceContext context,Object entityData) {
        String columnName = field.getValue().getSimpleValue().getColumns().get(0).getName();

        return bean.get(columnName);
    }
    public Object create(Object oentity,PersistenceContext context,Object entityData) {
    	String fieldName = field.getFieldName();
    	return getFieldValue(oentity,fieldName);
    }
}
