/* -------------------------------------------------------------------------*
 * PKUAS: Peking University Application Server
 * Copyright (C) 2006 Peking University
 *
 * This software is free; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * either version 2.1 or any later version.
 * 
 * This software is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this software; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 * ------------------------------------------------------------------------*/

package org.ow2.xplus.operator.accessor;

import org.apache.commons.beanutils.DynaBean;

import org.ow2.xplus.context.PersistenceContext;
import org.ow2.xplus.metadata.object.ClassMetaData;
import org.ow2.xplus.metadata.object.FieldMetaData;

/**
 * @author Lin Liang
 */

public class EmbeddedAccessor extends AbstractAccessor {

    private ClassMetaData embeddedClassMetaData;

    public EmbeddedAccessor(ClassMetaData classMetaData, FieldMetaData field) {
        super(classMetaData, field);
        embeddedClassMetaData = unit.getClassMetaData(field.getType().getCanonicalName());
    }

    @SuppressWarnings("unchecked")
    public void obtainValue(Object fieldData) {
        if (fieldData == null)
            return;
        ClassAccessor classAccessor = new ClassAccessor(embeddedClassMetaData);
        classAccessor.setNeedCascadeType(needCascadeType);
        classAccessor.obtainValue(fieldData);
        overrideColumn(classAccessor);
        columns.clear();
        values.clear();
        columns.addAll(classAccessor.getColumns());
        values.addAll(classAccessor.getValues());
    }

    public Object create(DynaBean bean, PersistenceContext context,Object entityData) {
        ClassAccessor classAccessor = new ClassAccessor(embeddedClassMetaData);
        return classAccessor.create(bean, context,entityData);
    }
    
    public Object create(Object oentity,PersistenceContext context,Object entityData) {
    	ClassAccessor classAccessor = new ClassAccessor(embeddedClassMetaData);
    	Object entity = getFieldValue(oentity,field.getFieldName());
    	return classAccessor.create(entity, context,entityData);
    }

    public boolean save(Object entityData, Object fieldValue) {
        if (fieldValue == null)
            return false;
        Object value = getFieldValue(entityData, field.getFieldName());
        if (!fieldValue.equals(value)) {
            setFieldValue(entityData, field.getFieldName(), fieldValue);
            return true;
        }
        return false;
    }
    public boolean save4Copy(Object entityData , Object fieldValue){
    	if (fieldValue == null)
    		return false;
    	Object value = getFieldValue(entityData , field.getFieldName());
    	if(!fieldValue.equals(value)){
    		setFieldValue4Copy(entityData,field.getFieldName(),fieldValue);
    		return true;
    	}
    	return false;
    }
}
