/* -------------------------------------------------------------------------*
 * PKUAS: Peking University Application Server
 * Copyright (C) 2006 Peking University
 *
 * This software is free; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * either version 2.1 or any later version.
 * 
 * This software is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this software; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 * ------------------------------------------------------------------------*/

package org.ow2.xplus.operator.accessor;

import java.util.HashSet;
import java.util.List;
import java.util.Set;

import org.apache.commons.beanutils.DynaBean;
import org.jdom.Element;

import org.ow2.xplus.context.PersistenceContext;
import org.ow2.xplus.metadata.object.ClassMetaData;
import org.ow2.xplus.metadata.object.FieldMetaData;
import org.ow2.xplus.metadata.relational.RdbColumn;
import org.ow2.xplus.metadata.relational.RdbForeignKey;
import org.ow2.xplus.metadata.relational.RdbTable;

/**
 * @author Lin Liang
 */

public class ForeignKeyAccessor extends AbstractAccessor {

    private ClassMetaData referencedEntityMetaData;

    public ForeignKeyAccessor(ClassMetaData classMetaData, FieldMetaData field) {
        super(classMetaData, field);
        Class referencedEntity = field.getType();
        referencedEntityMetaData = unit.getClassMetaData(referencedEntity.getCanonicalName());
    }

    @SuppressWarnings("unchecked")
    public void obtainValue(Object fieldData) {
        if (fieldData == null)
            return;

        for (FieldMetaData field : referencedEntityMetaData.getAllFields()) {
            if (field.isId()) {
                Accessor accessor = field.getAccessor();
                accessor.obtainValue(getFieldValue(fieldData, field.getFieldName()));
                overrideColumn(accessor);
                columns.clear();
                values.clear();
                columns.addAll(accessor.getColumns());
                values.addAll(accessor.getValues());
                return;
            }
        }
    }

    @Override
    public void generateXMLData(Element root, Object data, boolean generateid) {
        if (data == null)
            return;
        obtainValue(data);

        RdbTable primaryTable = unit.getTable(classMetaData.getPrimaryTableName());
        RdbForeignKey foreignKey = (RdbForeignKey) primaryTable.getRelation(field.getFieldName());

        List<String> joinColumnNames = foreignKey.getJoinColumnNames();
        List<String> referencedColumnNames = foreignKey.getReferencedColumnNames();

        int valuesSize = values.size();
        for (int index = 0; index < columns.size(); ++index) {
            RdbColumn column = columns.get(index);
            Object columnData = null;
            if (index < valuesSize)
                columnData = values.get(index);
            else
                break;
            String tableName = classMetaData.getPrimaryTableName();
            Element tableElem = root.getChild(tableName);
            String columnName = joinColumnNames.get(referencedColumnNames.indexOf(column.getName()));
            tableElem.setAttribute(columnName, columnData.toString());
        }
    }

    @Override
    public void generateCascadeXMLData(Element root, Object fieldData, boolean generateid) {
        if (fieldData == null)
            return;
        ClassAccessor cascadeClass = new ClassAccessor(referencedEntityMetaData);
        cascadeClass.setNeedCascadeType(needCascadeType);
        cascadeClass.obtainValue(fieldData);
        Element cascadeData = new Element("data");
        cascadeClass.generateXMLData(cascadeData, fieldData, generateid);
        root.addContent(cascadeData);
    }

    @Override
    public Set<String> getAffectTableNames(boolean cascade) {
        Set<String> affectTableNames = new HashSet<String>();
        if (cascade) {
            ClassAccessor cascadeClass = new ClassAccessor(referencedEntityMetaData);
            affectTableNames.addAll(cascadeClass.getAffectTableNames(cascade));
        }
        return affectTableNames;
    }

    public Object create(DynaBean bean, PersistenceContext context,Object entityData) {
        ClassAccessor classAccessor = new ClassAccessor(referencedEntityMetaData);
        return classAccessor.create(bean, context,entityData);
    }
    
    public Object create(Object oentity,PersistenceContext context,Object entityData) {
    	ClassAccessor classAccessor = new ClassAccessor(referencedEntityMetaData);
    	Object entity = getFieldValue(oentity,field.getFieldName());
    	return classAccessor.create(entity, context,entityData);
    }

    public boolean save(Object entityData, Object fieldValue) {
        if (fieldValue == null)
            return false;
        Object value = getFieldValue(entityData, field.getFieldName());
        if (!fieldValue.equals(value)) {
            setFieldValue(entityData, field.getFieldName(), fieldValue);
            return true;
        }
        return false;
    }
    public boolean save4Copy(Object entityData , Object fieldValue){
    	if (fieldValue == null)
    		return false;
    	Object value = getFieldValue(entityData , field.getFieldName());
    	if(!fieldValue.equals(value)){
    		setFieldValue4Copy(entityData,field.getFieldName(),fieldValue);
    		return true;
    	}
    	return false;
    }
}
