/* -------------------------------------------------------------------------*
 * PKUAS: Peking University Application Server
 * Copyright (C) 2006 Peking University
 *
 * This software is free; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * either version 2.1 or any later version.
 * 
 * This software is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this software; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 * ------------------------------------------------------------------------*/

package org.ow2.xplus.operator.accessor;

import java.util.ArrayList;
import java.util.HashSet;
import java.util.List;
import java.util.Map;
import java.util.Set;

import org.apache.commons.beanutils.DynaBean;
import org.jdom.Attribute;
import org.jdom.Element;

import org.ow2.xplus.context.PersistenceContext;
import org.ow2.xplus.metadata.object.ClassMetaData;
import org.ow2.xplus.metadata.object.FieldMetaData;
import org.ow2.xplus.metadata.relational.RdbColumn;
import org.ow2.xplus.metadata.relational.RdbRelationTable;
import org.ow2.xplus.metadata.relational.RdbTable;

/**
 * @author Lin Liang
 */

public class RelationTableAccessor extends AbstractCollectionAccessor {

    private ClassMetaData referencedEntityMetaData;

    public RelationTableAccessor(ClassMetaData classMetaData, FieldMetaData field) {
        super(classMetaData, field);
        Class referencedEntity = field.getType();
        referencedEntityMetaData = unit.getClassMetaData(referencedEntity.getCanonicalName());
    }

    public void obtainValue(Object fieldData) {
        if (fieldData == null)
            return;
        columns.clear();
        values.clear();
    }

    @SuppressWarnings("unchecked")
    @Override
    public void generateXMLData(Element root, Object fieldData, boolean generateid) {
        if (fieldData == null)
            return;
        obtainValue(fieldData);

        RdbTable primaryTable = unit.getTable(classMetaData.getPrimaryTableName());
        RdbRelationTable relationTable = (RdbRelationTable) primaryTable.getRelation(field.getFieldName());

        Class referencedEntity = field.getType();
        ClassMetaData referencedEntityMetaData = unit.getClassMetaData(referencedEntity.getCanonicalName());
        // ClassAccessor cascadeClass = new
        // ClassAccessor(referencedEntityMetaData);
        for (Object collectionData : visitor.getCollection(fieldData)) {
            Element relationTableElem = buildElement(root);

            columns.clear();
            values.clear();

            for (FieldMetaData field : referencedEntityMetaData.getAllFields()) {
                // if (field.isId()) {
                Accessor accessor = field.getAccessor();
                accessor.obtainValue(getFieldValue(collectionData, field.getFieldName()));

                columns.addAll(accessor.getColumns());
                values.addAll(accessor.getValues());
                // }
            }

            String checkColumnName = "";

            int valuesSize = values.size();
            for (int index = 0; index < columns.size(); ++index) {
                RdbColumn column = columns.get(index);
                Object value = null;
                if (index < valuesSize)
                    value = values.get(index);
                else {
                    break;
                }
                String targetRelationColumnName = relationTable.getTargetRelationColumnName(column.getName());
                if (targetRelationColumnName == null)
                    targetRelationColumnName = column.getName();

                RdbColumn targetRelationColumn = relationTable.getRelationTable().getColumn(targetRelationColumnName);
                if (targetRelationColumn != null) {
                    relationTableElem.setAttribute(targetRelationColumnName, value.toString());
                    if (targetRelationColumn.isUnique() || targetRelationColumn.isPrimaryKey())
                        checkColumnName = targetRelationColumnName;
                }
            }

            // check whether there is a desciption for this entity already
            for (Object child : root.getChildren()) {
                Element elem = (Element) child;
                if (elem.getName().equals(relationTableElem.getName())) {
                    if (elem.getAttribute(checkColumnName) == null)
                        continue;
                    if (elem.getAttribute(checkColumnName).getValue().equals(
                            relationTableElem.getAttribute(checkColumnName).getValue())) {
                        for (Object attribute : relationTableElem.getAttributes()) {
                            elem.setAttribute(((Attribute) attribute).getName(), ((Attribute) attribute).getValue());
                        }
                        return;
                    }
                }
            }
            // if not, add a new one
            root.addContent(relationTableElem);
        }
    }

    @SuppressWarnings("unchecked")
    public void generateRelationXMLData(Element root, Element relationRoot, Object fieldData, Map<String, List<String>> keyColumns) {
        if (fieldData == null)
            return;
        obtainValue(fieldData);

        RdbTable primaryTable = unit.getTable(classMetaData.getPrimaryTableName());
        RdbRelationTable relationTable = (RdbRelationTable) primaryTable.getRelation(field.getFieldName());
        RdbTable table = relationTable.getRelationTable();
        Element relationTableElem = new Element(table.getName());

        Element tableElem = root.getChild(primaryTable.getName());
        List<String> columnNames = new ArrayList<String>();
        for (RdbColumn column : primaryTable.getColumns()) {
            if (column.isPrimaryKey()) {
                String sourceRelationColumn = relationTable.getSourceRelationColumnName(column.getName());
                relationTableElem.setAttribute(sourceRelationColumn, tableElem.getAttribute(column.getName()).getValue());
                columnNames.add(sourceRelationColumn);
            }
        }
        keyColumns.put(table.getName(), columnNames);
        relationRoot.addContent(relationTableElem);

    }

    private Element buildElement(Element root) {
        RdbTable primaryTable = unit.getTable(classMetaData.getPrimaryTableName());
        RdbRelationTable relationTable = (RdbRelationTable) primaryTable.getRelation(field.getFieldName());
        RdbTable table = relationTable.getRelationTable();
        Element relationTableElem = new Element(table.getName());

        Element tableElem = root.getChild(primaryTable.getName());
        for (RdbColumn column : primaryTable.getColumns()) {
            if (column.isPrimaryKey()) {
                String sourceRelationColumn = relationTable.getSourceRelationColumnName(column.getName());
                relationTableElem.setAttribute(sourceRelationColumn, tableElem.getAttribute(column.getName()).getValue());
            }
        }

        return relationTableElem;
    }

    @Override
    public void generateCascadeXMLData(Element root, Object fieldData, boolean generateid) {
        if (fieldData == null)
            return;

        Class referencedEntity = field.getType();
        ClassMetaData referencedEntityMetaData = unit.getClassMetaData(referencedEntity.getCanonicalName());

        ClassAccessor cascadeClass = new ClassAccessor(referencedEntityMetaData);
        cascadeClass.setNeedCascadeType(needCascadeType);

        for (Object collectionData : visitor.getCollection(fieldData)) {
            cascadeClass.obtainValue(collectionData);
            Element cascadeData = new Element("data");
            cascadeClass.generateXMLData(cascadeData, collectionData, generateid);
            root.addContent(cascadeData);
        }
    }

    @Override
    public Set<String> getAffectTableNames(boolean cascade) {
        Set<String> affectTableNames = new HashSet<String>();
        RdbTable primaryTable = unit.getTable(classMetaData.getPrimaryTableName());
        RdbRelationTable relationTable = (RdbRelationTable) primaryTable.getRelation(field.getFieldName());
        affectTableNames.add(relationTable.getRelationTable().getCanonicalName());

        if (cascade) {
            Class referencedEntity = field.getType();
            ClassMetaData referencedEntityMetaData = unit.getClassMetaData(referencedEntity.getCanonicalName());
            ClassAccessor cascadeClass = new ClassAccessor(referencedEntityMetaData);
            affectTableNames.addAll(cascadeClass.getAffectTableNames(cascade));
        }
        return affectTableNames;
    }

    public Object create(DynaBean bean, PersistenceContext context,Object entityData) {
        ClassAccessor classAccessor = new ClassAccessor(referencedEntityMetaData);
        return classAccessor.create(bean, context,entityData);
    }
    
    public Object create(Object oentity,PersistenceContext context,Object entityData) {
    	ClassAccessor classAccessor = new ClassAccessor(referencedEntityMetaData);
    	Object entity = getFieldValue(oentity,field.getFieldName());
    	return classAccessor.create(entity, context,entityData);
    }
}
