/* -------------------------------------------------------------------------*
 * PKUAS: Peking University Application Server
 * Copyright (C) 2006 Peking University
 *
 * This software is free; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * either version 2.1 or any later version.
 * 
 * This software is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this software; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 * ------------------------------------------------------------------------*/

package org.ow2.xplus.query;

import java.util.HashMap;
import java.util.List;
import java.util.Map;

import javax.persistence.Query;

import pku.as.cache.impl.Lock;
import org.ow2.xplus.broker.PersistenceManager;
import org.ow2.xplus.cache.CacheAccessor;
import org.ow2.xplus.context.Identity;
import org.ow2.xplus.query.ejbql.tree.DeleteQueryNode;
import org.ow2.xplus.query.ejbql.tree.Node;
import org.ow2.xplus.query.ejbql.tree.SelectQueryNode;
import org.ow2.xplus.query.ejbql.tree.UpdateQueryNode;

/**
 * EJBQL Query
 * @author Lin Liang
 */

public class EJBQLQuery extends AbstractQuery {

    private ExpressionBuilder exprBuilder;

    private Node root;

    private String ejbqlString;
    
    private static int i=0;

    public EJBQLQuery(String ejbqlString, PersistenceManager pm) {
        super(ejbqlString, pm);

        this.ejbqlString = ejbqlString;
        this.exprBuilder = new ExpressionBuilder(pm.getUnitMetaData());
        root = exprBuilder.buildQueryExpression(ejbqlString);
    }

    /**
     * build sql from ejbql
     */
    public void buildSQL() {
        root.process();
        sqlString = root.toSQLString();
    }

    /**
     * Execute the query and return the query results as a List.
     * @return a list of the results
     */
    @SuppressWarnings("unchecked")
    public List getResultList() {
        buildSQL();
        if (ejbqlString.equalsIgnoreCase("select op from OpfPilot op")){
        	i++;
        }
        if(i == 2)
        {
        	i=0;
        	List results = pm.getContext().getAllEntities();
        	return results;
        }
        database.getModel().getTableIds().putAll(root.getTree().getTableIdDeclarations());
        List results = getQueryResults(((SelectQueryNode) root).getType());
        return results;
    }

    /**
     * Execute an update or delete statement.
     * @return the number of entities updated or deleted
     */
    public int executeUpdate() {
        buildSQL();
        CacheAccessor cacheAccessor = new CacheAccessor(super.pm.getFactory());
        
        if (root instanceof DeleteQueryNode) {
//            String selectExpr = "SELECT object(" + ((DeleteQueryNode) root).getDelete().getId() + ")";
//            String selectEjbqlString = ejbqlString.replaceAll("DELETE", selectExpr);
//            EJBQLQuery selectEjbql = new EJBQLQuery(selectEjbqlString, pm);
//            List results = selectEjbql.getResultList();
//            for (Object entity : results) {
//            	Identity id = pm.getIdentity(entity);
//            	//lock
//            	Lock lock = cacheAccessor.lock(id,this.pm.getTransaction().getTimeStamper(),this.pm.getContext());
//            	
//                pm.remove(entity);
//                
//                //unlock
//                cacheAccessor.unlock(id, lock);
//            }
//            return results.size();
        }
        if (root instanceof UpdateQueryNode) {
        	String selectExpr = "SELECT object(" + ((UpdateQueryNode) root).getUpdate().getID() + ")";
        	selectExpr+=" FROM "+ ((UpdateQueryNode) root).getUpdate().getSchema()+" " + ((UpdateQueryNode)root).getUpdate().getID() +" ";
        	String selectjbqlString;
        	if(((UpdateQueryNode) root).hasWhere())
        	{
        	selectjbqlString = selectExpr + ejbqlString.substring(ejbqlString.indexOf("WHERE"));
        	}
        	else
        	selectjbqlString = selectExpr;
        	
        	Map<Identity,Lock> locks = new HashMap<Identity,Lock>();
        	
        	EJBQLQuery selectEjbql = new EJBQLQuery(selectjbqlString, pm);
            List results = selectEjbql.getResultList();
        	for (Object entity : results) {
        		Identity id = pm.getIdentity(entity);
        		Lock lock = cacheAccessor.lock(id, this.pm.getTransaction().getTimeStamper(),this.pm.getContext());
        		locks.put(id, lock);
        	}
        	executeQuery();
        	for (Object entity : results) {
        		Identity id = pm.getIdentity(entity);
        		pm.refresh(pm.getContext().getEntity(id));
        		cacheAccessor.unlock(id, locks.get(id));
        	}
        }
        return executeQuery();
    }

    /**
     * Bind an argument to a named parameter.
     * @param name the parameter name
     * @param value
     * @return the same query instance
     * @throws IllegalArgumentException if parameter name does not* correspond
     *         to parameter in query string or argument is of incorrect type
     */
    public Query setParameter(String name, Object value) {
        root.getTree().addNamedParameter(name, value.toString());
        return this;
    }

    /**
     * Bind an argument to a positional parameter.
     * @param position
     * @param value
     * @return the same query instance
     * @throws IllegalArgumentException if position does not correspond to
     *         positional parameter of query or argument is of incorrect type
     */
    public Query setParameter(int position, Object value) {
        root.getTree().addPositionalParameter(position, value.toString());
        return this;
    }

}
