/* -------------------------------------------------------------------------*
 * PKUAS: Peking University Application Server
 * Copyright (C) 2006 Peking University
 *
 * This software is free; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * either version 2.1 or any later version.
 * 
 * This software is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this software; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 * ------------------------------------------------------------------------*/

package org.ow2.xplus.query;

import java.util.ArrayList;
import java.util.List;

import javax.persistence.Query;

import org.apache.commons.beanutils.DynaBean;
import org.apache.commons.beanutils.DynaProperty;
import org.apache.ddlutils.platform.ModelBasedResultSetIterator;

import org.ow2.xplus.broker.PersistenceManager;
import org.ow2.xplus.metadata.query.FieldMapping;
import org.ow2.xplus.metadata.query.ResultSetMapping;

/**
 * @author Lin Liang
 */

public class SQLQuery extends AbstractQuery {

    private Class resultClass;

    private ResultSetMapping resultSetMapping;

    private String propertyName;

    public SQLQuery(String sqlString, PersistenceManager pm) {
        super(sqlString, pm);
        this.sqlString = sqlString;
    }

    public Query setParameter(String name, Object value) {
        String para = ":" + name;
        sqlString = sqlString.replaceAll(para, value.toString());
        return this;
    }

    public Query setParameter(int position, Object value) {
        String para = "\\?" + position;
        sqlString = sqlString.replaceAll(para, value.toString());
        if (position == 1) {
            sqlString = sqlString.replaceAll("\\?", value.toString());
        }
        return this;
    }

    public String getSQL() {
        return sqlString;
    }

    public List getResultList() {
        List results = null;
        if (resultSetMapping != null) {
            for (FieldMapping field : resultSetMapping.getEntities().get(0).getFields()) {
                getColumnsToProperties().put(field.getColumn().toUpperCase(), field.getName().toUpperCase());
            }
            results = getQueryResults(resultSetMapping.getEntities().get(0).getEntityClass());
            return results;
        }
        if (resultClass != null) {
            results = getQueryResults(resultClass);
            return results;
        }
        results = getQueryResults(Object.class);
        return results;
    }

    public int executeUpdate() {
        return executeQuery();
    }

    public Class getResultClass() {
        return resultClass;
    }

    public void setResultClass(Class resultClass) {
        this.resultClass = resultClass;
    }

    public ResultSetMapping getResultSetMapping() {
        return resultSetMapping;
    }

    public void setResultSetMapping(ResultSetMapping resultSetMapping) {
        this.resultSetMapping = resultSetMapping;
    }

    @SuppressWarnings("unchecked")
    public List getSQLResult() {
        List results = new ArrayList();
        // execute the query
        ModelBasedResultSetIterator resultIter = (ModelBasedResultSetIterator) database.getPlatform().query(database.getModel(),
                sqlString);
        while (resultIter.hasNext()) {
            DynaBean bean = (DynaBean) resultIter.next();
            if (propertyName == null) {
                DynaProperty[] props = bean.getDynaClass().getDynaProperties();
                propertyName = props[0].getName();
            }
            results.add(bean.get(propertyName));
        }
        return results;
    }

    public void setPropertyName(String propertyName) {
        this.propertyName = propertyName;
    }

}
