/* -------------------------------------------------------------------------*
 * PKUAS: Peking University Application Server
 * Copyright (C) 2006 Peking University
 *
 * This software is free; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * either version 2.1 or any later version.
 * 
 * This software is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this software; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 * ------------------------------------------------------------------------*/

package org.ow2.xplus.query.ejbql;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.HashSet;
import java.util.List;
import java.util.Map;
import java.util.Set;

import org.ow2.xplus.metadata.UnitMetaData;
import org.ow2.xplus.query.dialect.Dialect;
import org.ow2.xplus.query.dialect.DialectFactory;
import org.ow2.xplus.query.ejbql.tree.QueryNode;

/**
 * @author Lin Liang
 */

public class EJBQLTree {

    private UnitMetaData unit;

    private QueryNode root;

    private NodeFactory factory;

    /**
     * Map<the name of id, the class type of id>
     */
    private Map<String, Class> idDeclarations = new HashMap<String, Class>();

    /**
     * Map<the name of id, the name of table>
     */
    private Map<String, String> tableIdDeclarations = new HashMap<String, String>();

    /**
     * the path that already parsed;
     */
    private Set<String> parsedPaths = new HashSet<String>();

    /**
     * Map<the name of para, the value string>
     */
    private Map<String, String> namedParameters = new HashMap<String, String>();

    /**
     * the values of positionalParameters
     */
    private List<String> positionalParameters = new ArrayList<String>();

    /**
     * Map<the name of the table, the name of id>
     */
    private Map<String, String> tableDeclarations = new HashMap<String, String>();

    private Dialect dialect = null;

    public EJBQLTree(UnitMetaData unit) {
        this.unit = unit;
        this.dialect = DialectFactory.getDialect(unit.getDatabaseDialect());
    }

    public QueryNode getRoot() {
        return root;
    }

    public void setRoot(QueryNode root) {
        this.root = root;
    }

    /**
     * add id declaration
     * @param ident the id var
     * @param idField the class type of this id
     */
    public void addDeclaration(String ident, Class idClass) {
        idDeclarations.put(ident, idClass);
        String tableName = unit.findClassMetaData(idClass.getCanonicalName()).getPrimaryTableName();
        tableDeclarations.put(tableName, ident);
        tableIdDeclarations.put(ident, tableName);
    }

    public Class getIdType(String ident) {
        return idDeclarations.get(ident);
    }

    public String getTableId(String ident) {
        return tableDeclarations.get(ident);
    }

    public void addParsedPath(String parsedPath) {
        parsedPaths.add(parsedPath);
    }

    public UnitMetaData getUnit() {
        return unit;
    }

    public NodeFactory getFactory() {
        return factory;
    }

    public void setFactory(NodeFactory factory) {
        this.factory = factory;
    }

    /**
     * clear all parameter values set before
     */
    public void clearParmeters() {
        namedParameters.clear();
        positionalParameters.clear();
    }

    /**
     * add the value of Named Parameter
     * @param name the name of the para
     * @param value the value string
     */
    public void addNamedParameter(String name, String value) {
        namedParameters.put(name, value);
    }

    /**
     * return the value string of the para
     * @param name the name of the para
     * @return the value string
     */
    public String getNamedParameter(String name) {
        return namedParameters.get(name);
    }

    /**
     * add value of PositionalParameter
     */
    public void addPositionalParameter(int position, String value) {
        positionalParameters.add(position - 1, value);
    }

    /**
     * return the value string of the para
     * @param pos the position of the para
     * @return the value string
     */
    public String getPositionalParameter(int pos) {
        return positionalParameters.get(pos - 1);
    }

    public Dialect getDialect() {
        return dialect;
    }

    public Map<String, String> getTableIdDeclarations() {
        return tableIdDeclarations;
    }

    public void addTableIdDeclaration(String id, String tableName) {
        tableIdDeclarations.put(id, tableName);
    }

}
