/* -------------------------------------------------------------------------*
 * PKUAS: Peking University Application Server
 * Copyright (C) 2006 Peking University
 *
 * This software is free; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * either version 2.1 or any later version.
 * 
 * This software is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this software; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 * ------------------------------------------------------------------------*/

package org.ow2.xplus.query.ejbql;

import java.lang.reflect.Constructor;
import java.lang.reflect.InvocationTargetException;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import org.ow2.xplus.query.ejbql.tree.*;
import org.ow2.xplus.util.ClassStrategy;

/**
 * @author Lin Liang
 */

public class NodeFactory {

    public static String SELECTQUERY = "selectQuery";

    public static String DELETEQUERY = "deleteQuery";

    public static String UPDATEQUERY = "updateQuery";

    public static String FROM = "fromClause";

    public static String IDVARDECL = "identificationVariableDeclaration";

    public static String RANGEVARDECL = "rangeVariableDeclaration";

    public static String JOIN = "join";

    public static String FETCHJOIN = "fetchjoin";

    public static String PATHEXPR = "pathExpression";

    public static String IDVAR = "identificationVariable";

    public static String COLLECTIONMEMBERDECL = "collectionMemberDeclaration";

    public static String UPDATE = "updateClause";

    public static String SET = "setClause";

    public static String UPDATEITEM = "updateItem";

    public static String DELETE = "deleteClause";

    public static String SELECT = "selectClause";

    public static String OBJECTEXPR = "objectExpression";

    public static String CONSTRUCTOREXPR = "constructorExpression";

    public static String AGGREGATE = "aggregateExpression";

    public static String WHERE = "whereClause";

    public static String GROUPBY = "groupByClause";

    public static String HAVING = "havingClause";

    public static String ORDERBY = "orderByClause";

    public static String ORDERBYITEM = "orderByItem";

    public static String SUBQUERY = "subquery";

    public static String SUBIDVARDECL = "subselectIdentificationVariableDeclaration";

    public static String CONDITIONALEXPR = "conditionalExpression";

    public static String CONDITIONALTERM = "conditionalTerm";

    public static String CONDITIONALFACTOR = "conditionalFactor";

    public static String BETWEENEXPR = "betweenExpression";

    public static String LITERAL = "Literal";

    public static String LIKEEXPR = "likeExpression";

    public static String ESCAPE = "escape";

    public static String ISNULL = "nullComparisonExpression";

    public static String ISEMPTY = "emptyCollectionComparisonExpression";

    public static String COLLECTIONMEMBEREXPR = "collectionMemberExpression";

    public static String EXISTSEXPR = "existsExpression";

    public static String ANYALLEXPR = "anyOrAllExpression";

    public static String COMPARISONEXPR = "comparisonExpression";

    public static String ARITHMETICEXPR = "arithmeticExpression";

    public static String ARITHMETICTERM = "arithmeticTerm";

    public static String ARITHMETICFACTOR = "arithmeticFactor";

    public static String DATETIME = "functionsReturningDatetime";

    public static String POSITIONALPARAMETER = "inputPositionalParameter";

    public static String NAMEDPARAMETER = "inputNamedParameter";

    public static String ABS = "abs";

    public static String LENGTH = "length";

    public static String MOD = "mod";

    public static String SQRT = "sqrt";

    public static String LOCATE = "locate";

    public static String SIZE = "size";

    public static String CONCAT = "concat";

    public static String SUBSTRING = "substring";

    public static String TRIM = "trim";

    public static String UPPER = "upper";

    public static String LOWER = "lower";

    private static Map<String, Class> classType = new HashMap<String, Class>();

    private static NodeFactory factory = new NodeFactory();

    private NodeFactory() {
        initialize();
    }

    public static NodeFactory getInstance() {
        return factory;
    }

    private void initialize() {
        classType.put(SELECTQUERY, SelectQueryNode.class);
        classType.put(DELETEQUERY, DeleteQueryNode.class);
        classType.put(UPDATEQUERY, UpdateQueryNode.class);
        classType.put(FROM, FromNode.class);
        classType.put(IDVARDECL, IdVarDeclarationNode.class);
        classType.put(RANGEVARDECL, RangeVarDeclarationNode.class);
        classType.put(JOIN, JoinNode.class);
        classType.put(FETCHJOIN, FetchJoinNode.class);
        classType.put(PATHEXPR, PathExprNode.class);
        classType.put(IDVAR, IdVarNode.class);
        classType.put(COLLECTIONMEMBERDECL, CollectionMemberDeclarationNode.class);
        classType.put(UPDATE, UpdateNode.class);
        classType.put(SET, SetNode.class);
        classType.put(UPDATEITEM, UpdateItemNode.class);
        classType.put(DELETE, DeleteNode.class);
        classType.put(SELECT, SelectNode.class);
        classType.put(OBJECTEXPR, ObjectExprNode.class);
        classType.put(CONSTRUCTOREXPR, ConstructorExprNode.class);
        classType.put(AGGREGATE, AggregateNode.class);
        classType.put(WHERE, WhereNode.class);
        classType.put(GROUPBY, GroupByNode.class);
        classType.put(HAVING, HavingNode.class);
        classType.put(ORDERBY, OrderByNode.class);
        classType.put(ORDERBYITEM, OrderByItemNode.class);
        classType.put(SUBQUERY, SubqueryNode.class);
        classType.put(SUBIDVARDECL, SubIdVarDeclarationNode.class);
        classType.put(CONDITIONALEXPR, ConditionalExprNode.class);
        classType.put(CONDITIONALTERM, ConditionalTermNode.class);
        classType.put(CONDITIONALFACTOR, ConditionalFactorNode.class);
        classType.put(BETWEENEXPR, BetweenExprNode.class);
        classType.put(LITERAL, LiteralNode.class);
        classType.put(LIKEEXPR, LikeExprNode.class);
        classType.put(ESCAPE, EscapeNode.class);
        classType.put(ISNULL, IsNullNode.class);
        classType.put(ISEMPTY, IsEmptyNode.class);
        classType.put(COLLECTIONMEMBEREXPR, CollectionMemberExpr.class);
        classType.put(EXISTSEXPR, ExistExprNode.class);
        classType.put(ANYALLEXPR, AnyAllExprNode.class);
        classType.put(COMPARISONEXPR, ComparisonExprNode.class);
        classType.put(ARITHMETICEXPR, ArithmeticExprNode.class);
        classType.put(ARITHMETICTERM, ArithmeticTermNode.class);
        classType.put(ARITHMETICFACTOR, ArithmeticFactorNode.class);
        classType.put(DATETIME, DatetimeNode.class);
        classType.put(POSITIONALPARAMETER, PositionalParameterNode.class);
        classType.put(NAMEDPARAMETER, NamedParameterNode.class);
        classType.put(ABS, AbsNode.class);
        classType.put(MOD, ModNode.class);
        classType.put(LENGTH, LengthNode.class);
        classType.put(SQRT, SqrtNode.class);
        classType.put(LOCATE, LocateNode.class);
        classType.put(SIZE, SizeNode.class);
        classType.put(CONCAT, ConcatNode.class);
        classType.put(SUBSTRING, SubstringNode.class);
        classType.put(TRIM, TrimNode.class);
        classType.put(UPPER, UpperNode.class);
        classType.put(LOWER, LowerNode.class);
    }

    public Node create(String nodeType, List args) {
        Class entity = classType.get(nodeType);
        Class[] types = { List.class };
        Node node = null;
        try {
            Constructor constructor = ClassStrategy.getConstructor(entity, types);
            node = (Node) constructor.newInstance(args);
        } catch (IllegalArgumentException e) {
            e.printStackTrace();
        } catch (InstantiationException e) {
            e.printStackTrace();
        } catch (IllegalAccessException e) {
            e.printStackTrace();
        } catch (InvocationTargetException e) {
            e.printStackTrace();
        }
        return node;
    }
}
