// $ANTLR : "ejbql.g" -> "EJBQLParser.java"$

package org.ow2.xplus.query.ejbql.antlr;

import java.util.ArrayList;
import java.util.List;

import org.ow2.xplus.query.ejbql.tree.Node;
import org.ow2.xplus.query.ejbql.tree.QueryNode;
import org.ow2.xplus.query.ejbql.NodeFactory;

import antlr.TokenBuffer;
import antlr.TokenStreamException;
import antlr.TokenStreamIOException;
import antlr.ANTLRException;
import antlr.LLkParser;
import antlr.Token;
import antlr.TokenStream;
import antlr.RecognitionException;
import antlr.NoViableAltException;
import antlr.MismatchedTokenException;
import antlr.SemanticException;
import antlr.ParserSharedInputState;
import antlr.collections.impl.BitSet;

/**
 * EJB Query Language Grammar
 * This grammar parses the query language for EJB 3.0. 
 */
public class EJBQLParser extends antlr.LLkParser       implements EJBQLTokenTypes
 {

    /** Flag indicating whether aggregates are allowed. */
    private boolean aggregatesAllowed = false;
    
    private NodeFactory factory = NodeFactory.getInstance();
    
    private QueryNode root;
    
    public QueryNode getRoot() { return root; }


protected EJBQLParser(TokenBuffer tokenBuf, int k) {
  super(tokenBuf,k);
  tokenNames = _tokenNames;
}

public EJBQLParser(TokenBuffer tokenBuf) {
  this(tokenBuf,3);
}

protected EJBQLParser(TokenStream lexer, int k) {
  super(lexer,k);
  tokenNames = _tokenNames;
}

public EJBQLParser(TokenStream lexer) {
  this(lexer,3);
}

public EJBQLParser(ParserSharedInputState state) {
  super(state,3);
  tokenNames = _tokenNames;
}

	public final void statement() throws RecognitionException, TokenStreamException {
		
		
		try {      // for error handling
			switch ( LA(1)) {
			case SELECT:
			{
				root=selectStatement();
				break;
			}
			case UPDATE:
			{
				root=updateStatement();
				break;
			}
			case DELETE:
			{
				root=deleteStatement();
				break;
			}
			default:
			{
				throw new NoViableAltException(LT(1), getFilename());
			}
			}
		}
		catch (RecognitionException ex) {
			if (inputState.guessing==0) {
				reportError(ex);
				recover(ex,_tokenSet_0);
			} else {
			  throw ex;
			}
		}
	}
	
	public final QueryNode  selectStatement() throws RecognitionException, TokenStreamException {
		QueryNode node;
		
		
		node = null;
		Node select, from;
		Node where = null;
		Node groupBy = null;
		Node having = null;
		Node orderBy = null;
		List nodes = new ArrayList();
		
		
		try {      // for error handling
			select=selectClause();
			if ( inputState.guessing==0 ) {
				nodes.add(select);
			}
			from=fromClause();
			if ( inputState.guessing==0 ) {
				nodes.add(from);
			}
			{
			switch ( LA(1)) {
			case WHERE:
			{
				where=whereClause();
				if ( inputState.guessing==0 ) {
					nodes.add(where);
				}
				break;
			}
			case EOF:
			case GROUP:
			case HAVING:
			case ORDER:
			{
				break;
			}
			default:
			{
				throw new NoViableAltException(LT(1), getFilename());
			}
			}
			}
			{
			switch ( LA(1)) {
			case GROUP:
			{
				groupBy=groupByClause();
				if ( inputState.guessing==0 ) {
					nodes.add(groupBy);
				}
				break;
			}
			case EOF:
			case HAVING:
			case ORDER:
			{
				break;
			}
			default:
			{
				throw new NoViableAltException(LT(1), getFilename());
			}
			}
			}
			{
			switch ( LA(1)) {
			case HAVING:
			{
				having=havingClause();
				if ( inputState.guessing==0 ) {
					nodes.add(having);
				}
				break;
			}
			case EOF:
			case ORDER:
			{
				break;
			}
			default:
			{
				throw new NoViableAltException(LT(1), getFilename());
			}
			}
			}
			{
			switch ( LA(1)) {
			case ORDER:
			{
				orderBy=orderByClause();
				if ( inputState.guessing==0 ) {
					nodes.add(orderBy);
				}
				break;
			}
			case EOF:
			{
				break;
			}
			default:
			{
				throw new NoViableAltException(LT(1), getFilename());
			}
			}
			}
			match(Token.EOF_TYPE);
			if ( inputState.guessing==0 ) {
				
					node = (QueryNode)factory.create(NodeFactory.SELECTQUERY, nodes );
				
			}
		}
		catch (RecognitionException ex) {
			if (inputState.guessing==0) {
				reportError(ex);
				recover(ex,_tokenSet_0);
			} else {
			  throw ex;
			}
		}
		return node;
	}
	
	public final QueryNode  updateStatement() throws RecognitionException, TokenStreamException {
		QueryNode node;
		
		
		node = null;
		Node update, set, where = null;
		List nodes = new ArrayList();
		
		
		try {      // for error handling
			update=updateClause();
			if ( inputState.guessing==0 ) {
				nodes.add(update);
			}
			set=setClause();
			if ( inputState.guessing==0 ) {
				nodes.add(set);
			}
			{
			switch ( LA(1)) {
			case WHERE:
			{
				where=whereClause();
				if ( inputState.guessing==0 ) {
					nodes.add(where);
				}
				break;
			}
			case EOF:
			{
				break;
			}
			default:
			{
				throw new NoViableAltException(LT(1), getFilename());
			}
			}
			}
			match(Token.EOF_TYPE);
			if ( inputState.guessing==0 ) {
					
				node = (QueryNode)factory.create(NodeFactory.UPDATEQUERY, nodes );
					
			}
		}
		catch (RecognitionException ex) {
			if (inputState.guessing==0) {
				reportError(ex);
				recover(ex,_tokenSet_0);
			} else {
			  throw ex;
			}
		}
		return node;
	}
	
	public final QueryNode  deleteStatement() throws RecognitionException, TokenStreamException {
		QueryNode node;
		
		
		node = null; 
		Node delete, where = null;
		List nodes = new ArrayList();
		
		
		try {      // for error handling
			delete=deleteClause();
			if ( inputState.guessing==0 ) {
				nodes.add(delete);
			}
			{
			switch ( LA(1)) {
			case WHERE:
			{
				where=whereClause();
				if ( inputState.guessing==0 ) {
					nodes.add(where);
				}
				break;
			}
			case EOF:
			{
				break;
			}
			default:
			{
				throw new NoViableAltException(LT(1), getFilename());
			}
			}
			}
			match(Token.EOF_TYPE);
			if ( inputState.guessing==0 ) {
					
				node = (QueryNode)factory.create(NodeFactory.DELETEQUERY, nodes );
					
			}
		}
		catch (RecognitionException ex) {
			if (inputState.guessing==0) {
				reportError(ex);
				recover(ex,_tokenSet_0);
			} else {
			  throw ex;
			}
		}
		return node;
	}
	
	public final Node  selectClause() throws RecognitionException, TokenStreamException {
		Node node;
		
		Token  t = null;
		
		node = null;
		boolean distinct = false;
		Node selectExpr;
		List nodes = new ArrayList();
		
		
		try {      // for error handling
			t = LT(1);
			match(SELECT);
			{
			switch ( LA(1)) {
			case DISTINCT:
			{
				match(DISTINCT);
				if ( inputState.guessing==0 ) {
					distinct = true;
				}
				break;
			}
			case AVG:
			case COUNT:
			case MAX:
			case MIN:
			case NEW:
			case OBJECT:
			case SUM:
			case IDENT:
			{
				break;
			}
			default:
			{
				throw new NoViableAltException(LT(1), getFilename());
			}
			}
			}
			if ( inputState.guessing==0 ) {
				nodes.add(distinct);
			}
			selectExpr=selectExpression();
			if ( inputState.guessing==0 ) {
				nodes.add(selectExpr);
			}
			{
			_loop47:
			do {
				if ((LA(1)==COMMA)) {
					match(COMMA);
					selectExpr=selectExpression();
					if ( inputState.guessing==0 ) {
						nodes.add(selectExpr);
					}
				}
				else {
					break _loop47;
				}
				
			} while (true);
			}
			if ( inputState.guessing==0 ) {
				
				node = factory.create(NodeFactory.SELECT, nodes );
				
			}
		}
		catch (RecognitionException ex) {
			if (inputState.guessing==0) {
				reportError(ex);
				recover(ex,_tokenSet_1);
			} else {
			  throw ex;
			}
		}
		return node;
	}
	
	public final Node  fromClause() throws RecognitionException, TokenStreamException {
		Node node;
		
		Token  t = null;
		
		node = null;
		Node idVar, collMem = null;
		List nodes = new ArrayList(); 
		
		
		try {      // for error handling
			t = LT(1);
			match(FROM);
			idVar=identificationVariableDeclaration();
			if ( inputState.guessing==0 ) {
				nodes.add(idVar);
			}
			{
			_loop14:
			do {
				if ((LA(1)==COMMA)) {
					match(COMMA);
					{
					if (((LA(1) >= ABS && LA(1) <= ESC)) && (LA(2)==AS||LA(2)==IDENT)) {
						idVar=identificationVariableDeclaration();
						if ( inputState.guessing==0 ) {
							nodes.add(idVar);
						}
					}
					else if ((LA(1)==IN) && (LA(2)==LEFT_ROUND_BRACKET)) {
						collMem=collectionMemberDeclaration();
						if ( inputState.guessing==0 ) {
							nodes.add(collMem);
						}
					}
					else {
						throw new NoViableAltException(LT(1), getFilename());
					}
					
					}
				}
				else {
					break _loop14;
				}
				
			} while (true);
			}
			if ( inputState.guessing==0 ) {
				node = factory.create(NodeFactory.FROM, nodes );
			}
		}
		catch (RecognitionException ex) {
			if (inputState.guessing==0) {
				reportError(ex);
				recover(ex,_tokenSet_2);
			} else {
			  throw ex;
			}
		}
		return node;
	}
	
	public final Node  whereClause() throws RecognitionException, TokenStreamException {
		Node node;
		
		Token  t = null;
		
			node = null; 
			Node condition;
			List nodes = new ArrayList();
		
		
		try {      // for error handling
			t = LT(1);
			match(WHERE);
			condition=conditionalExpression();
			if ( inputState.guessing==0 ) {
				nodes.add(condition);
			}
			if ( inputState.guessing==0 ) {
				node = factory.create(NodeFactory.WHERE, nodes );
			}
		}
		catch (RecognitionException ex) {
			if (inputState.guessing==0) {
				reportError(ex);
				recover(ex,_tokenSet_3);
			} else {
			  throw ex;
			}
		}
		return node;
	}
	
	public final Node  groupByClause() throws RecognitionException, TokenStreamException {
		Node node;
		
		Token  g = null;
		
		node = null; 
		Node item;
		List nodes = new ArrayList();
		
		
		try {      // for error handling
			g = LT(1);
			match(GROUP);
			match(BY);
			item=groupByItem();
			if ( inputState.guessing==0 ) {
				nodes.add(item);
			}
			{
			_loop68:
			do {
				if ((LA(1)==COMMA)) {
					match(COMMA);
					item=groupByItem();
					if ( inputState.guessing==0 ) {
						nodes.add(item);
					}
				}
				else {
					break _loop68;
				}
				
			} while (true);
			}
			if ( inputState.guessing==0 ) {
				node = factory.create(NodeFactory.GROUPBY, nodes );
			}
		}
		catch (RecognitionException ex) {
			if (inputState.guessing==0) {
				reportError(ex);
				recover(ex,_tokenSet_4);
			} else {
			  throw ex;
			}
		}
		return node;
	}
	
	public final Node  havingClause() throws RecognitionException, TokenStreamException {
		Node node;
		
		Token  h = null;
		
			node = null; 
			Node condition;
			List nodes = new ArrayList();
		
		
		try {      // for error handling
			h = LT(1);
			match(HAVING);
			if ( inputState.guessing==0 ) {
				aggregatesAllowed = true;
			}
			condition=conditionalExpression();
			if ( inputState.guessing==0 ) {
				nodes.add(condition);
			}
			if ( inputState.guessing==0 ) {
				
				aggregatesAllowed = false; 
				node = factory.create(NodeFactory.HAVING, nodes );
				
			}
		}
		catch (RecognitionException ex) {
			if (inputState.guessing==0) {
				reportError(ex);
				recover(ex,_tokenSet_5);
			} else {
			  throw ex;
			}
		}
		return node;
	}
	
	public final Node  orderByClause() throws RecognitionException, TokenStreamException {
		Node node;
		
		Token  o = null;
		
		node = null; 
		Node item;
		List nodes = new ArrayList();
		
		
		try {      // for error handling
			o = LT(1);
			match(ORDER);
			match(BY);
			item=orderByItem();
			if ( inputState.guessing==0 ) {
				nodes.add(item);
			}
			{
			_loop73:
			do {
				if ((LA(1)==COMMA)) {
					match(COMMA);
					item=orderByItem();
					if ( inputState.guessing==0 ) {
						nodes.add(item);
					}
				}
				else {
					break _loop73;
				}
				
			} while (true);
			}
			if ( inputState.guessing==0 ) {
				node = factory.create(NodeFactory.ORDERBY, nodes );
			}
		}
		catch (RecognitionException ex) {
			if (inputState.guessing==0) {
				reportError(ex);
				recover(ex,_tokenSet_0);
			} else {
			  throw ex;
			}
		}
		return node;
	}
	
	public final Node  updateClause() throws RecognitionException, TokenStreamException {
		Node node;
		
		Token  u = null;
		Token  i = null;
		
		node = null; 
		String schema = null;
		List nodes = new ArrayList(); 
		
		
		try {      // for error handling
			u = LT(1);
			match(UPDATE);
			schema=abstractSchemaName();
			if ( inputState.guessing==0 ) {
				nodes.add(schema);
			}
			{
			switch ( LA(1)) {
			case AS:
			case IDENT:
			{
				{
				switch ( LA(1)) {
				case AS:
				{
					match(AS);
					break;
				}
				case IDENT:
				{
					break;
				}
				default:
				{
					throw new NoViableAltException(LT(1), getFilename());
				}
				}
				}
				i = LT(1);
				match(IDENT);
				if ( inputState.guessing==0 ) {
					nodes.add(i.getText());
				}
				break;
			}
			case SET:
			{
				break;
			}
			default:
			{
				throw new NoViableAltException(LT(1), getFilename());
			}
			}
			}
			if ( inputState.guessing==0 ) {
				
				node = factory.create(NodeFactory.UPDATE, nodes ); 
				
			}
		}
		catch (RecognitionException ex) {
			if (inputState.guessing==0) {
				reportError(ex);
				recover(ex,_tokenSet_6);
			} else {
			  throw ex;
			}
		}
		return node;
	}
	
	public final Node  setClause() throws RecognitionException, TokenStreamException {
		Node node;
		
		Token  t = null;
		
		node = null; 
		Node item;
		List nodes = new ArrayList();
		
		
		try {      // for error handling
			t = LT(1);
			match(SET);
			item=updateItem();
			if ( inputState.guessing==0 ) {
				nodes.add(item);
			}
			{
			_loop38:
			do {
				if ((LA(1)==COMMA)) {
					match(COMMA);
					item=updateItem();
					if ( inputState.guessing==0 ) {
						nodes.add(item);
					}
				}
				else {
					break _loop38;
				}
				
			} while (true);
			}
			if ( inputState.guessing==0 ) {
				
					node = factory.create(NodeFactory.SET, nodes );  
				
			}
		}
		catch (RecognitionException ex) {
			if (inputState.guessing==0) {
				reportError(ex);
				recover(ex,_tokenSet_7);
			} else {
			  throw ex;
			}
		}
		return node;
	}
	
	public final Node  deleteClause() throws RecognitionException, TokenStreamException {
		Node node;
		
		Token  t = null;
		Token  i = null;
		
		node = null; 
		String schema = null;
		List nodes = new ArrayList();
		
		
		try {      // for error handling
			t = LT(1);
			match(DELETE);
			match(FROM);
			schema=abstractSchemaName();
			if ( inputState.guessing==0 ) {
				nodes.add(schema);
			}
			{
			switch ( LA(1)) {
			case AS:
			case IDENT:
			{
				{
				switch ( LA(1)) {
				case AS:
				{
					match(AS);
					break;
				}
				case IDENT:
				{
					break;
				}
				default:
				{
					throw new NoViableAltException(LT(1), getFilename());
				}
				}
				}
				i = LT(1);
				match(IDENT);
				if ( inputState.guessing==0 ) {
					nodes.add(i.getText());
				}
				break;
			}
			case EOF:
			case WHERE:
			{
				break;
			}
			default:
			{
				throw new NoViableAltException(LT(1), getFilename());
			}
			}
			}
			if ( inputState.guessing==0 ) {
				
				node = factory.create(NodeFactory.DELETE, nodes );
				
			}
		}
		catch (RecognitionException ex) {
			if (inputState.guessing==0) {
				reportError(ex);
				recover(ex,_tokenSet_7);
			} else {
			  throw ex;
			}
		}
		return node;
	}
	
	public final Node  identificationVariableDeclaration() throws RecognitionException, TokenStreamException {
		Node node;
		
		
			node = null; 
			Node rangeVar, joins = null;
			List nodes = new ArrayList(); 
		
		
		try {      // for error handling
			rangeVar=rangeVariableDeclaration();
			if ( inputState.guessing==0 ) {
				nodes.add(rangeVar);
			}
			{
			_loop17:
			do {
				if ((LA(1)==INNER||LA(1)==JOIN||LA(1)==LEFT)) {
					joins=join();
					if ( inputState.guessing==0 ) {
						nodes.add(joins);
					}
				}
				else {
					break _loop17;
				}
				
			} while (true);
			}
			if ( inputState.guessing==0 ) {
				node = factory.create(NodeFactory.IDVARDECL, nodes );
			}
		}
		catch (RecognitionException ex) {
			if (inputState.guessing==0) {
				reportError(ex);
				recover(ex,_tokenSet_8);
			} else {
			  throw ex;
			}
		}
		return node;
	}
	
	public final Node  collectionMemberDeclaration() throws RecognitionException, TokenStreamException {
		Node node;
		
		Token  t = null;
		Token  i = null;
		
			node = null; 
			Node pathExpr = null;
			List nodes = new ArrayList(); 
		
		
		try {      // for error handling
			t = LT(1);
			match(IN);
			match(LEFT_ROUND_BRACKET);
			pathExpr=pathExpression();
			if ( inputState.guessing==0 ) {
				nodes.add(pathExpr);
			}
			match(RIGHT_ROUND_BRACKET);
			{
			switch ( LA(1)) {
			case AS:
			{
				match(AS);
				break;
			}
			case IDENT:
			{
				break;
			}
			default:
			{
				throw new NoViableAltException(LT(1), getFilename());
			}
			}
			}
			i = LT(1);
			match(IDENT);
			if ( inputState.guessing==0 ) {
				nodes.add(i.getText());
			}
			if ( inputState.guessing==0 ) {
				
					node = factory.create(NodeFactory.COLLECTIONMEMBERDECL, nodes ); 
				
			}
		}
		catch (RecognitionException ex) {
			if (inputState.guessing==0) {
				reportError(ex);
				recover(ex,_tokenSet_8);
			} else {
			  throw ex;
			}
		}
		return node;
	}
	
	public final Node  rangeVariableDeclaration() throws RecognitionException, TokenStreamException {
		Node node;
		
		Token  i = null;
		
		node = null; 
		String schema;
		List nodes = new ArrayList(); 
		
		
		try {      // for error handling
			schema=abstractSchemaName();
			if ( inputState.guessing==0 ) {
				nodes.add(schema);
			}
			{
			switch ( LA(1)) {
			case AS:
			{
				match(AS);
				break;
			}
			case IDENT:
			{
				break;
			}
			default:
			{
				throw new NoViableAltException(LT(1), getFilename());
			}
			}
			}
			i = LT(1);
			match(IDENT);
			if ( inputState.guessing==0 ) {
				nodes.add(i.getText());
			}
			if ( inputState.guessing==0 ) {
				
				node = factory.create(NodeFactory.RANGEVARDECL, nodes ); 
				
			}
		}
		catch (RecognitionException ex) {
			if (inputState.guessing==0) {
				reportError(ex);
				recover(ex,_tokenSet_9);
			} else {
			  throw ex;
			}
		}
		return node;
	}
	
	public final Node  join() throws RecognitionException, TokenStreamException {
		Node node;
		
		Token  i = null;
		Token  t = null;
		
		node = null;
		boolean outerJoin; 
		Node pathExpr;
		List nodes = new ArrayList();
		
		
		try {      // for error handling
			outerJoin=joinSpec();
			if ( inputState.guessing==0 ) {
				nodes.add(outerJoin);
			}
			{
			switch ( LA(1)) {
			case IDENT:
			{
				pathExpr=pathExpression();
				if ( inputState.guessing==0 ) {
					nodes.add(pathExpr);
				}
				{
				switch ( LA(1)) {
				case AS:
				{
					match(AS);
					break;
				}
				case IDENT:
				{
					break;
				}
				default:
				{
					throw new NoViableAltException(LT(1), getFilename());
				}
				}
				}
				i = LT(1);
				match(IDENT);
				if ( inputState.guessing==0 ) {
					nodes.add(i.getText());
				}
				if ( inputState.guessing==0 ) {
					
					node = factory.create(NodeFactory.JOIN, nodes ); 
					
				}
				break;
			}
			case FETCH:
			{
				t = LT(1);
				match(FETCH);
				pathExpr=pathExpression();
				if ( inputState.guessing==0 ) {
					nodes.add(pathExpr);
				}
				if ( inputState.guessing==0 ) {
					
					node = factory.create(NodeFactory.FETCHJOIN, nodes );
					
				}
				break;
			}
			default:
			{
				throw new NoViableAltException(LT(1), getFilename());
			}
			}
			}
		}
		catch (RecognitionException ex) {
			if (inputState.guessing==0) {
				reportError(ex);
				recover(ex,_tokenSet_9);
			} else {
			  throw ex;
			}
		}
		return node;
	}
	
	public final String  abstractSchemaName() throws RecognitionException, TokenStreamException {
		String schema;
		
		Token  ident = null;
		schema = null;
		
		try {      // for error handling
			ident = LT(1);
			matchNot(EOF);
			if ( inputState.guessing==0 ) {
				
				schema = ident.getText();
				
			}
		}
		catch (RecognitionException ex) {
			if (inputState.guessing==0) {
				reportError(ex);
				recover(ex,_tokenSet_10);
			} else {
			  throw ex;
			}
		}
		return schema;
	}
	
	public final boolean  joinSpec() throws RecognitionException, TokenStreamException {
		boolean outer;
		
		outer = false;
		
		try {      // for error handling
			{
			switch ( LA(1)) {
			case LEFT:
			{
				match(LEFT);
				{
				switch ( LA(1)) {
				case OUTER:
				{
					match(OUTER);
					break;
				}
				case JOIN:
				{
					break;
				}
				default:
				{
					throw new NoViableAltException(LT(1), getFilename());
				}
				}
				}
				if ( inputState.guessing==0 ) {
					outer = true;
				}
				break;
			}
			case INNER:
			{
				match(INNER);
				break;
			}
			case JOIN:
			{
				break;
			}
			default:
			{
				throw new NoViableAltException(LT(1), getFilename());
			}
			}
			}
			match(JOIN);
		}
		catch (RecognitionException ex) {
			if (inputState.guessing==0) {
				reportError(ex);
				recover(ex,_tokenSet_11);
			} else {
			  throw ex;
			}
		}
		return outer;
	}
	
	public final Node  pathExpression() throws RecognitionException, TokenStreamException {
		Node node;
		
		Token  idVar = null;
		Token  d = null;
		Token  field = null;
		
		node = null; 
		List nodes = new ArrayList(); 
		
		
		try {      // for error handling
			idVar = LT(1);
			match(IDENT);
			if ( inputState.guessing==0 ) {
				nodes.add(idVar.getText());
			}
			{
			int _cnt29=0;
			_loop29:
			do {
				if ((LA(1)==DOT)) {
					d = LT(1);
					match(DOT);
					field = LT(1);
					match(IDENT);
					if ( inputState.guessing==0 ) {
						nodes.add(field.getText());
					}
				}
				else {
					if ( _cnt29>=1 ) { break _loop29; } else {throw new NoViableAltException(LT(1), getFilename());}
				}
				
				_cnt29++;
			} while (true);
			}
			if ( inputState.guessing==0 ) {
				node = factory.create(NodeFactory.PATHEXPR, nodes );
			}
		}
		catch (RecognitionException ex) {
			if (inputState.guessing==0) {
				reportError(ex);
				recover(ex,_tokenSet_12);
			} else {
			  throw ex;
			}
		}
		return node;
	}
	
	public final Node  identificationVariable() throws RecognitionException, TokenStreamException {
		Node node;
		
		Token  i = null;
		
			node = null; 
			List nodes = new ArrayList();
			
		
		
		try {      // for error handling
			i = LT(1);
			match(IDENT);
			if ( inputState.guessing==0 ) {
				
					nodes.add(i.getText());
					node = factory.create(NodeFactory.IDVAR, nodes ); 
				
			}
		}
		catch (RecognitionException ex) {
			if (inputState.guessing==0) {
				reportError(ex);
				recover(ex,_tokenSet_13);
			} else {
			  throw ex;
			}
		}
		return node;
	}
	
	public final Node  updateItem() throws RecognitionException, TokenStreamException {
		Node node;
		
		Token  t = null;
		
		node = null;
		Node pathExpr, value; 
		List nodes = new ArrayList();
		
		
		try {      // for error handling
			pathExpr=pathExpression();
			if ( inputState.guessing==0 ) {
				nodes.add(pathExpr);
			}
			t = LT(1);
			match(EQUALS);
			value=newValue();
			if ( inputState.guessing==0 ) {
				nodes.add(value);
			}
			if ( inputState.guessing==0 ) {
				
				node = factory.create(NodeFactory.UPDATEITEM, nodes );
				
			}
		}
		catch (RecognitionException ex) {
			if (inputState.guessing==0) {
				reportError(ex);
				recover(ex,_tokenSet_14);
			} else {
			  throw ex;
			}
		}
		return node;
	}
	
	public final Node  newValue() throws RecognitionException, TokenStreamException {
		Node node;
		
		node = null;
		
		try {      // for error handling
			node=simpleArithmeticExpression();
		}
		catch (RecognitionException ex) {
			if (inputState.guessing==0) {
				reportError(ex);
				recover(ex,_tokenSet_14);
			} else {
			  throw ex;
			}
		}
		return node;
	}
	
	public final Node  simpleArithmeticExpression() throws RecognitionException, TokenStreamException {
		Node node;
		
		Token  p = null;
		Token  m = null;
		
		node = null; 
		Node left, right;
		List nodes = new ArrayList();
		
		
		try {      // for error handling
			left=arithmeticTerm();
			if ( inputState.guessing==0 ) {
				nodes.add(left);
			}
			{
			_loop144:
			do {
				switch ( LA(1)) {
				case PLUS:
				{
					p = LT(1);
					match(PLUS);
					right=arithmeticTerm();
					if ( inputState.guessing==0 ) {
						nodes.add(p.getText()); nodes.add(right);
					}
					break;
				}
				case MINUS:
				{
					m = LT(1);
					match(MINUS);
					right=arithmeticTerm();
					if ( inputState.guessing==0 ) {
						nodes.add(p.getText()); nodes.add(right);
					}
					break;
				}
				default:
				{
					break _loop144;
				}
				}
			} while (true);
			}
			if ( inputState.guessing==0 ) {
				
					node = factory.create(NodeFactory.ARITHMETICEXPR, nodes );
				
			}
		}
		catch (RecognitionException ex) {
			if (inputState.guessing==0) {
				reportError(ex);
				recover(ex,_tokenSet_15);
			} else {
			  throw ex;
			}
		}
		return node;
	}
	
	public final Node  selectExpression() throws RecognitionException, TokenStreamException {
		Node node;
		
		
			node = null; 
		
		
		try {      // for error handling
			switch ( LA(1)) {
			case AVG:
			case COUNT:
			case MAX:
			case MIN:
			case SUM:
			{
				node=aggregateExpression();
				break;
			}
			case OBJECT:
			{
				node=objectExpression();
				break;
			}
			case NEW:
			{
				node=constructorExpression();
				break;
			}
			default:
				if ((LA(1)==IDENT) && (LA(2)==DOT)) {
					node=pathExpression();
				}
				else if ((LA(1)==IDENT) && (LA(2)==FROM||LA(2)==COMMA)) {
					node=identificationVariable();
				}
			else {
				throw new NoViableAltException(LT(1), getFilename());
			}
			}
		}
		catch (RecognitionException ex) {
			if (inputState.guessing==0) {
				reportError(ex);
				recover(ex,_tokenSet_16);
			} else {
			  throw ex;
			}
		}
		return node;
	}
	
	public final Node  aggregateExpression() throws RecognitionException, TokenStreamException {
		Node node;
		
		Token  t1 = null;
		Token  t2 = null;
		Token  t3 = null;
		Token  t4 = null;
		Token  t5 = null;
		
		node = null; 
		String operator = null;
		boolean distinct = false;
		Node expr;
		List nodes = new ArrayList();
		
		
		try {      // for error handling
			{
			switch ( LA(1)) {
			case AVG:
			{
				t1 = LT(1);
				match(AVG);
				if ( inputState.guessing==0 ) {
					operator = t1.getText();
				}
				match(LEFT_ROUND_BRACKET);
				{
				switch ( LA(1)) {
				case DISTINCT:
				{
					match(DISTINCT);
					if ( inputState.guessing==0 ) {
						distinct = true;
					}
					break;
				}
				case IDENT:
				{
					break;
				}
				default:
				{
					throw new NoViableAltException(LT(1), getFilename());
				}
				}
				}
				expr=pathExpression();
				match(RIGHT_ROUND_BRACKET);
				break;
			}
			case MAX:
			{
				t2 = LT(1);
				match(MAX);
				if ( inputState.guessing==0 ) {
					operator = t2.getText();
				}
				match(LEFT_ROUND_BRACKET);
				{
				switch ( LA(1)) {
				case DISTINCT:
				{
					match(DISTINCT);
					if ( inputState.guessing==0 ) {
						distinct = true;
					}
					break;
				}
				case IDENT:
				{
					break;
				}
				default:
				{
					throw new NoViableAltException(LT(1), getFilename());
				}
				}
				}
				expr=pathExpression();
				match(RIGHT_ROUND_BRACKET);
				break;
			}
			case MIN:
			{
				t3 = LT(1);
				match(MIN);
				if ( inputState.guessing==0 ) {
					operator = t3.getText();
				}
				match(LEFT_ROUND_BRACKET);
				{
				switch ( LA(1)) {
				case DISTINCT:
				{
					match(DISTINCT);
					if ( inputState.guessing==0 ) {
						distinct = true;
					}
					break;
				}
				case IDENT:
				{
					break;
				}
				default:
				{
					throw new NoViableAltException(LT(1), getFilename());
				}
				}
				}
				expr=pathExpression();
				match(RIGHT_ROUND_BRACKET);
				break;
			}
			case SUM:
			{
				t4 = LT(1);
				match(SUM);
				if ( inputState.guessing==0 ) {
					operator = t4.getText();
				}
				match(LEFT_ROUND_BRACKET);
				{
				switch ( LA(1)) {
				case DISTINCT:
				{
					match(DISTINCT);
					if ( inputState.guessing==0 ) {
						distinct = true;
					}
					break;
				}
				case IDENT:
				{
					break;
				}
				default:
				{
					throw new NoViableAltException(LT(1), getFilename());
				}
				}
				}
				expr=pathExpression();
				match(RIGHT_ROUND_BRACKET);
				break;
			}
			case COUNT:
			{
				t5 = LT(1);
				match(COUNT);
				if ( inputState.guessing==0 ) {
					operator = t5.getText();
				}
				match(LEFT_ROUND_BRACKET);
				{
				switch ( LA(1)) {
				case DISTINCT:
				{
					match(DISTINCT);
					if ( inputState.guessing==0 ) {
						distinct = true;
					}
					break;
				}
				case IDENT:
				{
					break;
				}
				default:
				{
					throw new NoViableAltException(LT(1), getFilename());
				}
				}
				}
				{
				if ((LA(1)==IDENT) && (LA(2)==RIGHT_ROUND_BRACKET)) {
					expr=identificationVariable();
				}
				else if ((LA(1)==IDENT) && (LA(2)==DOT)) {
					expr=pathExpression();
				}
				else {
					throw new NoViableAltException(LT(1), getFilename());
				}
				
				}
				match(RIGHT_ROUND_BRACKET);
				break;
			}
			default:
			{
				throw new NoViableAltException(LT(1), getFilename());
			}
			}
			}
			if ( inputState.guessing==0 ) {
				
					nodes.add(operator);
					nodes.add(distinct);
					nodes.add(expr);
					node = factory.create(NodeFactory.AGGREGATE, nodes ); 
				
			}
		}
		catch (RecognitionException ex) {
			if (inputState.guessing==0) {
				reportError(ex);
				recover(ex,_tokenSet_17);
			} else {
			  throw ex;
			}
		}
		return node;
	}
	
	public final Node  objectExpression() throws RecognitionException, TokenStreamException {
		Node node;
		
		Token  t = null;
		
		node = null;
		Node idVar;
		List nodes = new ArrayList();
		
		
		try {      // for error handling
			t = LT(1);
			match(OBJECT);
			match(LEFT_ROUND_BRACKET);
			idVar=identificationVariable();
			if ( inputState.guessing==0 ) {
				nodes.add(idVar);
			}
			match(RIGHT_ROUND_BRACKET);
			if ( inputState.guessing==0 ) {
				
				node = factory.create(NodeFactory.OBJECTEXPR, nodes );; 
				
			}
		}
		catch (RecognitionException ex) {
			if (inputState.guessing==0) {
				reportError(ex);
				recover(ex,_tokenSet_16);
			} else {
			  throw ex;
			}
		}
		return node;
	}
	
	public final Node  constructorExpression() throws RecognitionException, TokenStreamException {
		Node node;
		
		Token  t = null;
		
		node = null;
		String className = null; 
		Node item;
		List nodes = new ArrayList();
		
		
		try {      // for error handling
			t = LT(1);
			match(NEW);
			className=constructorName();
			if ( inputState.guessing==0 ) {
				nodes.add(className);
			}
			match(LEFT_ROUND_BRACKET);
			item=constructorItem();
			if ( inputState.guessing==0 ) {
				nodes.add(item);
			}
			{
			_loop52:
			do {
				if ((LA(1)==COMMA)) {
					match(COMMA);
					item=constructorItem();
					if ( inputState.guessing==0 ) {
						nodes.add(item);
					}
				}
				else {
					break _loop52;
				}
				
			} while (true);
			}
			match(RIGHT_ROUND_BRACKET);
			if ( inputState.guessing==0 ) {
				
				node = factory.create(NodeFactory.SELECT, nodes );; 
				
			}
		}
		catch (RecognitionException ex) {
			if (inputState.guessing==0) {
				reportError(ex);
				recover(ex,_tokenSet_16);
			} else {
			  throw ex;
			}
		}
		return node;
	}
	
	public final String  constructorName() throws RecognitionException, TokenStreamException {
		String className;
		
		Token  i1 = null;
		Token  i2 = null;
		
		className = null;
		StringBuffer buf = new StringBuffer(); 
		
		
		try {      // for error handling
			i1 = LT(1);
			match(IDENT);
			if ( inputState.guessing==0 ) {
				buf.append(i1.getText());
			}
			{
			_loop55:
			do {
				if ((LA(1)==DOT)) {
					match(DOT);
					i2 = LT(1);
					match(IDENT);
					if ( inputState.guessing==0 ) {
						buf.append('.').append(i2.getText());
					}
				}
				else {
					break _loop55;
				}
				
			} while (true);
			}
			if ( inputState.guessing==0 ) {
				className = buf.toString();
			}
		}
		catch (RecognitionException ex) {
			if (inputState.guessing==0) {
				reportError(ex);
				recover(ex,_tokenSet_18);
			} else {
			  throw ex;
			}
		}
		return className;
	}
	
	public final Node  constructorItem() throws RecognitionException, TokenStreamException {
		Node node;
		
		node = null;
		
		try {      // for error handling
			switch ( LA(1)) {
			case IDENT:
			{
				node=pathExpression();
				break;
			}
			case AVG:
			case COUNT:
			case MAX:
			case MIN:
			case SUM:
			{
				node=aggregateExpression();
				break;
			}
			default:
			{
				throw new NoViableAltException(LT(1), getFilename());
			}
			}
		}
		catch (RecognitionException ex) {
			if (inputState.guessing==0) {
				reportError(ex);
				recover(ex,_tokenSet_19);
			} else {
			  throw ex;
			}
		}
		return node;
	}
	
	public final Node  conditionalExpression() throws RecognitionException, TokenStreamException {
		Node node;
		
		Token  t = null;
		
		node = null;
		Node term;
		List nodes = new ArrayList();
		
		
		try {      // for error handling
			term=conditionalTerm();
			if ( inputState.guessing==0 ) {
				nodes.add(term);
			}
			{
			_loop90:
			do {
				if ((LA(1)==OR)) {
					t = LT(1);
					match(OR);
					term=conditionalTerm();
					if ( inputState.guessing==0 ) {
						nodes.add(term);
					}
				}
				else {
					break _loop90;
				}
				
			} while (true);
			}
			if ( inputState.guessing==0 ) {
				node = factory.create(NodeFactory.CONDITIONALEXPR, nodes );
			}
		}
		catch (RecognitionException ex) {
			if (inputState.guessing==0) {
				reportError(ex);
				recover(ex,_tokenSet_3);
			} else {
			  throw ex;
			}
		}
		return node;
	}
	
	public final Node  groupByItem() throws RecognitionException, TokenStreamException {
		Node node;
		
		
			node = null; 
		
		
		try {      // for error handling
			if ((LA(1)==IDENT) && (LA(2)==DOT)) {
				node=pathExpression();
			}
			else if ((LA(1)==IDENT) && (_tokenSet_20.member(LA(2)))) {
				node=identificationVariable();
			}
			else {
				throw new NoViableAltException(LT(1), getFilename());
			}
			
		}
		catch (RecognitionException ex) {
			if (inputState.guessing==0) {
				reportError(ex);
				recover(ex,_tokenSet_20);
			} else {
			  throw ex;
			}
		}
		return node;
	}
	
	public final Node  orderByItem() throws RecognitionException, TokenStreamException {
		Node node;
		
		Token  a = null;
		Token  d = null;
		
			node = null;
			Node pathExpr;
			List nodes = new ArrayList(); 
		
		
		try {      // for error handling
			pathExpr=pathExpression();
			if ( inputState.guessing==0 ) {
				nodes.add(pathExpr);
			}
			{
			switch ( LA(1)) {
			case ASC:
			{
				a = LT(1);
				match(ASC);
				if ( inputState.guessing==0 ) {
					nodes.add(a.getText());
				}
				break;
			}
			case DESC:
			{
				d = LT(1);
				match(DESC);
				if ( inputState.guessing==0 ) {
					nodes.add(d.getText());
				}
				break;
			}
			case EOF:
			case COMMA:
			{
				break;
			}
			default:
			{
				throw new NoViableAltException(LT(1), getFilename());
			}
			}
			}
			if ( inputState.guessing==0 ) {
				node = factory.create(NodeFactory.ORDERBYITEM, nodes );
			}
		}
		catch (RecognitionException ex) {
			if (inputState.guessing==0) {
				reportError(ex);
				recover(ex,_tokenSet_21);
			} else {
			  throw ex;
			}
		}
		return node;
	}
	
	public final Node  subquery() throws RecognitionException, TokenStreamException {
		Node node;
		
		
		node = null; 
		Node select, from;
		Node where = null;
		Node groupBy = null;
		Node having = null;
		List nodes = new ArrayList(); 
		
		
		try {      // for error handling
			select=simpleSelectClause();
			if ( inputState.guessing==0 ) {
				nodes.add(select);
			}
			from=subqueryFromClause();
			if ( inputState.guessing==0 ) {
				nodes.add(from);
			}
			{
			switch ( LA(1)) {
			case WHERE:
			{
				where=whereClause();
				if ( inputState.guessing==0 ) {
					nodes.add(where);
				}
				break;
			}
			case GROUP:
			case HAVING:
			case RIGHT_ROUND_BRACKET:
			{
				break;
			}
			default:
			{
				throw new NoViableAltException(LT(1), getFilename());
			}
			}
			}
			{
			switch ( LA(1)) {
			case GROUP:
			{
				groupBy=groupByClause();
				if ( inputState.guessing==0 ) {
					nodes.add(groupBy);
				}
				break;
			}
			case HAVING:
			case RIGHT_ROUND_BRACKET:
			{
				break;
			}
			default:
			{
				throw new NoViableAltException(LT(1), getFilename());
			}
			}
			}
			{
			switch ( LA(1)) {
			case HAVING:
			{
				having=havingClause();
				if ( inputState.guessing==0 ) {
					nodes.add(having);
				}
				break;
			}
			case RIGHT_ROUND_BRACKET:
			{
				break;
			}
			default:
			{
				throw new NoViableAltException(LT(1), getFilename());
			}
			}
			}
			if ( inputState.guessing==0 ) {
				
				node = factory.create(NodeFactory.SUBQUERY, nodes ); 
				
			}
		}
		catch (RecognitionException ex) {
			if (inputState.guessing==0) {
				reportError(ex);
				recover(ex,_tokenSet_22);
			} else {
			  throw ex;
			}
		}
		return node;
	}
	
	public final Node  simpleSelectClause() throws RecognitionException, TokenStreamException {
		Node node;
		
		Token  s = null;
		
		node = null; 
		boolean distinct = false;
		Node simpleSelectExpr;
		List nodes = new ArrayList();
		
		
		try {      // for error handling
			s = LT(1);
			match(SELECT);
			{
			switch ( LA(1)) {
			case DISTINCT:
			{
				match(DISTINCT);
				if ( inputState.guessing==0 ) {
					distinct = true; nodes.add(distinct);
				}
				break;
			}
			case AVG:
			case COUNT:
			case MAX:
			case MIN:
			case SUM:
			case IDENT:
			{
				break;
			}
			default:
			{
				throw new NoViableAltException(LT(1), getFilename());
			}
			}
			}
			simpleSelectExpr=simpleSelectExpression();
			if ( inputState.guessing==0 ) {
				
				nodes.add(simpleSelectExpr);
				node = factory.create(NodeFactory.SELECT, nodes );
				
			}
		}
		catch (RecognitionException ex) {
			if (inputState.guessing==0) {
				reportError(ex);
				recover(ex,_tokenSet_1);
			} else {
			  throw ex;
			}
		}
		return node;
	}
	
	public final Node  subqueryFromClause() throws RecognitionException, TokenStreamException {
		Node node;
		
		Token  f = null;
		
		node = null; 
		Node subIdVar = null;
		List nodes = new ArrayList(); 
		
		
		try {      // for error handling
			f = LT(1);
			match(FROM);
			subIdVar=subselectIdentificationVariableDeclaration();
			if ( inputState.guessing==0 ) {
				nodes.add(subIdVar);
			}
			{
			_loop82:
			do {
				if ((LA(1)==COMMA)) {
					match(COMMA);
					subIdVar=subselectIdentificationVariableDeclaration();
					if ( inputState.guessing==0 ) {
						nodes.add(subIdVar);
					}
				}
				else {
					break _loop82;
				}
				
			} while (true);
			}
			if ( inputState.guessing==0 ) {
				node = factory.create(NodeFactory.FROM, nodes );
			}
		}
		catch (RecognitionException ex) {
			if (inputState.guessing==0) {
				reportError(ex);
				recover(ex,_tokenSet_23);
			} else {
			  throw ex;
			}
		}
		return node;
	}
	
	public final Node  subselectIdentificationVariableDeclaration() throws RecognitionException, TokenStreamException {
		Node node;
		
		Token  ident = null;
		
			node = null;
			Node pathExpr;
			List nodes = new ArrayList(); 
		
		
		try {      // for error handling
			if (((LA(1) >= ABS && LA(1) <= ESC)) && (LA(2)==AS||LA(2)==IDENT)) {
				node=identificationVariableDeclaration();
			}
			else if ((LA(1)==IDENT) && (LA(2)==DOT)) {
				pathExpr=pathExpression();
				{
				switch ( LA(1)) {
				case AS:
				{
					match(AS);
					break;
				}
				case IDENT:
				{
					break;
				}
				default:
				{
					throw new NoViableAltException(LT(1), getFilename());
				}
				}
				}
				ident = LT(1);
				match(IDENT);
				if ( inputState.guessing==0 ) {
					
						nodes.add(pathExpr);
						nodes.add(ident.getText());
						node =  factory.create(NodeFactory.SUBIDVARDECL, nodes );
					
				}
			}
			else if ((LA(1)==IN) && (LA(2)==LEFT_ROUND_BRACKET)) {
				node=collectionMemberDeclaration();
			}
			else {
				throw new NoViableAltException(LT(1), getFilename());
			}
			
		}
		catch (RecognitionException ex) {
			if (inputState.guessing==0) {
				reportError(ex);
				recover(ex,_tokenSet_24);
			} else {
			  throw ex;
			}
		}
		return node;
	}
	
	public final Node  simpleSelectExpression() throws RecognitionException, TokenStreamException {
		Node node;
		
		
			node = null; 
		
		
		try {      // for error handling
			if ((LA(1)==IDENT) && (LA(2)==DOT)) {
				node=pathExpression();
			}
			else if ((_tokenSet_25.member(LA(1)))) {
				node=aggregateExpression();
			}
			else if ((LA(1)==IDENT) && (LA(2)==FROM)) {
				node=identificationVariable();
			}
			else {
				throw new NoViableAltException(LT(1), getFilename());
			}
			
		}
		catch (RecognitionException ex) {
			if (inputState.guessing==0) {
				reportError(ex);
				recover(ex,_tokenSet_1);
			} else {
			  throw ex;
			}
		}
		return node;
	}
	
	public final Node  conditionalTerm() throws RecognitionException, TokenStreamException {
		Node node;
		
		Token  t = null;
		
		node = null; 
		Node factor;
		List nodes = new ArrayList();
		
		
		try {      // for error handling
			factor=conditionalFactor();
			if ( inputState.guessing==0 ) {
				nodes.add(factor);
			}
			{
			_loop93:
			do {
				if ((LA(1)==AND)) {
					t = LT(1);
					match(AND);
					factor=conditionalFactor();
					if ( inputState.guessing==0 ) {
						nodes.add(factor);
					}
				}
				else {
					break _loop93;
				}
				
			} while (true);
			}
			if ( inputState.guessing==0 ) {
				node = factory.create(NodeFactory.CONDITIONALTERM, nodes );
			}
		}
		catch (RecognitionException ex) {
			if (inputState.guessing==0) {
				reportError(ex);
				recover(ex,_tokenSet_26);
			} else {
			  throw ex;
			}
		}
		return node;
	}
	
	public final Node  conditionalFactor() throws RecognitionException, TokenStreamException {
		Node node;
		
		Token  n = null;
		
		node = null;
		boolean not = false;
		Node primary;
		List nodes = new ArrayList();
		
		
		try {      // for error handling
			{
			switch ( LA(1)) {
			case NOT:
			{
				n = LT(1);
				match(NOT);
				if ( inputState.guessing==0 ) {
					not = true;
				}
				break;
			}
			case ABS:
			case AVG:
			case CONCAT:
			case COUNT:
			case EXISTS:
			case LENGTH:
			case LOCATE:
			case LOWER:
			case MAX:
			case MIN:
			case MOD:
			case SIZE:
			case SQRT:
			case SUBSTRING:
			case SUM:
			case TRIM:
			case UPPER:
			case IDENT:
			case LEFT_ROUND_BRACKET:
			case NUM_INT:
			case NUM_FLOAT:
			case STRING_LITERAL_DOUBLE_QUOTED:
			case STRING_LITERAL_SINGLE_QUOTED:
			case PLUS:
			case MINUS:
			case POSITIONAL_PARAM:
			case NAMED_PARAM:
			{
				break;
			}
			default:
			{
				throw new NoViableAltException(LT(1), getFilename());
			}
			}
			}
			if ( inputState.guessing==0 ) {
				nodes.add(not);
			}
			primary=conditionalPrimary();
			if ( inputState.guessing==0 ) {
				nodes.add(primary);
			}
			if ( inputState.guessing==0 ) {
				
				node = factory.create(NodeFactory.CONDITIONALFACTOR, nodes );
				
			}
		}
		catch (RecognitionException ex) {
			if (inputState.guessing==0) {
				reportError(ex);
				recover(ex,_tokenSet_27);
			} else {
			  throw ex;
			}
		}
		return node;
	}
	
	public final Node  conditionalPrimary() throws RecognitionException, TokenStreamException {
		Node node;
		
		
			node = null; 
		
		
		try {      // for error handling
			boolean synPredMatched98 = false;
			if (((LA(1)==LEFT_ROUND_BRACKET) && (_tokenSet_28.member(LA(2))) && (_tokenSet_29.member(LA(3))))) {
				int _m98 = mark();
				synPredMatched98 = true;
				inputState.guessing++;
				try {
					{
					match(LEFT_ROUND_BRACKET);
					conditionalExpression();
					}
				}
				catch (RecognitionException pe) {
					synPredMatched98 = false;
				}
				rewind(_m98);
inputState.guessing--;
			}
			if ( synPredMatched98 ) {
				match(LEFT_ROUND_BRACKET);
				node=conditionalExpression();
				match(RIGHT_ROUND_BRACKET);
			}
			else if ((_tokenSet_30.member(LA(1))) && (_tokenSet_31.member(LA(2))) && (_tokenSet_32.member(LA(3)))) {
				node=simpleConditionalExpression();
			}
			else {
				throw new NoViableAltException(LT(1), getFilename());
			}
			
		}
		catch (RecognitionException ex) {
			if (inputState.guessing==0) {
				reportError(ex);
				recover(ex,_tokenSet_27);
			} else {
			  throw ex;
			}
		}
		return node;
	}
	
	public final Node  simpleConditionalExpression() throws RecognitionException, TokenStreamException {
		Node node;
		
		
		node = null; 
		Node left = null;
		
		
		try {      // for error handling
			switch ( LA(1)) {
			case ABS:
			case AVG:
			case CONCAT:
			case COUNT:
			case LENGTH:
			case LOCATE:
			case LOWER:
			case MAX:
			case MIN:
			case MOD:
			case SIZE:
			case SQRT:
			case SUBSTRING:
			case SUM:
			case TRIM:
			case UPPER:
			case IDENT:
			case LEFT_ROUND_BRACKET:
			case NUM_INT:
			case NUM_FLOAT:
			case STRING_LITERAL_DOUBLE_QUOTED:
			case STRING_LITERAL_SINGLE_QUOTED:
			case PLUS:
			case MINUS:
			case POSITIONAL_PARAM:
			case NAMED_PARAM:
			{
				left=arithmeticExpression();
				node=simpleConditionalExpressionRemainder(left);
				break;
			}
			case EXISTS:
			{
				node=existsExpression();
				break;
			}
			default:
			{
				throw new NoViableAltException(LT(1), getFilename());
			}
			}
		}
		catch (RecognitionException ex) {
			if (inputState.guessing==0) {
				reportError(ex);
				recover(ex,_tokenSet_27);
			} else {
			  throw ex;
			}
		}
		return node;
	}
	
	public final Node  arithmeticExpression() throws RecognitionException, TokenStreamException {
		Node node;
		
		
			node = null; 
		
		
		try {      // for error handling
			if ((_tokenSet_33.member(LA(1))) && (_tokenSet_34.member(LA(2))) && (_tokenSet_35.member(LA(3)))) {
				node=simpleArithmeticExpression();
			}
			else if ((LA(1)==LEFT_ROUND_BRACKET) && (LA(2)==SELECT) && (_tokenSet_36.member(LA(3)))) {
				match(LEFT_ROUND_BRACKET);
				node=subquery();
				match(RIGHT_ROUND_BRACKET);
			}
			else {
				throw new NoViableAltException(LT(1), getFilename());
			}
			
		}
		catch (RecognitionException ex) {
			if (inputState.guessing==0) {
				reportError(ex);
				recover(ex,_tokenSet_37);
			} else {
			  throw ex;
			}
		}
		return node;
	}
	
	public final Node  simpleConditionalExpressionRemainder(
		Node left
	) throws RecognitionException, TokenStreamException {
		Node node;
		
		
			node = null; 
		
		
		try {      // for error handling
			if ((_tokenSet_38.member(LA(1)))) {
				node=comparisonExpression(left);
			}
			else {
				boolean synPredMatched103 = false;
				if (((LA(1)==BETWEEN||LA(1)==NOT) && (_tokenSet_39.member(LA(2))) && (_tokenSet_40.member(LA(3))))) {
					int _m103 = mark();
					synPredMatched103 = true;
					inputState.guessing++;
					try {
						{
						{
						switch ( LA(1)) {
						case NOT:
						{
							match(NOT);
							break;
						}
						case BETWEEN:
						{
							break;
						}
						default:
						{
							throw new NoViableAltException(LT(1), getFilename());
						}
						}
						}
						match(BETWEEN);
						}
					}
					catch (RecognitionException pe) {
						synPredMatched103 = false;
					}
					rewind(_m103);
inputState.guessing--;
				}
				if ( synPredMatched103 ) {
					node=betweenExpression(left);
				}
				else {
					boolean synPredMatched106 = false;
					if (((LA(1)==LIKE||LA(1)==NOT) && (_tokenSet_41.member(LA(2))) && (_tokenSet_42.member(LA(3))))) {
						int _m106 = mark();
						synPredMatched106 = true;
						inputState.guessing++;
						try {
							{
							{
							switch ( LA(1)) {
							case NOT:
							{
								match(NOT);
								break;
							}
							case LIKE:
							{
								break;
							}
							default:
							{
								throw new NoViableAltException(LT(1), getFilename());
							}
							}
							}
							match(LIKE);
							}
						}
						catch (RecognitionException pe) {
							synPredMatched106 = false;
						}
						rewind(_m106);
inputState.guessing--;
					}
					if ( synPredMatched106 ) {
						node=likeExpression(left);
					}
					else {
						boolean synPredMatched109 = false;
						if (((LA(1)==IN||LA(1)==NOT) && (LA(2)==IN||LA(2)==LEFT_ROUND_BRACKET) && (_tokenSet_43.member(LA(3))))) {
							int _m109 = mark();
							synPredMatched109 = true;
							inputState.guessing++;
							try {
								{
								{
								switch ( LA(1)) {
								case NOT:
								{
									match(NOT);
									break;
								}
								case IN:
								{
									break;
								}
								default:
								{
									throw new NoViableAltException(LT(1), getFilename());
								}
								}
								}
								match(IN);
								}
							}
							catch (RecognitionException pe) {
								synPredMatched109 = false;
							}
							rewind(_m109);
inputState.guessing--;
						}
						if ( synPredMatched109 ) {
							node=inExpression(left);
						}
						else {
							boolean synPredMatched112 = false;
							if (((LA(1)==IS) && (LA(2)==NOT||LA(2)==NULL) && (_tokenSet_44.member(LA(3))))) {
								int _m112 = mark();
								synPredMatched112 = true;
								inputState.guessing++;
								try {
									{
									match(IS);
									{
									switch ( LA(1)) {
									case NOT:
									{
										match(NOT);
										break;
									}
									case NULL:
									{
										break;
									}
									default:
									{
										throw new NoViableAltException(LT(1), getFilename());
									}
									}
									}
									match(NULL);
									}
								}
								catch (RecognitionException pe) {
									synPredMatched112 = false;
								}
								rewind(_m112);
inputState.guessing--;
							}
							if ( synPredMatched112 ) {
								node=nullComparisonExpression(left);
							}
							else if ((LA(1)==IS) && (LA(2)==EMPTY||LA(2)==NOT) && (_tokenSet_45.member(LA(3)))) {
								node=emptyCollectionComparisonExpression(left);
							}
							else if ((LA(1)==MEMBER||LA(1)==NOT) && (LA(2)==MEMBER||LA(2)==OF||LA(2)==IDENT) && (LA(3)==OF||LA(3)==IDENT||LA(3)==DOT)) {
								node=collectionMemberExpression(left);
							}
							else {
								throw new NoViableAltException(LT(1), getFilename());
							}
							}}}}
						}
						catch (RecognitionException ex) {
							if (inputState.guessing==0) {
								reportError(ex);
								recover(ex,_tokenSet_27);
							} else {
							  throw ex;
							}
						}
						return node;
					}
					
	public final Node  existsExpression() throws RecognitionException, TokenStreamException {
		Node node;
		
		Token  t = null;
		
			node = null; 
			Node subqr;
			List nodes = new ArrayList();
		
		
		try {      // for error handling
			t = LT(1);
			match(EXISTS);
			match(LEFT_ROUND_BRACKET);
			subqr=subquery();
			match(RIGHT_ROUND_BRACKET);
			if ( inputState.guessing==0 ) {
				
				nodes.add(subqr);
					node = factory.create(NodeFactory.EXISTSEXPR, nodes );
				
			}
		}
		catch (RecognitionException ex) {
			if (inputState.guessing==0) {
				reportError(ex);
				recover(ex,_tokenSet_27);
			} else {
			  throw ex;
			}
		}
		return node;
	}
	
	public final Node  comparisonExpression(
		Node left
	) throws RecognitionException, TokenStreamException {
		Node node;
		
		Token  t1 = null;
		Token  t2 = null;
		Token  t3 = null;
		Token  t4 = null;
		Token  t5 = null;
		Token  t6 = null;
		
			node = null; 
			Node right;
			List nodes = new ArrayList();
		
		
		try {      // for error handling
			switch ( LA(1)) {
			case EQUALS:
			{
				t1 = LT(1);
				match(EQUALS);
				right=comparisonExpressionRightOperand();
				if ( inputState.guessing==0 ) {
					
						nodes.add(t1.getText());
						nodes.add(right); 
						nodes.add(left); 
						node = factory.create(NodeFactory.COMPARISONEXPR, nodes );
					
				}
				break;
			}
			case NOT_EQUAL_TO:
			{
				t2 = LT(1);
				match(NOT_EQUAL_TO);
				right=comparisonExpressionRightOperand();
				if ( inputState.guessing==0 ) {
					
						nodes.add(t2.getText());
						nodes.add(right); 
						nodes.add(left); 
						node = factory.create(NodeFactory.COMPARISONEXPR, nodes );
					
				}
				break;
			}
			case GREATER_THAN:
			{
				t3 = LT(1);
				match(GREATER_THAN);
				right=comparisonExpressionRightOperand();
				if ( inputState.guessing==0 ) {
					
						nodes.add(t3.getText());
						nodes.add(right); 
						nodes.add(left); 
						node = factory.create(NodeFactory.COMPARISONEXPR, nodes );
					
				}
				break;
			}
			case GREATER_THAN_EQUAL_TO:
			{
				t4 = LT(1);
				match(GREATER_THAN_EQUAL_TO);
				right=comparisonExpressionRightOperand();
				if ( inputState.guessing==0 ) {
					
						nodes.add(t4.getText());
						nodes.add(right); 
						nodes.add(left); 
						node = factory.create(NodeFactory.COMPARISONEXPR, nodes );
					
				}
				break;
			}
			case LESS_THAN:
			{
				t5 = LT(1);
				match(LESS_THAN);
				right=comparisonExpressionRightOperand();
				if ( inputState.guessing==0 ) {
					
						nodes.add(t5.getText());
						nodes.add(right);
						nodes.add(left);  
						node = factory.create(NodeFactory.COMPARISONEXPR, nodes );
					
				}
				break;
			}
			case LESS_THAN_EQUAL_TO:
			{
				t6 = LT(1);
				match(LESS_THAN_EQUAL_TO);
				right=comparisonExpressionRightOperand();
				if ( inputState.guessing==0 ) {
					
						nodes.add(t6.getText());
						nodes.add(right); 
						nodes.add(left); 
						node = factory.create(NodeFactory.COMPARISONEXPR, nodes );
					
				}
				break;
			}
			default:
			{
				throw new NoViableAltException(LT(1), getFilename());
			}
			}
		}
		catch (RecognitionException ex) {
			if (inputState.guessing==0) {
				reportError(ex);
				recover(ex,_tokenSet_27);
			} else {
			  throw ex;
			}
		}
		return node;
	}
	
	public final Node  betweenExpression(
		Node left
	) throws RecognitionException, TokenStreamException {
		Node node;
		
		Token  n = null;
		Token  t = null;
		
		node = null; 
		Node lower, upper;
		List nodes = new ArrayList();
		
		
		try {      // for error handling
			{
			switch ( LA(1)) {
			case NOT:
			{
				n = LT(1);
				match(NOT);
				if ( inputState.guessing==0 ) {
					nodes.add(n.getText());
				}
				break;
			}
			case BETWEEN:
			{
				break;
			}
			default:
			{
				throw new NoViableAltException(LT(1), getFilename());
			}
			}
			}
			t = LT(1);
			match(BETWEEN);
			lower=arithmeticExpression();
			match(AND);
			upper=arithmeticExpression();
			if ( inputState.guessing==0 ) {
				
					nodes.add(lower);
					nodes.add(upper);
					nodes.add(left);
				node = factory.create(NodeFactory.BETWEENEXPR, nodes );
				
			}
		}
		catch (RecognitionException ex) {
			if (inputState.guessing==0) {
				reportError(ex);
				recover(ex,_tokenSet_27);
			} else {
			  throw ex;
			}
		}
		return node;
	}
	
	public final Node  likeExpression(
		Node left
	) throws RecognitionException, TokenStreamException {
		Node node;
		
		Token  n = null;
		Token  t = null;
		
		node = null; 
		boolean notlike = false;
		Node pattern, esc = null;
		List nodes = new ArrayList();
		
		
		try {      // for error handling
			{
			switch ( LA(1)) {
			case NOT:
			{
				n = LT(1);
				match(NOT);
				if ( inputState.guessing==0 ) {
					notlike = true;
				}
				break;
			}
			case LIKE:
			{
				break;
			}
			default:
			{
				throw new NoViableAltException(LT(1), getFilename());
			}
			}
			}
			if ( inputState.guessing==0 ) {
				nodes.add(notlike);
			}
			t = LT(1);
			match(LIKE);
			pattern=patternValue();
			if ( inputState.guessing==0 ) {
				nodes.add(pattern);
			}
			{
			switch ( LA(1)) {
			case ESCAPE:
			{
				esc=escape();
				if ( inputState.guessing==0 ) {
					nodes.add(esc);
				}
				break;
			}
			case EOF:
			case AND:
			case GROUP:
			case HAVING:
			case OR:
			case ORDER:
			case RIGHT_ROUND_BRACKET:
			{
				break;
			}
			default:
			{
				throw new NoViableAltException(LT(1), getFilename());
			}
			}
			}
			if ( inputState.guessing==0 ) {
				
					nodes.add(left);
				node = factory.create(NodeFactory.LIKEEXPR, nodes );
				
			}
		}
		catch (RecognitionException ex) {
			if (inputState.guessing==0) {
				reportError(ex);
				recover(ex,_tokenSet_27);
			} else {
			  throw ex;
			}
		}
		return node;
	}
	
	public final Node  inExpression(
		Node left
	) throws RecognitionException, TokenStreamException {
		Node node;
		
		Token  n = null;
		Token  t = null;
		
		node = null;
		List nodes = new ArrayList();
		Node subqr, item; 
		
		
		try {      // for error handling
			{
			switch ( LA(1)) {
			case NOT:
			{
				n = LT(1);
				match(NOT);
				break;
			}
			case IN:
			{
				break;
			}
			default:
			{
				throw new NoViableAltException(LT(1), getFilename());
			}
			}
			}
			t = LT(1);
			match(IN);
			match(LEFT_ROUND_BRACKET);
			{
			switch ( LA(1)) {
			case FALSE:
			case TRUE:
			case NUM_INT:
			case NUM_FLOAT:
			case STRING_LITERAL_DOUBLE_QUOTED:
			case STRING_LITERAL_SINGLE_QUOTED:
			case POSITIONAL_PARAM:
			case NAMED_PARAM:
			{
				item=inItem();
				if ( inputState.guessing==0 ) {
					nodes.add(item);
				}
				{
				_loop119:
				do {
					if ((LA(1)==COMMA)) {
						match(COMMA);
						item=inItem();
						if ( inputState.guessing==0 ) {
							nodes.add(item);
						}
					}
					else {
						break _loop119;
					}
					
				} while (true);
				}
				break;
			}
			case SELECT:
			{
				subqr=subquery();
				if ( inputState.guessing==0 ) {
					nodes.add(subqr);
				}
				break;
			}
			default:
			{
				throw new NoViableAltException(LT(1), getFilename());
			}
			}
			}
			match(RIGHT_ROUND_BRACKET);
			if ( inputState.guessing==0 ) {
				
					nodes.add(left);
				node = factory.create(NodeFactory.BETWEENEXPR, nodes );
				
			}
		}
		catch (RecognitionException ex) {
			if (inputState.guessing==0) {
				reportError(ex);
				recover(ex,_tokenSet_27);
			} else {
			  throw ex;
			}
		}
		return node;
	}
	
	public final Node  nullComparisonExpression(
		Node left
	) throws RecognitionException, TokenStreamException {
		Node node;
		
		Token  t = null;
		Token  n = null;
		
			node = null; 
			List nodes = new ArrayList();
		
		
		try {      // for error handling
			t = LT(1);
			match(IS);
			{
			switch ( LA(1)) {
			case NOT:
			{
				n = LT(1);
				match(NOT);
				if ( inputState.guessing==0 ) {
					nodes.add(n.getText());
				}
				break;
			}
			case NULL:
			{
				break;
			}
			default:
			{
				throw new NoViableAltException(LT(1), getFilename());
			}
			}
			}
			match(NULL);
			if ( inputState.guessing==0 ) {
				
					nodes.add(left);
					node = factory.create(NodeFactory.ESCAPE, nodes ); 
				
			}
		}
		catch (RecognitionException ex) {
			if (inputState.guessing==0) {
				reportError(ex);
				recover(ex,_tokenSet_27);
			} else {
			  throw ex;
			}
		}
		return node;
	}
	
	public final Node  emptyCollectionComparisonExpression(
		Node left
	) throws RecognitionException, TokenStreamException {
		Node node;
		
		Token  t = null;
		Token  n = null;
		
			node = null; 
			List nodes = new ArrayList();
		
		
		try {      // for error handling
			t = LT(1);
			match(IS);
			{
			switch ( LA(1)) {
			case NOT:
			{
				n = LT(1);
				match(NOT);
				if ( inputState.guessing==0 ) {
					nodes.add(n.getText());
				}
				break;
			}
			case EMPTY:
			{
				break;
			}
			default:
			{
				throw new NoViableAltException(LT(1), getFilename());
			}
			}
			}
			match(EMPTY);
			if ( inputState.guessing==0 ) {
				
					nodes.add(left);
					node = factory.create(NodeFactory.ISEMPTY, nodes ); 
				
			}
		}
		catch (RecognitionException ex) {
			if (inputState.guessing==0) {
				reportError(ex);
				recover(ex,_tokenSet_27);
			} else {
			  throw ex;
			}
		}
		return node;
	}
	
	public final Node  collectionMemberExpression(
		Node left
	) throws RecognitionException, TokenStreamException {
		Node node;
		
		Token  n = null;
		Token  t = null;
		
			node = null; 
			Node pathExpr;
			List nodes = new ArrayList();
		
		
		try {      // for error handling
			{
			switch ( LA(1)) {
			case NOT:
			{
				n = LT(1);
				match(NOT);
				break;
			}
			case MEMBER:
			{
				break;
			}
			default:
			{
				throw new NoViableAltException(LT(1), getFilename());
			}
			}
			}
			t = LT(1);
			match(MEMBER);
			{
			switch ( LA(1)) {
			case OF:
			{
				match(OF);
				break;
			}
			case IDENT:
			{
				break;
			}
			default:
			{
				throw new NoViableAltException(LT(1), getFilename());
			}
			}
			}
			pathExpr=pathExpression();
			if ( inputState.guessing==0 ) {
				
					nodes.add(pathExpr);
				nodes.add(left);
					node = factory.create(NodeFactory.COLLECTIONMEMBEREXPR, nodes ); 
				
			}
		}
		catch (RecognitionException ex) {
			if (inputState.guessing==0) {
				reportError(ex);
				recover(ex,_tokenSet_27);
			} else {
			  throw ex;
			}
		}
		return node;
	}
	
	public final Node  inItem() throws RecognitionException, TokenStreamException {
		Node node;
		
		
			node = null; 
		
		
		try {      // for error handling
			switch ( LA(1)) {
			case FALSE:
			case TRUE:
			case NUM_INT:
			case NUM_FLOAT:
			case STRING_LITERAL_DOUBLE_QUOTED:
			case STRING_LITERAL_SINGLE_QUOTED:
			{
				node=literal();
				break;
			}
			case POSITIONAL_PARAM:
			case NAMED_PARAM:
			{
				node=inputParameter();
				break;
			}
			default:
			{
				throw new NoViableAltException(LT(1), getFilename());
			}
			}
		}
		catch (RecognitionException ex) {
			if (inputState.guessing==0) {
				reportError(ex);
				recover(ex,_tokenSet_19);
			} else {
			  throw ex;
			}
		}
		return node;
	}
	
	public final Node  literal() throws RecognitionException, TokenStreamException {
		Node node;
		
		node = null;
		
		try {      // for error handling
			switch ( LA(1)) {
			case NUM_INT:
			case NUM_FLOAT:
			{
				node=literalNumeric();
				break;
			}
			case FALSE:
			case TRUE:
			{
				node=literalBoolean();
				break;
			}
			case STRING_LITERAL_DOUBLE_QUOTED:
			case STRING_LITERAL_SINGLE_QUOTED:
			{
				node=literalString();
				break;
			}
			default:
			{
				throw new NoViableAltException(LT(1), getFilename());
			}
			}
		}
		catch (RecognitionException ex) {
			if (inputState.guessing==0) {
				reportError(ex);
				recover(ex,_tokenSet_19);
			} else {
			  throw ex;
			}
		}
		return node;
	}
	
	public final Node  inputParameter() throws RecognitionException, TokenStreamException {
		Node node;
		
		Token  p = null;
		Token  n = null;
		
			node = null; 
			List nodes = new ArrayList();
		
		
		try {      // for error handling
			switch ( LA(1)) {
			case POSITIONAL_PARAM:
			{
				p = LT(1);
				match(POSITIONAL_PARAM);
				if ( inputState.guessing==0 ) {
					
					String text = p.getText().substring(1);
					nodes.add(text);
					node = factory.create(NodeFactory.POSITIONALPARAMETER, nodes );
					
				}
				break;
			}
			case NAMED_PARAM:
			{
				n = LT(1);
				match(NAMED_PARAM);
				if ( inputState.guessing==0 ) {
					
					String text = n.getText().substring(1);
					nodes.add(text);
					node = factory.create(NodeFactory.NAMEDPARAMETER, nodes );
					
				}
				break;
			}
			default:
			{
				throw new NoViableAltException(LT(1), getFilename());
			}
			}
		}
		catch (RecognitionException ex) {
			if (inputState.guessing==0) {
				reportError(ex);
				recover(ex,_tokenSet_46);
			} else {
			  throw ex;
			}
		}
		return node;
	}
	
	public final Node  literalNumeric() throws RecognitionException, TokenStreamException {
		Node node;
		
		Token  i = null;
		Token  f = null;
		
			node = null; 
			List nodes = new ArrayList();
		
		
		try {      // for error handling
			switch ( LA(1)) {
			case NUM_INT:
			{
				i = LT(1);
				match(NUM_INT);
				if ( inputState.guessing==0 ) {
					
							nodes.add(i.getText());
						node = factory.create(NodeFactory.LITERAL, nodes );
					
				}
				break;
			}
			case NUM_FLOAT:
			{
				f = LT(1);
				match(NUM_FLOAT);
				if ( inputState.guessing==0 ) {
					
							nodes.add(f.getText()); 
						node = factory.create(NodeFactory.LITERAL, nodes );
						
				}
				break;
			}
			default:
			{
				throw new NoViableAltException(LT(1), getFilename());
			}
			}
		}
		catch (RecognitionException ex) {
			if (inputState.guessing==0) {
				reportError(ex);
				recover(ex,_tokenSet_47);
			} else {
			  throw ex;
			}
		}
		return node;
	}
	
	public final Node  literalBoolean() throws RecognitionException, TokenStreamException {
		Node node;
		
		Token  t = null;
		Token  f = null;
		
			node = null; 
			List nodes = new ArrayList();
		
		
		try {      // for error handling
			switch ( LA(1)) {
			case TRUE:
			{
				t = LT(1);
				match(TRUE);
				if ( inputState.guessing==0 ) {
					
							nodes.add(t.getText()); 
						node = factory.create(NodeFactory.LITERAL, nodes );
						
				}
				break;
			}
			case FALSE:
			{
				f = LT(1);
				match(FALSE);
				if ( inputState.guessing==0 ) {
					
							nodes.add(f.getText());
						node = factory.create(NodeFactory.LITERAL, nodes );
					
				}
				break;
			}
			default:
			{
				throw new NoViableAltException(LT(1), getFilename());
			}
			}
		}
		catch (RecognitionException ex) {
			if (inputState.guessing==0) {
				reportError(ex);
				recover(ex,_tokenSet_19);
			} else {
			  throw ex;
			}
		}
		return node;
	}
	
	public final Node  literalString() throws RecognitionException, TokenStreamException {
		Node node;
		
		Token  d = null;
		Token  s = null;
		
			node = null; 
			List nodes = new ArrayList();
		
		
		try {      // for error handling
			switch ( LA(1)) {
			case STRING_LITERAL_DOUBLE_QUOTED:
			{
				d = LT(1);
				match(STRING_LITERAL_DOUBLE_QUOTED);
				if ( inputState.guessing==0 ) {
					
							nodes.add(d.getText()); 
					node = factory.create(NodeFactory.LITERAL, nodes );
					
				}
				break;
			}
			case STRING_LITERAL_SINGLE_QUOTED:
			{
				s = LT(1);
				match(STRING_LITERAL_SINGLE_QUOTED);
				if ( inputState.guessing==0 ) {
					
						nodes.add(s.getText()); 
					node = factory.create(NodeFactory.LITERAL, nodes );
					
				}
				break;
			}
			default:
			{
				throw new NoViableAltException(LT(1), getFilename());
			}
			}
		}
		catch (RecognitionException ex) {
			if (inputState.guessing==0) {
				reportError(ex);
				recover(ex,_tokenSet_46);
			} else {
			  throw ex;
			}
		}
		return node;
	}
	
	public final Node  patternValue() throws RecognitionException, TokenStreamException {
		Node node;
		
		
			node = null; 
		
		
		try {      // for error handling
			switch ( LA(1)) {
			case STRING_LITERAL_DOUBLE_QUOTED:
			case STRING_LITERAL_SINGLE_QUOTED:
			{
				node=literalString();
				break;
			}
			case POSITIONAL_PARAM:
			case NAMED_PARAM:
			{
				node=inputParameter();
				break;
			}
			default:
			{
				throw new NoViableAltException(LT(1), getFilename());
			}
			}
		}
		catch (RecognitionException ex) {
			if (inputState.guessing==0) {
				reportError(ex);
				recover(ex,_tokenSet_48);
			} else {
			  throw ex;
			}
		}
		return node;
	}
	
	public final Node  escape() throws RecognitionException, TokenStreamException {
		Node node;
		
		Token  t = null;
		
			node = null; 
			Node literal;
			List nodes = new ArrayList();
		
		
		try {      // for error handling
			t = LT(1);
			match(ESCAPE);
			if ( inputState.guessing==0 ) {
				nodes.add(t.getText());
			}
			literal=literalString();
			if ( inputState.guessing==0 ) {
				nodes.add(literal);
			}
			if ( inputState.guessing==0 ) {
				node = factory.create(NodeFactory.ESCAPE, nodes );
			}
		}
		catch (RecognitionException ex) {
			if (inputState.guessing==0) {
				reportError(ex);
				recover(ex,_tokenSet_27);
			} else {
			  throw ex;
			}
		}
		return node;
	}
	
	public final Node  anyOrAllExpression() throws RecognitionException, TokenStreamException {
		Node node;
		
		Token  a = null;
		Token  y = null;
		Token  s = null;
		
			node = null;
			Node subqr;
			List nodes = new ArrayList();
		
		
		try {      // for error handling
			switch ( LA(1)) {
			case ALL:
			{
				a = LT(1);
				match(ALL);
				match(LEFT_ROUND_BRACKET);
				subqr=subquery();
				match(RIGHT_ROUND_BRACKET);
				if ( inputState.guessing==0 ) {
					
						nodes.add(a.getText());
						nodes.add(subqr);
						node = factory.create(NodeFactory.ANYALLEXPR, nodes );
					
				}
				break;
			}
			case ANY:
			{
				y = LT(1);
				match(ANY);
				match(LEFT_ROUND_BRACKET);
				subqr=subquery();
				match(RIGHT_ROUND_BRACKET);
				if ( inputState.guessing==0 ) {
					
						nodes.add(y.getText());
						nodes.add(subqr);
						node = factory.create(NodeFactory.ANYALLEXPR, nodes );
					
				}
				break;
			}
			case SOME:
			{
				s = LT(1);
				match(SOME);
				match(LEFT_ROUND_BRACKET);
				subqr=subquery();
				match(RIGHT_ROUND_BRACKET);
				if ( inputState.guessing==0 ) {
					
						nodes.add(s.getText());
						nodes.add(subqr);
						node = factory.create(NodeFactory.ANYALLEXPR, nodes );
					
				}
				break;
			}
			default:
			{
				throw new NoViableAltException(LT(1), getFilename());
			}
			}
		}
		catch (RecognitionException ex) {
			if (inputState.guessing==0) {
				reportError(ex);
				recover(ex,_tokenSet_27);
			} else {
			  throw ex;
			}
		}
		return node;
	}
	
	public final Node  comparisonExpressionRightOperand() throws RecognitionException, TokenStreamException {
		Node node;
		
		
			node = null; 
		
		
		try {      // for error handling
			switch ( LA(1)) {
			case ABS:
			case AVG:
			case CONCAT:
			case COUNT:
			case LENGTH:
			case LOCATE:
			case LOWER:
			case MAX:
			case MIN:
			case MOD:
			case SIZE:
			case SQRT:
			case SUBSTRING:
			case SUM:
			case TRIM:
			case UPPER:
			case IDENT:
			case LEFT_ROUND_BRACKET:
			case NUM_INT:
			case NUM_FLOAT:
			case STRING_LITERAL_DOUBLE_QUOTED:
			case STRING_LITERAL_SINGLE_QUOTED:
			case PLUS:
			case MINUS:
			case POSITIONAL_PARAM:
			case NAMED_PARAM:
			{
				node=arithmeticExpression();
				break;
			}
			case ALL:
			case ANY:
			case SOME:
			{
				node=anyOrAllExpression();
				break;
			}
			default:
			{
				throw new NoViableAltException(LT(1), getFilename());
			}
			}
		}
		catch (RecognitionException ex) {
			if (inputState.guessing==0) {
				reportError(ex);
				recover(ex,_tokenSet_27);
			} else {
			  throw ex;
			}
		}
		return node;
	}
	
	public final Node  arithmeticTerm() throws RecognitionException, TokenStreamException {
		Node node;
		
		Token  m = null;
		Token  d = null;
		
		node = null; 
		Node left, right;
		List nodes = new ArrayList();
		
		
		try {      // for error handling
			left=arithmeticFactor();
			if ( inputState.guessing==0 ) {
				nodes.add(left);
			}
			{
			_loop147:
			do {
				switch ( LA(1)) {
				case MULTIPLY:
				{
					m = LT(1);
					match(MULTIPLY);
					right=arithmeticFactor();
					if ( inputState.guessing==0 ) {
						nodes.add(m.getText()); nodes.add(right);
					}
					break;
				}
				case DIVIDE:
				{
					d = LT(1);
					match(DIVIDE);
					right=arithmeticFactor();
					if ( inputState.guessing==0 ) {
						nodes.add(d.getText()); nodes.add(right);
					}
					break;
				}
				default:
				{
					break _loop147;
				}
				}
			} while (true);
			}
			if ( inputState.guessing==0 ) {
				
					node = factory.create(NodeFactory.ARITHMETICTERM, nodes );
				
			}
		}
		catch (RecognitionException ex) {
			if (inputState.guessing==0) {
				reportError(ex);
				recover(ex,_tokenSet_49);
			} else {
			  throw ex;
			}
		}
		return node;
	}
	
	public final Node  arithmeticFactor() throws RecognitionException, TokenStreamException {
		Node node;
		
		Token  p = null;
		Token  m = null;
		
		node = null; 
		Node right;
		List nodes = new ArrayList();
		
		
		try {      // for error handling
			switch ( LA(1)) {
			case PLUS:
			{
				p = LT(1);
				match(PLUS);
				right=arithmeticPrimary();
				if ( inputState.guessing==0 ) {
					
						nodes.add(p.getText()); 
						nodes.add(right); 
						node = factory.create(NodeFactory.ARITHMETICFACTOR, nodes );
					
				}
				break;
			}
			case MINUS:
			{
				m = LT(1);
				match(MINUS);
				right=arithmeticPrimary();
				if ( inputState.guessing==0 ) {
					
						nodes.add(m.getText()); 
						nodes.add(right); 
						node = factory.create(NodeFactory.ARITHMETICFACTOR, nodes );
					
				}
				break;
			}
			case ABS:
			case AVG:
			case CONCAT:
			case COUNT:
			case LENGTH:
			case LOCATE:
			case LOWER:
			case MAX:
			case MIN:
			case MOD:
			case SIZE:
			case SQRT:
			case SUBSTRING:
			case SUM:
			case TRIM:
			case UPPER:
			case IDENT:
			case LEFT_ROUND_BRACKET:
			case NUM_INT:
			case NUM_FLOAT:
			case STRING_LITERAL_DOUBLE_QUOTED:
			case STRING_LITERAL_SINGLE_QUOTED:
			case POSITIONAL_PARAM:
			case NAMED_PARAM:
			{
				node=arithmeticPrimary();
				break;
			}
			default:
			{
				throw new NoViableAltException(LT(1), getFilename());
			}
			}
		}
		catch (RecognitionException ex) {
			if (inputState.guessing==0) {
				reportError(ex);
				recover(ex,_tokenSet_47);
			} else {
			  throw ex;
			}
		}
		return node;
	}
	
	public final Node  arithmeticPrimary() throws RecognitionException, TokenStreamException {
		Node node;
		
		
			node = null; 
		
		
		try {      // for error handling
			switch ( LA(1)) {
			case NUM_INT:
			case NUM_FLOAT:
			{
				node=literalNumeric();
				break;
			}
			case ABS:
			case LENGTH:
			case LOCATE:
			case MOD:
			case SIZE:
			case SQRT:
			{
				node=functionsReturningNumerics();
				break;
			}
			default:
				if ((LA(1)==IDENT) && (LA(2)==DOT) && (LA(3)==IDENT)) {
					node=pathExpression();
				}
				else if ((LA(1)==LEFT_ROUND_BRACKET) && (_tokenSet_33.member(LA(2))) && (_tokenSet_50.member(LA(3)))) {
					match(LEFT_ROUND_BRACKET);
					node=arithmeticExpression();
					match(RIGHT_ROUND_BRACKET);
				}
				else if ((LA(1)==POSITIONAL_PARAM||LA(1)==NAMED_PARAM) && (_tokenSet_47.member(LA(2))) && (_tokenSet_51.member(LA(3)))) {
					node=inputParameter();
				}
				else if (((_tokenSet_25.member(LA(1))) && (LA(2)==LEFT_ROUND_BRACKET) && (LA(3)==DISTINCT||LA(3)==IDENT))&&( aggregatesAllowed )) {
					node=aggregateExpression();
				}
				else if ((_tokenSet_52.member(LA(1))) && (_tokenSet_53.member(LA(2))) && (_tokenSet_54.member(LA(3)))) {
					node=stringExpression();
				}
				else if ((LA(1)==IDENT||LA(1)==POSITIONAL_PARAM||LA(1)==NAMED_PARAM) && (_tokenSet_55.member(LA(2))) && (_tokenSet_51.member(LA(3)))) {
					node=entityExpression();
				}
			else {
				throw new NoViableAltException(LT(1), getFilename());
			}
			}
		}
		catch (RecognitionException ex) {
			if (inputState.guessing==0) {
				reportError(ex);
				recover(ex,_tokenSet_47);
			} else {
			  throw ex;
			}
		}
		return node;
	}
	
	public final Node  functionsReturningNumerics() throws RecognitionException, TokenStreamException {
		Node node;
		
		Token  a = null;
		Token  l = null;
		Token  m = null;
		Token  s = null;
		Token  lo = null;
		Token  si = null;
		
			node = null; 
			Node expr, str, left, right, pattern, startPos = null;
			List nodes = new ArrayList();
		
		
		try {      // for error handling
			switch ( LA(1)) {
			case ABS:
			{
				a = LT(1);
				match(ABS);
				match(LEFT_ROUND_BRACKET);
				expr=simpleArithmeticExpression();
				match(RIGHT_ROUND_BRACKET);
				if ( inputState.guessing==0 ) {
					
						nodes.add(expr);
						node = factory.create(NodeFactory.ABS, nodes );
					
				}
				break;
			}
			case LENGTH:
			{
				l = LT(1);
				match(LENGTH);
				match(LEFT_ROUND_BRACKET);
				str=stringPrimary();
				match(RIGHT_ROUND_BRACKET);
				if ( inputState.guessing==0 ) {
					
						nodes.add(str);
						node = factory.create(NodeFactory.LENGTH, nodes );
					
				}
				break;
			}
			case MOD:
			{
				m = LT(1);
				match(MOD);
				match(LEFT_ROUND_BRACKET);
				left=simpleArithmeticExpression();
				match(COMMA);
				right=simpleArithmeticExpression();
				match(RIGHT_ROUND_BRACKET);
				if ( inputState.guessing==0 ) {
					
						nodes.add(left);
						nodes.add(right);
						node = factory.create(NodeFactory.MOD, nodes );
					
				}
				break;
			}
			case SQRT:
			{
				s = LT(1);
				match(SQRT);
				match(LEFT_ROUND_BRACKET);
				expr=simpleArithmeticExpression();
				match(RIGHT_ROUND_BRACKET);
				if ( inputState.guessing==0 ) {
					
						nodes.add(expr);
						node = factory.create(NodeFactory.SQRT, nodes );
					
				}
				break;
			}
			case LOCATE:
			{
				lo = LT(1);
				match(LOCATE);
				match(LEFT_ROUND_BRACKET);
				pattern=stringPrimary();
				match(COMMA);
				str=stringPrimary();
				{
				switch ( LA(1)) {
				case COMMA:
				{
					match(COMMA);
					startPos=simpleArithmeticExpression();
					break;
				}
				case RIGHT_ROUND_BRACKET:
				{
					break;
				}
				default:
				{
					throw new NoViableAltException(LT(1), getFilename());
				}
				}
				}
				match(RIGHT_ROUND_BRACKET);
				if ( inputState.guessing==0 ) {
					
						nodes.add(pattern);
						nodes.add(str);
						nodes.add(startPos);
						node = factory.create(NodeFactory.LOCATE, nodes );
					
				}
				break;
			}
			case SIZE:
			{
				si = LT(1);
				match(SIZE);
				match(LEFT_ROUND_BRACKET);
				expr=pathExpression();
				match(RIGHT_ROUND_BRACKET);
				if ( inputState.guessing==0 ) {
					
						nodes.add(expr);
						node = factory.create(NodeFactory.SIZE, nodes );
					
				}
				break;
			}
			default:
			{
				throw new NoViableAltException(LT(1), getFilename());
			}
			}
		}
		catch (RecognitionException ex) {
			if (inputState.guessing==0) {
				reportError(ex);
				recover(ex,_tokenSet_47);
			} else {
			  throw ex;
			}
		}
		return node;
	}
	
	public final Node  stringExpression() throws RecognitionException, TokenStreamException {
		Node node;
		
		
			node = null; 
		
		
		try {      // for error handling
			switch ( LA(1)) {
			case AVG:
			case CONCAT:
			case COUNT:
			case LOWER:
			case MAX:
			case MIN:
			case SUBSTRING:
			case SUM:
			case TRIM:
			case UPPER:
			case IDENT:
			case STRING_LITERAL_DOUBLE_QUOTED:
			case STRING_LITERAL_SINGLE_QUOTED:
			case POSITIONAL_PARAM:
			case NAMED_PARAM:
			{
				node=stringPrimary();
				break;
			}
			case LEFT_ROUND_BRACKET:
			{
				match(LEFT_ROUND_BRACKET);
				node=subquery();
				match(RIGHT_ROUND_BRACKET);
				break;
			}
			default:
			{
				throw new NoViableAltException(LT(1), getFilename());
			}
			}
		}
		catch (RecognitionException ex) {
			if (inputState.guessing==0) {
				reportError(ex);
				recover(ex,_tokenSet_47);
			} else {
			  throw ex;
			}
		}
		return node;
	}
	
	public final Node  entityExpression() throws RecognitionException, TokenStreamException {
		Node node;
		
		
			node = null; 
		
		
		try {      // for error handling
			switch ( LA(1)) {
			case IDENT:
			{
				node=pathExpression();
				break;
			}
			case POSITIONAL_PARAM:
			case NAMED_PARAM:
			{
				node=inputParameter();
				break;
			}
			default:
			{
				throw new NoViableAltException(LT(1), getFilename());
			}
			}
		}
		catch (RecognitionException ex) {
			if (inputState.guessing==0) {
				reportError(ex);
				recover(ex,_tokenSet_47);
			} else {
			  throw ex;
			}
		}
		return node;
	}
	
	public final Node  stringPrimary() throws RecognitionException, TokenStreamException {
		Node node;
		
		
			node = null; 
		
		
		try {      // for error handling
			switch ( LA(1)) {
			case IDENT:
			{
				node=pathExpression();
				break;
			}
			case STRING_LITERAL_DOUBLE_QUOTED:
			case STRING_LITERAL_SINGLE_QUOTED:
			{
				node=literalString();
				break;
			}
			case POSITIONAL_PARAM:
			case NAMED_PARAM:
			{
				node=inputParameter();
				break;
			}
			case CONCAT:
			case LOWER:
			case SUBSTRING:
			case TRIM:
			case UPPER:
			{
				node=functionsReturningStrings();
				break;
			}
			default:
				if (((_tokenSet_25.member(LA(1))))&&( aggregatesAllowed )) {
					node=aggregateExpression();
				}
			else {
				throw new NoViableAltException(LT(1), getFilename());
			}
			}
		}
		catch (RecognitionException ex) {
			if (inputState.guessing==0) {
				reportError(ex);
				recover(ex,_tokenSet_47);
			} else {
			  throw ex;
			}
		}
		return node;
	}
	
	public final Node  functionsReturningStrings() throws RecognitionException, TokenStreamException {
		Node node;
		
		Token  c = null;
		Token  s = null;
		Token  t = null;
		Token  u = null;
		Token  l = null;
		
			node = null; 
			Node firstArg, secondArg, string, start, length;
			Node trimChar = null;
			String trimSpec = null;
			List nodes = new ArrayList();
		
		
		try {      // for error handling
			switch ( LA(1)) {
			case CONCAT:
			{
				c = LT(1);
				match(CONCAT);
				match(LEFT_ROUND_BRACKET);
				firstArg=stringPrimary();
				match(COMMA);
				secondArg=stringPrimary();
				match(RIGHT_ROUND_BRACKET);
				if ( inputState.guessing==0 ) {
					
						nodes.add(firstArg);
						nodes.add(secondArg);
						node = factory.create(NodeFactory.CONCAT, nodes );
					
				}
				break;
			}
			case SUBSTRING:
			{
				s = LT(1);
				match(SUBSTRING);
				match(LEFT_ROUND_BRACKET);
				string=stringPrimary();
				match(COMMA);
				start=simpleArithmeticExpression();
				match(COMMA);
				length=simpleArithmeticExpression();
				match(RIGHT_ROUND_BRACKET);
				if ( inputState.guessing==0 ) {
					
						nodes.add(string);
						nodes.add(start);
						nodes.add(length);
						node = factory.create(NodeFactory.SUBSTRING, nodes );
					
				}
				break;
			}
			case TRIM:
			{
				t = LT(1);
				match(TRIM);
				match(LEFT_ROUND_BRACKET);
				{
				boolean synPredMatched159 = false;
				if (((_tokenSet_56.member(LA(1))) && (_tokenSet_57.member(LA(2))))) {
					int _m159 = mark();
					synPredMatched159 = true;
					inputState.guessing++;
					try {
						{
						trimSpecification();
						trimCharacter();
						match(FROM);
						}
					}
					catch (RecognitionException pe) {
						synPredMatched159 = false;
					}
					rewind(_m159);
inputState.guessing--;
				}
				if ( synPredMatched159 ) {
					trimSpec=trimSpecification();
					trimChar=trimCharacter();
					match(FROM);
				}
				else if ((_tokenSet_58.member(LA(1))) && ((LA(2) >= DOT && LA(2) <= RIGHT_ROUND_BRACKET))) {
				}
				else {
					throw new NoViableAltException(LT(1), getFilename());
				}
				
				}
				string=stringPrimary();
				match(RIGHT_ROUND_BRACKET);
				if ( inputState.guessing==0 ) {
					
						nodes.add(trimSpec);
						nodes.add(trimChar);
						nodes.add(string);
						node = factory.create(NodeFactory.TRIM, nodes );
					
				}
				break;
			}
			case UPPER:
			{
				u = LT(1);
				match(UPPER);
				match(LEFT_ROUND_BRACKET);
				string=stringPrimary();
				match(RIGHT_ROUND_BRACKET);
				if ( inputState.guessing==0 ) {
					
						nodes.add(string);
						node = factory.create(NodeFactory.UPPER, nodes );
					
				}
				break;
			}
			case LOWER:
			{
				l = LT(1);
				match(LOWER);
				match(LEFT_ROUND_BRACKET);
				string=stringPrimary();
				match(RIGHT_ROUND_BRACKET);
				if ( inputState.guessing==0 ) {
					
						nodes.add(string);
						node = factory.create(NodeFactory.LOWER, nodes );
					
				}
				break;
			}
			default:
			{
				throw new NoViableAltException(LT(1), getFilename());
			}
			}
		}
		catch (RecognitionException ex) {
			if (inputState.guessing==0) {
				reportError(ex);
				recover(ex,_tokenSet_47);
			} else {
			  throw ex;
			}
		}
		return node;
	}
	
	public final Node  functionsReturningDatetime() throws RecognitionException, TokenStreamException {
		Node node;
		
		Token  d = null;
		Token  t = null;
		Token  ts = null;
		
			node = null; 
			List nodes = new ArrayList();
		
		
		try {      // for error handling
			switch ( LA(1)) {
			case CURRENT_DATE:
			{
				d = LT(1);
				match(CURRENT_DATE);
				if ( inputState.guessing==0 ) {
					
						nodes.add(d.getText()); 
						node = factory.create(NodeFactory.DATETIME, nodes );
					
				}
				break;
			}
			case CURRENT_TIME:
			{
				t = LT(1);
				match(CURRENT_TIME);
				if ( inputState.guessing==0 ) {
					
						nodes.add(t.getText()); 
						node = factory.create(NodeFactory.DATETIME, nodes );
					
				}
				break;
			}
			case CURRENT_TIMESTAMP:
			{
				ts = LT(1);
				match(CURRENT_TIMESTAMP);
				if ( inputState.guessing==0 ) {
					
						nodes.add(ts.getText()); 
						node = factory.create(NodeFactory.DATETIME, nodes );
					
				}
				break;
			}
			default:
			{
				throw new NoViableAltException(LT(1), getFilename());
			}
			}
		}
		catch (RecognitionException ex) {
			if (inputState.guessing==0) {
				reportError(ex);
				recover(ex,_tokenSet_0);
			} else {
			  throw ex;
			}
		}
		return node;
	}
	
	public final String  trimSpecification() throws RecognitionException, TokenStreamException {
		String trimSpec;
		
		
			trimSpec = "BOTH"; 
		
		
		try {      // for error handling
			switch ( LA(1)) {
			case LEADING:
			{
				match(LEADING);
				if ( inputState.guessing==0 ) {
					trimSpec = "LEADING";
				}
				break;
			}
			case TRAILING:
			{
				match(TRAILING);
				if ( inputState.guessing==0 ) {
					trimSpec = "TRAILING";
				}
				break;
			}
			case BOTH:
			{
				match(BOTH);
				if ( inputState.guessing==0 ) {
					trimSpec = "BOTH";
				}
				break;
			}
			case FROM:
			case STRING_LITERAL_DOUBLE_QUOTED:
			case STRING_LITERAL_SINGLE_QUOTED:
			case POSITIONAL_PARAM:
			case NAMED_PARAM:
			{
				break;
			}
			default:
			{
				throw new NoViableAltException(LT(1), getFilename());
			}
			}
		}
		catch (RecognitionException ex) {
			if (inputState.guessing==0) {
				reportError(ex);
				recover(ex,_tokenSet_59);
			} else {
			  throw ex;
			}
		}
		return trimSpec;
	}
	
	public final Node  trimCharacter() throws RecognitionException, TokenStreamException {
		Node node;
		
		
			node = null; 
		
		
		try {      // for error handling
			switch ( LA(1)) {
			case STRING_LITERAL_DOUBLE_QUOTED:
			case STRING_LITERAL_SINGLE_QUOTED:
			{
				node=literalString();
				break;
			}
			case POSITIONAL_PARAM:
			case NAMED_PARAM:
			{
				node=inputParameter();
				break;
			}
			case FROM:
			{
				break;
			}
			default:
			{
				throw new NoViableAltException(LT(1), getFilename());
			}
			}
		}
		catch (RecognitionException ex) {
			if (inputState.guessing==0) {
				reportError(ex);
				recover(ex,_tokenSet_1);
			} else {
			  throw ex;
			}
		}
		return node;
	}
	
	
	public static final String[] _tokenNames = {
		"<0>",
		"EOF",
		"<2>",
		"NULL_TREE_LOOKAHEAD",
		"\"abs\"",
		"\"all\"",
		"\"and\"",
		"\"any\"",
		"\"as\"",
		"\"asc\"",
		"\"avg\"",
		"\"between\"",
		"\"both\"",
		"\"by\"",
		"\"concat\"",
		"\"count\"",
		"\"current_date\"",
		"\"current_time\"",
		"\"current_timestamp\"",
		"\"desc\"",
		"\"delete\"",
		"\"distinct\"",
		"\"empty\"",
		"\"escape\"",
		"\"exists\"",
		"\"false\"",
		"\"fetch\"",
		"\"from\"",
		"\"group\"",
		"\"having\"",
		"\"in\"",
		"\"inner\"",
		"\"is\"",
		"\"join\"",
		"\"leading\"",
		"\"left\"",
		"\"length\"",
		"\"like\"",
		"\"locate\"",
		"\"lower\"",
		"\"max\"",
		"\"member\"",
		"\"min\"",
		"\"mod\"",
		"\"new\"",
		"\"not\"",
		"\"null\"",
		"\"object\"",
		"\"of\"",
		"\"or\"",
		"\"order\"",
		"\"outer\"",
		"\"select\"",
		"\"set\"",
		"\"size\"",
		"\"sqrt\"",
		"\"some\"",
		"\"substring\"",
		"\"sum\"",
		"\"trailing\"",
		"\"trim\"",
		"\"true\"",
		"\"unknown\"",
		"\"update\"",
		"\"upper\"",
		"\"where\"",
		"COMMA",
		"IDENT",
		"DOT",
		"LEFT_ROUND_BRACKET",
		"RIGHT_ROUND_BRACKET",
		"EQUALS",
		"NUM_INT",
		"NUM_FLOAT",
		"STRING_LITERAL_DOUBLE_QUOTED",
		"STRING_LITERAL_SINGLE_QUOTED",
		"NOT_EQUAL_TO",
		"GREATER_THAN",
		"GREATER_THAN_EQUAL_TO",
		"LESS_THAN",
		"LESS_THAN_EQUAL_TO",
		"PLUS",
		"MINUS",
		"MULTIPLY",
		"DIVIDE",
		"POSITIONAL_PARAM",
		"NAMED_PARAM",
		"HEX_DIGIT",
		"WS",
		"TEXTCHAR",
		"EXPONENT",
		"FLOAT_SUFFIX",
		"ESC"
	};
	
	private static final long[] mk_tokenSet_0() {
		long[] data = { 2L, 0L};
		return data;
	}
	public static final BitSet _tokenSet_0 = new BitSet(mk_tokenSet_0());
	private static final long[] mk_tokenSet_1() {
		long[] data = { 134217728L, 0L};
		return data;
	}
	public static final BitSet _tokenSet_1 = new BitSet(mk_tokenSet_1());
	private static final long[] mk_tokenSet_2() {
		long[] data = { 1125900712148994L, 2L, 0L, 0L};
		return data;
	}
	public static final BitSet _tokenSet_2 = new BitSet(mk_tokenSet_2());
	private static final long[] mk_tokenSet_3() {
		long[] data = { 1125900712148994L, 64L, 0L, 0L};
		return data;
	}
	public static final BitSet _tokenSet_3 = new BitSet(mk_tokenSet_3());
	private static final long[] mk_tokenSet_4() {
		long[] data = { 1125900443713538L, 64L, 0L, 0L};
		return data;
	}
	public static final BitSet _tokenSet_4 = new BitSet(mk_tokenSet_4());
	private static final long[] mk_tokenSet_5() {
		long[] data = { 1125899906842626L, 64L, 0L, 0L};
		return data;
	}
	public static final BitSet _tokenSet_5 = new BitSet(mk_tokenSet_5());
	private static final long[] mk_tokenSet_6() {
		long[] data = { 9007199254740992L, 0L};
		return data;
	}
	public static final BitSet _tokenSet_6 = new BitSet(mk_tokenSet_6());
	private static final long[] mk_tokenSet_7() {
		long[] data = { 2L, 2L, 0L, 0L};
		return data;
	}
	public static final BitSet _tokenSet_7 = new BitSet(mk_tokenSet_7());
	private static final long[] mk_tokenSet_8() {
		long[] data = { 1125900712148994L, 70L, 0L, 0L};
		return data;
	}
	public static final BitSet _tokenSet_8 = new BitSet(mk_tokenSet_8());
	private static final long[] mk_tokenSet_9() {
		long[] data = { 1125945809305602L, 70L, 0L, 0L};
		return data;
	}
	public static final BitSet _tokenSet_9 = new BitSet(mk_tokenSet_9());
	private static final long[] mk_tokenSet_10() {
		long[] data = { 9007199254741250L, 10L, 0L, 0L};
		return data;
	}
	public static final BitSet _tokenSet_10 = new BitSet(mk_tokenSet_10());
	private static final long[] mk_tokenSet_11() {
		long[] data = { 67108864L, 8L, 0L, 0L};
		return data;
	}
	public static final BitSet _tokenSet_11 = new BitSet(mk_tokenSet_11());
	private static final long[] mk_tokenSet_12() {
		long[] data = { 1726422100478786L, 2093262L, 0L, 0L};
		return data;
	}
	public static final BitSet _tokenSet_12 = new BitSet(mk_tokenSet_12());
	private static final long[] mk_tokenSet_13() {
		long[] data = { 1125900577931266L, 68L, 0L, 0L};
		return data;
	}
	public static final BitSet _tokenSet_13 = new BitSet(mk_tokenSet_13());
	private static final long[] mk_tokenSet_14() {
		long[] data = { 2L, 6L, 0L, 0L};
		return data;
	}
	public static final BitSet _tokenSet_14 = new BitSet(mk_tokenSet_14());
	private static final long[] mk_tokenSet_15() {
		long[] data = { 1726376868579394L, 127174L, 0L, 0L};
		return data;
	}
	public static final BitSet _tokenSet_15 = new BitSet(mk_tokenSet_15());
	private static final long[] mk_tokenSet_16() {
		long[] data = { 134217728L, 4L, 0L, 0L};
		return data;
	}
	public static final BitSet _tokenSet_16 = new BitSet(mk_tokenSet_16());
	private static final long[] mk_tokenSet_17() {
		long[] data = { 1726377002797122L, 2093254L, 0L, 0L};
		return data;
	}
	public static final BitSet _tokenSet_17 = new BitSet(mk_tokenSet_17());
	private static final long[] mk_tokenSet_18() {
		long[] data = { 0L, 32L, 0L, 0L};
		return data;
	}
	public static final BitSet _tokenSet_18 = new BitSet(mk_tokenSet_18());
	private static final long[] mk_tokenSet_19() {
		long[] data = { 0L, 68L, 0L, 0L};
		return data;
	}
	public static final BitSet _tokenSet_19 = new BitSet(mk_tokenSet_19());
	private static final long[] mk_tokenSet_20() {
		long[] data = { 1125900443713538L, 68L, 0L, 0L};
		return data;
	}
	public static final BitSet _tokenSet_20 = new BitSet(mk_tokenSet_20());
	private static final long[] mk_tokenSet_21() {
		long[] data = { 2L, 4L, 0L, 0L};
		return data;
	}
	public static final BitSet _tokenSet_21 = new BitSet(mk_tokenSet_21());
	private static final long[] mk_tokenSet_22() {
		long[] data = { 0L, 64L, 0L, 0L};
		return data;
	}
	public static final BitSet _tokenSet_22 = new BitSet(mk_tokenSet_22());
	private static final long[] mk_tokenSet_23() {
		long[] data = { 805306368L, 66L, 0L, 0L};
		return data;
	}
	public static final BitSet _tokenSet_23 = new BitSet(mk_tokenSet_23());
	private static final long[] mk_tokenSet_24() {
		long[] data = { 805306368L, 70L, 0L, 0L};
		return data;
	}
	public static final BitSet _tokenSet_24 = new BitSet(mk_tokenSet_24());
	private static final long[] mk_tokenSet_25() {
		long[] data = { 288235873709884416L, 0L};
		return data;
	}
	public static final BitSet _tokenSet_25 = new BitSet(mk_tokenSet_25());
	private static final long[] mk_tokenSet_26() {
		long[] data = { 1688850665570306L, 64L, 0L, 0L};
		return data;
	}
	public static final BitSet _tokenSet_26 = new BitSet(mk_tokenSet_26());
	private static final long[] mk_tokenSet_27() {
		long[] data = { 1688850665570370L, 64L, 0L, 0L};
		return data;
	}
	public static final BitSet _tokenSet_27 = new BitSet(mk_tokenSet_27());
	private static final long[] mk_tokenSet_28() {
		long[] data = { 1639360635756135440L, 6688553L, 0L, 0L};
		return data;
	}
	public static final BitSet _tokenSet_28 = new BitSet(mk_tokenSet_28());
	private static final long[] mk_tokenSet_29() {
		long[] data = { 1643866577214426128L, 8388537L, 0L, 0L};
		return data;
	}
	public static final BitSet _tokenSet_29 = new BitSet(mk_tokenSet_29());
	private static final long[] mk_tokenSet_30() {
		long[] data = { 1639325451384046608L, 6688553L, 0L, 0L};
		return data;
	}
	public static final BitSet _tokenSet_30 = new BitSet(mk_tokenSet_30());
	private static final long[] mk_tokenSet_31() {
		long[] data = { 1643866577197648912L, 8388537L, 0L, 0L};
		return data;
	}
	public static final BitSet _tokenSet_31 = new BitSet(mk_tokenSet_31());
	private static final long[] mk_tokenSet_32() {
		long[] data = { 2292736784580271280L, 8388601L, 0L, 0L};
		return data;
	}
	public static final BitSet _tokenSet_32 = new BitSet(mk_tokenSet_32());
	private static final long[] mk_tokenSet_33() {
		long[] data = { 1639325451367269392L, 6688553L, 0L, 0L};
		return data;
	}
	public static final BitSet _tokenSet_33 = new BitSet(mk_tokenSet_33());
	private static final long[] mk_tokenSet_34() {
		long[] data = { 1645555427863219282L, 8388601L, 0L, 0L};
		return data;
	}
	public static final BitSet _tokenSet_34 = new BitSet(mk_tokenSet_34());
	private static final long[] mk_tokenSet_35() {
		long[] data = { 2294425635262627058L, 8388607L, 0L, 0L};
		return data;
	}
	public static final BitSet _tokenSet_35 = new BitSet(mk_tokenSet_35());
	private static final long[] mk_tokenSet_36() {
		long[] data = { 288235873711981568L, 8L, 0L, 0L};
		return data;
	}
	public static final BitSet _tokenSet_36 = new BitSet(mk_tokenSet_36());
	private static final long[] mk_tokenSet_37() {
		long[] data = { 1726376868579394L, 127168L, 0L, 0L};
		return data;
	}
	public static final BitSet _tokenSet_37 = new BitSet(mk_tokenSet_37());
	private static final long[] mk_tokenSet_38() {
		long[] data = { 0L, 127104L, 0L, 0L};
		return data;
	}
	public static final BitSet _tokenSet_38 = new BitSet(mk_tokenSet_38());
	private static final long[] mk_tokenSet_39() {
		long[] data = { 1639325451367271440L, 6688553L, 0L, 0L};
		return data;
	}
	public static final BitSet _tokenSet_39 = new BitSet(mk_tokenSet_39());
	private static final long[] mk_tokenSet_40() {
		long[] data = { 1643829050994639952L, 8261433L, 0L, 0L};
		return data;
	}
	public static final BitSet _tokenSet_40 = new BitSet(mk_tokenSet_40());
	private static final long[] mk_tokenSet_41() {
		long[] data = { 137438953472L, 6294528L, 0L, 0L};
		return data;
	}
	public static final BitSet _tokenSet_41 = new BitSet(mk_tokenSet_41());
	private static final long[] mk_tokenSet_42() {
		long[] data = { 1688850673958978L, 6294592L, 0L, 0L};
		return data;
	}
	public static final BitSet _tokenSet_42 = new BitSet(mk_tokenSet_42());
	private static final long[] mk_tokenSet_43() {
		long[] data = { 2310346608874618880L, 6295328L, 0L, 0L};
		return data;
	}
	public static final BitSet _tokenSet_43 = new BitSet(mk_tokenSet_43());
	private static final long[] mk_tokenSet_44() {
		long[] data = { 1759219409748034L, 64L, 0L, 0L};
		return data;
	}
	public static final BitSet _tokenSet_44 = new BitSet(mk_tokenSet_44());
	private static final long[] mk_tokenSet_45() {
		long[] data = { 1688850669764674L, 64L, 0L, 0L};
		return data;
	}
	public static final BitSet _tokenSet_45 = new BitSet(mk_tokenSet_45());
	private static final long[] mk_tokenSet_46() {
		long[] data = { 1726377011185730L, 2093254L, 0L, 0L};
		return data;
	}
	public static final BitSet _tokenSet_46 = new BitSet(mk_tokenSet_46());
	private static final long[] mk_tokenSet_47() {
		long[] data = { 1726376868579394L, 2093254L, 0L, 0L};
		return data;
	}
	public static final BitSet _tokenSet_47 = new BitSet(mk_tokenSet_47());
	private static final long[] mk_tokenSet_48() {
		long[] data = { 1688850673958978L, 64L, 0L, 0L};
		return data;
	}
	public static final BitSet _tokenSet_48 = new BitSet(mk_tokenSet_48());
	private static final long[] mk_tokenSet_49() {
		long[] data = { 1726376868579394L, 520390L, 0L, 0L};
		return data;
	}
	public static final BitSet _tokenSet_49 = new BitSet(mk_tokenSet_49());
	private static final long[] mk_tokenSet_50() {
		long[] data = { 1643829050994639888L, 8261497L, 0L, 0L};
		return data;
	}
	public static final BitSet _tokenSet_50 = new BitSet(mk_tokenSet_50());
	private static final long[] mk_tokenSet_51() {
		long[] data = { 1713461266015644914L, 8388591L, 0L, 0L};
		return data;
	}
	public static final BitSet _tokenSet_51 = new BitSet(mk_tokenSet_51());
	private static final long[] mk_tokenSet_52() {
		long[] data = { 1585273116148417536L, 6294569L, 0L, 0L};
		return data;
	}
	public static final BitSet _tokenSet_52 = new BitSet(mk_tokenSet_52());
	private static final long[] mk_tokenSet_53() {
		long[] data = { 6229976495949890L, 2093302L, 0L, 0L};
		return data;
	}
	public static final BitSet _tokenSet_53 = new BitSet(mk_tokenSet_53());
	private static final long[] mk_tokenSet_54() {
		long[] data = { 2289922035635256562L, 8388591L, 0L, 0L};
		return data;
	}
	public static final BitSet _tokenSet_54 = new BitSet(mk_tokenSet_54());
	private static final long[] mk_tokenSet_55() {
		long[] data = { 1726376868579394L, 2093270L, 0L, 0L};
		return data;
	}
	public static final BitSet _tokenSet_55 = new BitSet(mk_tokenSet_55());
	private static final long[] mk_tokenSet_56() {
		long[] data = { 576460769617514496L, 6294528L, 0L, 0L};
		return data;
	}
	public static final BitSet _tokenSet_56 = new BitSet(mk_tokenSet_56());
	private static final long[] mk_tokenSet_57() {
		long[] data = { 1585273116282635264L, 6294537L, 0L, 0L};
		return data;
	}
	public static final BitSet _tokenSet_57 = new BitSet(mk_tokenSet_57());
	private static final long[] mk_tokenSet_58() {
		long[] data = { 1585273116148417536L, 6294537L, 0L, 0L};
		return data;
	}
	public static final BitSet _tokenSet_58 = new BitSet(mk_tokenSet_58());
	private static final long[] mk_tokenSet_59() {
		long[] data = { 134217728L, 6294528L, 0L, 0L};
		return data;
	}
	public static final BitSet _tokenSet_59 = new BitSet(mk_tokenSet_59());
	
	}
