/* -------------------------------------------------------------------------*
 * PKUAS: Peking University Application Server
 * Copyright (C) 2006 Peking University
 *
 * This software is free; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * either version 2.1 or any later version.
 * 
 * This software is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this software; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 * ------------------------------------------------------------------------*/

package org.ow2.xplus.query.ejbql.tree;

import java.util.ArrayList;
import java.util.List;

import org.ow2.xplus.util.ClassStrategy;

/**
 * @author Lin Liang
 */

public class ConstructorExprNode extends AbstractNode implements ConstructorNode {

    private String className;

    private List<Node> paras = new ArrayList<Node>();

    private Class type;

    public ConstructorExprNode(List args) {
        super(args);

        className = (String) args.get(0);
        type = ClassStrategy.forName(className);

        for (int index = 1; index < args.size(); ++index) {
            paras.add((Node) args.get(index));
        }
    }

    @Override
    public void process() {
        if (processed)
            return;

        for (Node para : paras) {
            para.setRoot(getRoot());
            para.process();
        }

        super.process();
    }

    public String toSQLString() {
        StringBuilder str = new StringBuilder(" ");

        int size = paras.size();
        str.append(paras.get(0).toSQLString());

        for (int index = 1; index < size; ++index) {
            str.append(", ");
            str.append(paras.get(index).toSQLString());
        }

        return str.toString();
    }

    public String getClassName() {
        return className;
    }

    public Class getType() {
        return type;
    }

}
