/* -------------------------------------------------------------------------*
 * PKUAS: Peking University Application Server
 * Copyright (C) 2006 Peking University
 *
 * This software is free; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * either version 2.1 or any later version.
 * 
 * This software is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this software; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 * ------------------------------------------------------------------------*/

package org.ow2.xplus.query.ejbql.tree;

import java.util.List;

/**
 * @author Lin Liang
 */

public class IsEmptyNode extends AbstractNode {

    private String not = null;

    private Node left = null;

    @SuppressWarnings("unchecked")
    public IsEmptyNode(List args) {
        super(args);
        if (args.size() == 2) {
            not = (String) args.get(0);
            left = (Node) args.get(1);
        } else {
            left = (Node) args.get(0);
        }
    }

    @Override
    public void process() {
        if (processed)
            return;

        left.setRoot(getRoot());
        left.process();

        super.process();
    }

    public String toSQLString() {
        StringBuilder expr = new StringBuilder(1024);

        String tableId = left.toSQLString();
        if (not != null)
            expr.append(" EXISTS ( SELECT * FROM ");
        else
            expr.append(" NOT EXISTS ( SELECT * FROM ");

        String tableName = getRoot().getTree().getUnit().getClassMetaData(getRoot().getTree().getIdType(tableId).getName())
                .getPrimaryTableName();
        expr.append(getRoot().getTree().getDialect().getDeclaration(tableName, tableId));
        expr.append(" WHERE ");

        List<String> joins = getRoot().getFrom().getJoinDeclaration(tableId);
        expr.append(joins.get(0));

        for (int index = 1; index < joins.size(); ++index) {
            expr.append(" AND ");
            expr.append(joins.get(index));
        }
        expr.append(" ) ");

        return expr.toString();
    }

}
