/* -------------------------------------------------------------------------*
 * PKUAS: Peking University Application Server
 * Copyright (C) 2006 Peking University
 *
 * This software is free; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * either version 2.1 or any later version.
 * 
 * This software is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this software; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 * ------------------------------------------------------------------------*/

package org.ow2.xplus.query.ejbql.tree;

import java.util.List;

/**
 * @author Lin Liang
 */

public class LikeExprNode extends AbstractNode {

    private Boolean notlike = new Boolean(false);

    private Node pattern, esc, left;

    public LikeExprNode(List args) {
        super(args);

        if (args.size() == 3) {
            notlike = (Boolean) args.get(0);
            pattern = (Node) args.get(1);
            esc = null;
            left = (Node) args.get(2);
        } else {
            notlike = (Boolean) args.get(0);
            pattern = (Node) args.get(1);
            esc = (Node) args.get(2);
            left = (Node) args.get(3);
        }
    }

    @Override
    public void process() {
        if (processed)
            return;

        pattern.setRoot(getRoot());
        pattern.process();
        if (esc != null) {
            esc.setRoot(getRoot());
            esc.process();
        }
        left.setRoot(getRoot());
        left.process();

        super.process();
    }

    // F_NAME LIKE 'B%'
    // F_NAME LIKE 'B\_SMITH ESCAPE '\''
    public String toSQLString() {
        StringBuilder like = new StringBuilder(1024);

        like.append(left.toSQLString());
        like.append(" LIKE ");
        like.append(pattern.toSQLString());
        if (esc != null) {
            like.append(" ESCAPE '").append(esc.toSQLString()).append("'");
        }

        if (notlike.booleanValue())
            return "NOT (" + like.toString() + ")";
        return like.toString();
    }

}
