/* -------------------------------------------------------------------------*
 * PKUAS: Peking University Application Server
 * Copyright (C) 2006 Peking University
 *
 * This software is free; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * either version 2.1 or any later version.
 * 
 * This software is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this software; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 * ------------------------------------------------------------------------*/

package org.ow2.xplus.query.ejbql.tree;

import java.util.ArrayList;
import java.util.List;

import org.ow2.xplus.exception.EjbqlParserException;
import org.ow2.xplus.metadata.UnitMetaData;
import org.ow2.xplus.metadata.object.ClassMetaData;
import org.ow2.xplus.metadata.object.FieldMetaData;
import org.ow2.xplus.metadata.object.Relationship;
import org.ow2.xplus.metadata.relational.RdbColumn;
import org.ow2.xplus.metadata.relational.RdbForeignKey;
import org.ow2.xplus.metadata.relational.RdbRelation;
import org.ow2.xplus.metadata.relational.RdbRelationTable;
import org.ow2.xplus.metadata.relational.RdbTable;
import org.ow2.xplus.query.utils.NameGenerator;

/**
 * @author Lin Liang
 */

public class PathExprNode extends AbstractNode {

    private String idVar;

    private Class idType;

    private String tableIdName;

    private String columnName;

    private List<String> fields = new ArrayList<String>();

    private String ident;

    private boolean generateColumns = false;

    public PathExprNode(List args) {
        super(args);

        idVar = (String) args.get(0);
        for (int index = 1; index < args.size(); ++index) {
            fields.add((String) args.get(index));
        }
    }

    public void setGenerateColumns(boolean generateColumns) {
        this.generateColumns = generateColumns;
    }

    @Override
    public void process() {
        if (processed)
            return;

        String id = idVar;
        idType = getRoot().getTree().getIdType(id);

        UnitMetaData unit = getRoot().getTree().getUnit();

        for (int index = 0; index < fields.size(); ++index) {
            String field = fields.get(index);
            idType = getRoot().getTree().getIdType(id);
            ClassMetaData entityMetaData = unit.findClassMetaData(idType.getCanonicalName());

            if (entityMetaData == null) {
                throw new EjbqlParserException("field not exists", EjbqlParserException.EJBQL_PARSER_SEMANTIC_ERROR);
            }
            FieldMetaData fieldMetaData = entityMetaData.findFieldMetaData(field);

            while (fieldMetaData == null) {
                entityMetaData = unit.findClassMetaData(entityMetaData.getParent().getCanonicalName());
                fieldMetaData = entityMetaData.findFieldMetaData(field);
            }

            Relationship relationship = fieldMetaData.getRelation();
            if (relationship != null) {
                Class targetEntity = relationship.getTargetEntity();
                ClassMetaData targetEntityMetaData = unit.findClassMetaData(targetEntity.getCanonicalName());
                RdbTable targetTable = unit.getTable(targetEntityMetaData.getPrimaryTableName());

                String targetTableName = null;
                if (index == fields.size() - 1)
                    targetTableName = getIdent();
                if (targetTableName == null)
                    targetTableName = NameGenerator.getTableName();
                getRoot().getTree().addDeclaration(targetTableName, targetEntity);
                FromNode from = getRoot().getFrom();
                targetTableName = from.addDeclaration(targetTable.getName(), targetTableName);

                for (RdbRelation relation : targetTable.getRelations()) {
                    if (relation.getReferencedEntityName().equals(idType.getCanonicalName())) {
                        if (relation instanceof RdbForeignKey) {
                            processForeignKey((RdbForeignKey) relation, id, targetTableName, true);
                        } else {
                            processRelationTable((RdbRelationTable) relation, id, targetTableName, true);
                        }
                    }
                }

                if (targetTable.getRelations().size() == 0) {
                    RdbTable ownerTable = unit.getTable(entityMetaData.getPrimaryTableName());

                    for (RdbRelation relation : ownerTable.getRelations()) {
                        if (relation.getReferencedEntityName().equals(targetEntity.getCanonicalName())) {
                            if (relation instanceof RdbForeignKey) {
                                processForeignKey((RdbForeignKey) relation, id, targetTableName, false);
                            } else {
                                processRelationTable((RdbRelationTable) relation, id, targetTableName, false);
                            }
                        }
                    }
                }

                tableIdName = targetTableName;
                columnName = null;
                id = tableIdName;
                getRoot().getFrom().addDeclaration(targetTable.getName(), tableIdName);
                getRoot().getTree().addDeclaration(tableIdName, targetEntity);
            } else {
                RdbTable table = unit.getTable(entityMetaData.getPrimaryTableName());
                RdbColumn column = table.getColumn(field);
                tableIdName = id;
                columnName = column.getName();
            }

            idType = fieldMetaData.getType();
        }

        super.process();
    }

    private void processForeignKey(RdbForeignKey foreignKey, String ownerName, String targetName, boolean reverse) {
        List<String> joinColumnNames = foreignKey.getJoinColumnNames();
        List<String> referencedColumnNames = foreignKey.getReferencedColumnNames();

        getRoot().getFrom().saveRelation(targetName, ownerName);

        FromNode from = getRoot().getFrom();
        for (int index = 0; index < joinColumnNames.size(); ++index) {
            String condition = null;
            if (reverse) {
                condition = buildJoinCondition(targetName, joinColumnNames.get(index), ownerName, referencedColumnNames
                        .get(index));
            } else {
                condition = buildJoinCondition(ownerName, joinColumnNames.get(index), targetName, referencedColumnNames
                        .get(index));
            }

            from.addJoinDeclaration(targetName, condition);
        }
    }

    private void processRelationTable(RdbRelationTable relation, String ownerName, String targetName, boolean reverse) {
        RdbTable relationTable = relation.getRelationTable();
        String relationTableName = NameGenerator.getTableName();
        FromNode from = getRoot().getFrom();
        relationTableName = from.addDeclaration(relationTable.getName(), relationTableName);

        List<String> sourceRelationColumnNames = relation.getSourceRelationColumnNames();
        List<String> sourceColumnNames = relation.getSourceColumnNames();

        getRoot().getFrom().saveRelation(relationTableName, ownerName);
        for (int index = 0; index < sourceRelationColumnNames.size(); ++index) {
            String condition = buildJoinCondition(ownerName, sourceColumnNames.get(index), relationTableName,
                    sourceRelationColumnNames.get(index));
            from.addJoinDeclaration(targetName, condition);
        }

        getRoot().getFrom().saveRelation(targetName, relationTableName);
        List<String> targetRelationColumnNames = relation.getTargetRelationColumnNames();
        List<String> targetColumnNames = relation.getTargetColumnNames();
        for (int index = 0; index < targetRelationColumnNames.size(); ++index) {
            String condition = buildJoinCondition(relationTableName, targetRelationColumnNames.get(index), targetName,
                    targetColumnNames.get(index));
            from.addJoinDeclaration(targetName, condition);
        }
    }

    private String buildJoinCondition(String table1, String column1, String table2, String column2) {
        StringBuilder cond = new StringBuilder(256);
        cond.append(table1).append(".").append(column1).append(" = ").append(table2).append(".").append(column2);
        return cond.toString();
    }

    public Class getIdType() {
        return idType;
    }

    public String getTableName() {
        return tableIdName;
    }

    public String toSQLString() {
        if (columnName == null) {
            if (!generateColumns)
                return tableIdName;
            return generateColumn(tableIdName);
        }
        return tableIdName + "." + columnName + "";
    }

    private String generateColumn(String tableIdName) {
        StringBuilder selectColumns = new StringBuilder(1024);

        UnitMetaData unit = getRoot().getTree().getUnit();
        ClassMetaData idMetaData = unit.findClassMetaData(idType.getCanonicalName());
        RdbTable table = unit.getTable(idMetaData.getPrimaryTableName());

        for (RdbColumn column : table.getColumns()) {
            selectColumns.append(tableIdName + "." + getRoot().getTree().getDialect().getColumnName(column.getName()) + ", ");
        }

        String expr = selectColumns.toString().substring(0, selectColumns.length() - 2);
        return expr;
    }

    public String getIdent() {
        return ident;
    }

    public void setIdent(String ident) {
        this.ident = ident;
    }

}
