/* -------------------------------------------------------------------------*
 * PKUAS: Peking University Application Server
 * Copyright (C) 2006 Peking University
 *
 * This software is free; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * either version 2.1 or any later version.
 * 
 * This software is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this software; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 * ------------------------------------------------------------------------*/

package org.ow2.xplus.query.ejbql.tree;

import java.util.ArrayList;
import java.util.List;

/**
 * @author Lin Liang
 */

public class SelectNode extends AbstractNode {

    private Boolean distinct = new Boolean(false);

    private Class type = null;

    private List<Node> selectExprs = new ArrayList<Node>();

    public SelectNode(List args) {
        super(args);

        distinct = (Boolean) args.get(0);

        for (int index = 1; index < args.size(); ++index) {
            Node selectExpr = (Node) args.get(index);
            selectExprs.add(selectExpr);
        }
    }

    public Class getType() {
        return type;
    }

    @Override
    @SuppressWarnings("unchecked")
    public void process() {
        if (processed)
            return;

        // for (Node selectExpr : selectExprs) {
        Node selectExpr = selectExprs.get(0);
        if (selectExpr instanceof IdVarNode) {
            List idVar = new ArrayList();
            idVar.add(selectExpr);
            ObjectExprNode node = new ObjectExprNode(idVar);
            selectExpr = node;
        }

        selectExprs.clear();
        selectExprs.add(selectExpr);

        selectExpr.setRoot(getRoot());
        selectExpr.process();
        if (selectExpr instanceof ConstructorNode) {
            type = ((ConstructorNode) selectExpr).getType();
        } else if (selectExpr instanceof PathExprNode) {
            ((PathExprNode) selectExpr).setGenerateColumns(true);
            type = ((PathExprNode) selectExpr).getIdType();
        }
        // }

        super.process();
    }

    public String toSQLString() {
        StringBuilder select = new StringBuilder(1024);
        select.append(" SELECT ");
        if (distinct.booleanValue())
            select.append(" DISTINCT ");

        select.append(selectExprs.get(0).toSQLString());

        for (int index = 1; index < selectExprs.size(); ++index) {
            select.append(", ");
            select.append(selectExprs.get(index).toSQLString());
        }

        return select.toString();
    }

}
