/* -------------------------------------------------------------------------*
 * PKUAS: Peking University Application Server
 * Copyright (C) 2006 Peking University
 *
 * This software is free; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * either version 2.1 or any later version.
 * 
 * This software is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this software; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 * ------------------------------------------------------------------------*/

package org.ow2.xplus.query.ejbql.tree;

import java.util.ArrayList;
import java.util.List;

/**
 * @author Lin Liang
 */

public class SelectQueryNode extends QueryNode {

    private SelectNode select;

    private List<Node> conditions = new ArrayList<Node>();

    public SelectQueryNode(List args) {
        super(args);

        select = (SelectNode) args.get(0);
        from = (FromNode) args.get(1);

        int start = 2;

        if (args.size() > 2) {
            if (args.get(2) instanceof WhereNode) {
                where = (WhereNode) args.get(2);
                start = 3;
            }
        }

        for (int index = start; index < args.size(); ++index) {
            conditions.add((Node) args.get(index));
        }
    }

    public Class getType() {
        return select.getType();
    }

    @Override
    public void process() {
        if (processed)
            return;
        this.setRoot(this);

        from.setRoot(getRoot());
        from.process();

        if (where != null) {
            where.setRoot(getRoot());
            where.process();
        }
        for (Node condition : conditions) {
            condition.setRoot(getRoot());
            condition.process();
        }

        select.setRoot(getRoot());
        select.process();

        from.process();

        super.process();
    }

    @Override
    public String toSQLString() {
        StringBuilder selectQuery = new StringBuilder(1024);

        selectQuery.append(select.toSQLString());
        selectQuery.append(" ");
        selectQuery.append(from.toSQLString());
        selectQuery.append(" ");
        if (where != null) {
            selectQuery.append(where.toSQLString());
            selectQuery.append(" ");
        }
        for (Node condition : conditions) {
            selectQuery.append(condition.toSQLString());
            selectQuery.append(" ");
        }

        return selectQuery.toString();
    }

    public SelectNode getSelect() {
        return select;
    }

}
