package org.ow2.xplus.setting;

import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.io.InputStream;
import java.util.Properties;

import javax.xml.parsers.ParserConfigurationException;
import javax.xml.parsers.SAXParser;
import javax.xml.parsers.SAXParserFactory;

import org.xml.sax.SAXException;

import org.ow2.xplus.logger.LogFactory;
import org.ow2.xplus.logger.Logger;

public class CacheSetting {
    private static String CONFIG_FILE_NAME = "cache.conf.xml";
    
    private Properties properties = new Properties();

    private static Logger logger = LogFactory.getLogger(CacheSetting.class);
    
    private static String useCache = "usecache";
    
    private static String cacheConcurrencyStrategy = "cacheconcurrencystrategy";
    
    private static String cacheType ="cachetype";
    
    public CacheSetting(CacheConfigInfo info){
    	try {
			readProperties();
			setProperties(info);
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
    }
    
    private void readProperties() throws IOException {
        String confPath = System.getProperty("ConfigHome");
        if (confPath == null) {
            confPath = new File(".").getCanonicalPath();
        }
        File confFile = new File(confPath, CONFIG_FILE_NAME);

        if (!confFile.exists()) {
            logger.debug("error in finding configure file in path: " + confPath);
            return;
        }
        InputStream is = new FileInputStream(confFile);

        ConfigParser handler = new ConfigParser();
        SAXParserFactory factory = SAXParserFactory.newInstance();
        factory.setNamespaceAware(false);
        factory.setValidating(false);

        SAXParser parser = null;
        try {
            parser = factory.newSAXParser();
        } catch (ParserConfigurationException e) {
            e.printStackTrace();
        } catch (SAXException e) {
            e.printStackTrace();
        }

        if (parser == null)
            return;

        try {
            parser.parse(is, handler);
            properties = handler.getProps();
        } catch (SAXException e) {
            e.printStackTrace();
        } catch (IOException e) {
            e.printStackTrace();
        } finally {
            factory = null;
            parser = null;
            handler = null;
        }

        is.close();

    }
    
    public void setProperties(CacheConfigInfo info){
    	if(((String)(properties.get(useCache))).equalsIgnoreCase("true"))
    		info.setUseCache(true);
    	else 
    		info.setUseCache(false);
    	info.setCacheConcurrencyStrategy((String)properties.getProperty(cacheConcurrencyStrategy));
    	info.setCacheType((String)properties.getProperty(cacheType));
    	
    }
}
