/* -------------------------------------------------------------------------*
 * PKUAS: Peking University Application Server
 * Copyright (C) 2006 Peking University
 *
 * This software is free; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * either version 2.1 or any later version.
 * 
 * This software is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this software; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 * ------------------------------------------------------------------------*/

package org.ow2.xplus.transaction;

import javax.persistence.EntityTransaction;
import javax.resource.spi.IllegalStateException;
import javax.transaction.InvalidTransactionException;
import javax.transaction.NotSupportedException;
import javax.transaction.RollbackException;
import javax.transaction.SystemException;
import javax.transaction.Transaction;
import javax.transaction.TransactionManager;

import pku.as.cache.impl.Timestamper;
import org.ow2.xplus.broker.PersistenceManager;

/**
 * @author Lin Liang
 */

public class PersistenceTransaction implements EntityTransaction {

    private TransactionManager tm;

    private boolean rollbackOnly = false;

    private boolean begun = false;

    private Transaction originTx = null;
    
    PersistenceManager pm ;
    
    private long timeStamper;

    public PersistenceTransaction(TransactionManager tm, PersistenceManager pm) {
        this.tm = tm;
        this.pm = pm;
        timeStamper = Timestamper.next();
    }

    public void begin() {
        if (begun)
            return;
        begun = true;
        try {
            tm.begin();
        } catch (NotSupportedException nse) {
            try {
                originTx = tm.suspend();
                tm.begin();
            } catch (Exception e) {
                e.printStackTrace();
            }
        } catch (SystemException se) {
            se.printStackTrace();
        }
    }

    public void commit() {
        if (!begun)
            return;
        Transaction tx = null;
        try {
            tx = tm.getTransaction();
        } catch (SystemException e) {
            e.printStackTrace();
        }
        if (tx != null) {
            try {
                if (rollbackOnly)
                    tx.setRollbackOnly();
                else
                {
                pm.flush();
                }
                tx.commit();
            } catch (RollbackException re) {
                throw new javax.persistence.RollbackException(re);
            } catch (Exception e) {
                e.printStackTrace();
            }
        }
        begun = false;
        if (originTx != null)
            try {
                tm.resume(originTx);
            } catch (InvalidTransactionException e) {
                e.printStackTrace();
            } catch (java.lang.IllegalStateException e) {
                e.printStackTrace();
            } catch (SystemException e) {
                e.printStackTrace();
            }
    }

    public void rollback() {
        if (!begun)
            return;
        Transaction tx = null;
        try {
            tx = tm.getTransaction();
        } catch (SystemException e) {
            e.printStackTrace();
        }
        if (tx != null) {
            try {
            	pm.getContext().clear();
                tx.rollback(); 
            } catch (Exception e) {
                e.printStackTrace();
            }
        }
        begun = false;
        if (originTx != null)
            try {
                tm.resume(originTx);
            } catch (InvalidTransactionException e) {
                e.printStackTrace();
            } catch (java.lang.IllegalStateException e) {
                e.printStackTrace();
            } catch (SystemException e) {
                e.printStackTrace();
            }
    }

    public boolean isActive() {
        return begun;
    }

    /**
     * Mark the current transaction so that the only possible outcome of the
     * transaction is for the transaction to be rolled back.
     * @throws IllegalStateException if isActive() is false.
     */
    public void setRollbackOnly() {
        //if (isActive()) throw new IllegalStateException();
        rollbackOnly = true;

        if (!begun)
            return;
        Transaction tx = null;
        try {
            tx = tm.getTransaction();
        } catch (SystemException e) {
            e.printStackTrace();
        }
        if (tx != null) {
            try {
                tx.setRollbackOnly();
            } catch (Exception e) {
                e.printStackTrace();
            }
        }
    }

    /**
     * Determine whether the current transaction has been marked
     * for rollback.
     * @throws IllegalStateException if isActive() is false.
     */
    public boolean getRollbackOnly() {
        //if (isActive()) throw new IllegalStateException();
        return rollbackOnly;
    }

	public long getTimeStamper() {
		return timeStamper;
	}

	public void setTimeStamper(long timeStamper) {
		this.timeStamper = timeStamper;
	}

}
