/* -------------------------------------------------------------------------*
 * PKUAS: Peking University Application Server
 * Copyright (C) 2006 Peking University
 *
 * This software is free; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * either version 2.1 or any later version.
 * 
 * This software is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this software; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 * ------------------------------------------------------------------------*/

package org.ow2.xplus.util;

import java.lang.reflect.Constructor;
import java.lang.reflect.Method;
import java.lang.reflect.Modifier;
import java.util.ArrayList;
import java.util.Collection;
import java.util.HashMap;
import java.util.HashSet;
import java.util.List;
import java.util.Map;
import java.util.Set;

/**
 * @author Lin Liang
 */

public class ClassStrategy {

    private static Object[][] primitiveMap = new Object[][] { new Object[] { "boolean", Boolean.TYPE },
            new Object[] { "byte", Byte.TYPE }, new Object[] { "char", Character.TYPE }, new Object[] { "double", Double.TYPE },
            new Object[] { "float", Float.TYPE }, new Object[] { "int", Integer.TYPE }, new Object[] { "long", Long.TYPE },
            new Object[] { "short", Short.TYPE } };

    private static Class[] primitiveClass = { Boolean.TYPE, Byte.TYPE, Character.TYPE, Double.TYPE, Float.TYPE, Integer.TYPE,
            Long.TYPE, Short.TYPE };

    private static Class[] primitiveWrapperClass = { Boolean.class, Byte.class, Character.class, Double.class, Float.class,
            Integer.class, Long.class, Short.class };

    private static Class[] ClobType = { char[].class, String.class, Character[].class, java.sql.Clob.class };

    private static Class[] BlobType = { byte[].class, Byte[].class, java.sql.Blob.class };

    private static Object[][] sqlMap = new Object[][] { new Object[] { Boolean.class, "BIT" },
            new Object[] { Boolean.TYPE, "BIT" }, new Object[] { Byte.class, "TINYINT" }, new Object[] { Byte.TYPE, "TINYINT" },
            new Object[] { Character.class, "CHAR" }, new Object[] { Character.TYPE, "CHAR" },
            new Object[] { Double.class, "DOUBLE" }, new Object[] { Double.TYPE, "DOUBLE" },
            new Object[] { Float.class, "REAL" }, new Object[] { Float.TYPE, "REAL" }, new Object[] { Integer.class, "INTEGER" },
            new Object[] { Integer.TYPE, "INTEGER" }, new Object[] { Long.class, "BIGINT" },
            new Object[] { Long.TYPE, "BIGINT" }, new Object[] { Short.class, "SMALLINT" },
            new Object[] { Short.TYPE, "SMALLINT" }, new Object[] { String.class, "VARCHAR" },
            new Object[] { java.math.BigDecimal.class, "DECIMAL" }, new Object[] { java.net.URL.class, "DATALINK" },
            new Object[] { java.util.Date.class, "DATE" }, new Object[] { java.sql.Date.class, "DATE" },
            new Object[] { java.sql.Time.class, "TIME" }, new Object[] { java.sql.Timestamp.class, "TIMESTAMP" } };

    /*
     * private static Class[] basicTypes = { Boolean.TYPE, Byte.TYPE,
     * Character.TYPE, Double.TYPE, Float.TYPE, Integer.TYPE, Long.TYPE,
     * Short.TYPE, Boolean.class, Byte.class, Character.class, Double.class,
     * Float.class, Integer.class, Long.class, Short.class, String.class };
     * private static Class[] charTypes = { Byte.TYPE, Character.TYPE,
     * Byte.class, Character.class };
     */
    public static Class<?> forName(String className) {

        for (int i = 0; i < primitiveMap.length; i++)
            if (className.equals(primitiveMap[i][0]))
                return (Class) primitiveMap[i][1];

        Class clazz = void.class;
        try {
            clazz = Class.forName(className, true, Thread.currentThread().getContextClassLoader());
        } catch (ClassNotFoundException e) {
            // TODO Auto-generated catch block
            e.printStackTrace();
        }
        return clazz;

    }

    public static boolean hasInterface(Class clazz, Class interf) {
        if (clazz == interf) {
            return true;
        }
        Class[] interfaces = clazz.getInterfaces();

        for (Class inf : interfaces) {
            if (inf.equals(interf) || hasInterface(inf, interf)) {
                return true;
            }
        }

        Class parent = clazz.getSuperclass();
        if (parent == null) {
            return false;
        }
        return hasInterface(parent, interf);
    }

    public static boolean isPrimitiveClass(Class clazz) {
        for (Class primitiveClazz : primitiveClass) {
            if (primitiveClazz.equals(clazz)) {
                return true;
            }
        }
        return false;
    }

    public static boolean isPrimitiveWrapperClass(Class clazz) {
        for (Class primitiveWrapperClazz : primitiveWrapperClass) {
            if (primitiveWrapperClazz.equals(clazz)) {
                return true;
            }
        }
        return false;
    }

    public static Class getPrimitiveClass(Class clazz) {
        if (!isPrimitiveWrapperClass(clazz))
            return null;
        for (int index = 0; index < primitiveWrapperClass.length; ++index) {
            if (primitiveWrapperClass[index].equals(clazz)) {
                return primitiveClass[index];
            }
        }
        return null;
    }

    public static boolean isCLobType(Class clazz) {
        for (Class clobType : ClobType) {
            if (clobType.equals(clazz)) {
                return true;
            }
        }
        return false;
    }

    public static boolean isBLobType(Class clazz) {
        for (Class blobType : BlobType) {
            if (blobType.equals(clazz)) {
                return true;
            }
        }
        if (hasInterface(clazz, java.io.Serializable.class))
            return true;

        return false;
    }

    public static boolean isLobType(Class clazz) {
        return (isBLobType(clazz) || isCLobType(clazz));
    }

    public static boolean isSerializedType(Class clazz) {
        return (isPrimitiveWrapperClass(clazz) || isPrimitiveClass(clazz) || isBLobType(clazz) || isCLobType(clazz));
    }

    public static String getSqlType(Class javaType) {
        for (int i = 0; i < sqlMap.length; i++)
            if (javaType.equals(sqlMap[i][0]))
                return (String) sqlMap[i][1];

        if (isBLobType(javaType))
            return "BLOB";
        if (isCLobType(javaType))
            return "CLOB";

        return "OTHER";
    }

    @SuppressWarnings("unchecked")
    public static Constructor getConstructor(Class clazz, Class[] types) {
        final Constructor[] candidates = clazz.getConstructors();
        for (int i = 0; i < candidates.length; i++) {
            final Constructor constructor = candidates[i];
            final Class[] params = constructor.getParameterTypes();
            if (params.length == types.length) {
                boolean found = true;
                for (int j = 0; j < params.length; j++) {
                    final boolean match = params[j].isAssignableFrom(types[j])
                            || (isPrimitiveWrapperClass(types[j]) && params[j].equals(getPrimitiveClass(types[j])));
                    if (!match) {
                        found = false;
                        break;
                    }
                }
                if (found) {
                    if (!Modifier.isPublic(clazz.getModifiers()))
                        constructor.setAccessible(true);
                    return constructor;
                }
            }
        }
        return null;
    }

    public static Object newInstance(Class type) throws InstantiationException, IllegalAccessException {
        if (type.equals(Collection.class))
            return new ArrayList();
        if (type.equals(List.class))
            return new ArrayList();
        if (type.equals(Map.class))
            return new HashMap();
        if (type.equals(Set.class))
            return new HashSet();
        return type.newInstance();

    }

    public static String toName(Class javaType) {
        String typeName = null;

        for (int i = 0; i < primitiveMap.length; i++)
            if (javaType.equals(primitiveMap[i][1]))
                typeName = (String) primitiveMap[i][0];

        if (typeName == null) {
            typeName = javaType.getName();
            int loc = typeName.lastIndexOf(".");
            typeName = (loc < 0) ? typeName : typeName.substring(typeName.lastIndexOf(".") + 1);
        }

        StringBuffer type = new StringBuffer();
        type.append(typeName.substring(0, 1).toUpperCase());
        type.append(typeName.substring(1));

        return type.toString();
    }

    public static List<Method> getSetMethod(Class clazz) {
        Method[] methods = clazz.getMethods();
        List<Method> setMethods = new ArrayList<Method>();
        for (Method method : methods) {
            if (method.getName().startsWith("set")) {
                setMethods.add(method);
            }
        }
        return setMethods;
    }

    public static Method getGetMethodFromSet(Class clazz, Method setMethod) {
        String getMethodName = setMethod.getName().replace("set", "get");
        Method getMethod = null;
        try {
            getMethod = clazz.getMethod(getMethodName, new Class[] {});
        } catch (Exception e) {
            e.printStackTrace();
        }
        return getMethod;
    }

}
