/* -------------------------------------------------------------------------*
 * PKUAS: Peking University Application Server
 * Copyright (C) 2006 Peking University
 *
 * This software is free; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * either version 2.1 or any later version.
 * 
 * This software is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this software; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 * ------------------------------------------------------------------------*/

package org.ow2.xplus.util;

/**
 * @author Lin Liang
 */

public class NamingStrategy {
    /**
     * unqualify, e.g. org.ow2.xplus -> persistence
     * @param qualifiedName the qualified name
     * @return the unqualified name
     */
    public static String unqualify(String qualifiedName) {
        int loc = qualifiedName.lastIndexOf(".");
        return (loc < 0) ? qualifiedName : qualifiedName.substring(qualifiedName.lastIndexOf(".") + 1);
    }

    /**
     * the name of the referencing relationship property or field of the
     * referencing entity; "_"; the name of the referenced primary key column.
     * @param referencingPropertyName the name of the referencing relationship
     *        property or field
     * @param referencedColumnName the name of the referenced primary key
     *        column.
     * @return join column name
     */
    public static String getJoinColumnName(String referencingPropertyName, String referencedColumnName) {
        StringBuffer joinColumnName = new StringBuffer();
        joinColumnName = joinColumnName.append(unqualify(referencingPropertyName)).append("_").append(
                unqualify(referencedColumnName));
        return joinColumnName.toString();
    }

    /**
     * The concatenated names of the two associated primary entity tables,
     * separated by an underscore.
     * @param referencingTableName the name of referencing Table
     * @param referencedEntityTableName the name of referenced Entity Table
     * @return the name of JoinTable
     */
    public static String getJoinTableName(String referencingTableName, String referencedEntityTableName) {
        StringBuffer joinTableName = new StringBuffer();
        joinTableName = joinTableName.append(unqualify(referencingTableName)).append("_").append(
                unqualify(referencedEntityTableName));
        return joinTableName.toString();
    }

    /**
     * the full name of the table, catalog.schema.name
     * @param catalog The catalog of the table.
     * @param schema The schema of the table.
     * @param name The name of the table.
     * @return table name
     */
    public static String getFullTableName(String catalog, String schema, String name) {
        StringBuffer tableFullNameBuffer = new StringBuffer();
        if (!catalog.equals("")) {
            tableFullNameBuffer.append(catalog).append('.');
        }
        if (!schema.equals("")) {
            tableFullNameBuffer.append(schema).append('.');
        }
        String tableFullName = tableFullNameBuffer.append(name).toString();
        return tableFullName;
    }

    /**
     * the name of collection table, entityName + "_" + propertyName
     * @param entityName the name of entity
     * @param propertyName the name of property
     * @return the name of collection table
     */
    public static String getCollectionTableName(String entityName, String propertyName) {
        StringBuffer collectionTableName = new StringBuffer();
        collectionTableName = collectionTableName.append(unqualify(entityName)).append("_").append(unqualify(propertyName));
        return collectionTableName.toString();
    }

    /**
     * get Field Access Method
     * @param fieldName the name of field
     * @param access the type name of access, {GET, SET, IS}
     * @return the name of field access method
     */
    public static String getFieldAccessMethod(String fieldName, String access) {
        StringBuffer methodName = new StringBuffer();
        methodName.append(access.toLowerCase());
        methodName.append(fieldName.substring(0, 1).toUpperCase());
        methodName.append(fieldName.substring(1));
        return methodName.toString();
    }

    /**
     * get Canonical Field Name
     * @param propertyName the name of property
     * @param embeddedColumnFieldName the name of embeddedColumn field
     * @return Canonical Field Name
     */
    public static String getCanonicalFieldName(String propertyName, String embeddedColumnFieldName) {
        StringBuffer canonicalFieldName = new StringBuffer();
        canonicalFieldName.append(propertyName);
        canonicalFieldName.append(".");
        canonicalFieldName.append(embeddedColumnFieldName);
        return canonicalFieldName.toString();
    }

    /**
     * the name of the property or field of the referencing entity; "_"; the
     * name of the column to be embedded.
     * @param propertyName the name of the property or field
     * @param embeddedColumnName the name of the column to be embedded.
     * @return the name of embedded column
     */
    public static String getEmbeddedColumnName(String propertyName, String embeddedColumnName) {
        StringBuffer joinColumnName = new StringBuffer();
        joinColumnName = joinColumnName.append(unqualify(propertyName)).append("_").append(unqualify(embeddedColumnName));
        return joinColumnName.toString();
    }

}
